bool antiDownyMildewUnderPressureClosingPredicate_Body(EntityMethod* pM)
//------------------------------------------------------------------
// renvoie vrai ssi il y a lieu d'arreter la serie de traitement anti-mildiou en presence de pression
//------------------------------------------------------------------
// Selon une option propre au manager, la condition premiere necessaire
// pour arreter les traitements anti-mildiou est :
// SOIT - un stade pheno est atteint pour la Late Variety
// SOIT - le risque mildiou est faible
// SOIT - les traitements anti-oidium sont arretes
//
{
  Entity* pIteration = pM->DescribedEntity();
  Vineyard* pEA = (Vineyard*)App_GetVineyard((Activity*)pIteration);

  bool result = false;

  int option = App_GetIntDecisionOption(pEA, 
										OPT_DOWNY_MILDEW_UNDER_PRESSURE_CLOSING);

  switch (option) {
	//
	// les 'case' stade
	//
  case 0 : {             // FRUIT_SET 
	result = App_IsStageReachedForLateGuidingVariety(pEA, 
													 "fruitSet");
	break;
  }
  case 1 : {             // PEA_SIZED  
	result = App_IsStageReachedForLateGuidingVariety(pEA, 
													 "berriesPeaSized");
	break;
  }
  case 2 : {             // BERRY-TOUCH 
	result = App_IsStageReachedForLateGuidingVariety(pEA, 
													 "beginningOfBerryTouch");
	break;
  }
  case 3 : {             // EARLY_RIPENING 
	result = App_IsStageReachedForLateGuidingVariety(pEA, 
													 "beginningOfBerryRipening");
	break;
  }
  case 4 : {             // END_RIPENING 
	result = App_IsStageReachedForLateGuidingVariety(pEA, 
													 "endOfBerryRipening");
	break;
  }
	//
	// le 'case' risque mildiou (ssi au moins floraison atteinte)
	//
  case 5 : {
	bool floraisonAtteinte =  
	  App_IsStageReachedForLateGuidingVariety(pEA, "lateFlowering");
	if(floraisonAtteinte) {
	  float downyMildewRiskLevel = 
		App_GetFloatPestInfo(pEA, DOWNY_MILDEW_RISK_LEVEL);
	  float threshold = GetRealParameterValue("virtualManager", "riskLevelForDownyMildewClosing");
	  if((downyMildewRiskLevel >= -5)  // -infini par defaut (App_GetFloatPestInfo)
		 &&
		 (downyMildewRiskLevel < threshold)) {
		result = TRUE;
	  }
	}
	break;
  }
	//
	// le 'case' anti-oidium arrete
	//
  case 6 : {
	result = pIteration->GetIntConstValue(POWDERY_MILDEW_TO_CLOSE);
	break;
  }
  }
  return result;
};
