float returnPassedRowsFrequency_Body(EntityMethod* pM)
//----------------------------------------------------
// Renvoie la fr�quence de passage dans les inter-rangs en fonction de l'operation
//----------------------------------------------------
{
  Entity* pPlot = pM->DescribedEntity();
  Operation* pOt = (Operation*)pM->GetEntityArgValue(OPERATION_ARGUMENT);
  UserClassId operationClassId = pOt->ClassId();
  float result = (float)0.5;     // default value

  float q_plot = 
	pPlot->GetFloatConstValue(RIDING_ORIENTED_INTER_ROW_PROPORTION);

  switch (operationClassId) {
  case CLIPPING_OPERATION :
  case CHEMICAL_DEBRANCHER_OPERATED_SPRAYING : {
	result = (float)1.0;
	break;
  }
  case FERTILIZATION_OPERATION : {
	// default value
	break;
  }
  case HAND_HELD_GUN_OPERATED_SPRAYING : 
  case HORIZONTAL_RAIL_OPERATED_SPRAYING : 
  case VERTICAL_RAIL_OPERATED_SPRAYING : {
	// default value
	break;
  }
  case GRINDER_OPERATED_SHOOT_THINNING : {
	// default value
	break;
  }
  case ROTARY_OPERATED_SHOOT_THINNING : {
	result = (float)1. - q_plot;
	break;
  }
  case AIR_ASSISTED_SPRAYING : {
	Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
	//
	// determination / position dans l'iteration de traitements
	//
	Vineyard* pEA = App_GetVineyard((Activity*)pInst);
	Entity* pIterate = (Entity*)pInst->GetFirstContainer(ACTIVITY_ITERATION,
														 FALSE);
	if(pIterate) {  // + JPR 121113
	  int iterationIndex = pIterate->GetIntVarValue(REPLICATION_COUNTER); // 0, 1, etc...
	  int indexMaxForBeginning =        // 1, 2, etc...
		App_GetIntDecisionOption(pEA, OPT_SPRAY_NUMBER_AT_BEGINNING);
	  if(iterationIndex < indexMaxForBeginning)
		result = App_GetFloatDecisionOption(pEA, OPT_SPRAY_PATH_AT_BEGINNING);
	  else {
		//
		// determination / stage vegetatif avant/apres floraison
		//
		bool isFloweringReached =
		  App_IsStageReachedForEarlyGuidingVariety(pEA,
												   "beginningOfFlowering");
		if(isFloweringReached)
		  result = App_GetFloatDecisionOption(pEA, 
											  OPT_SPRAY_PATH_SINCE_FLOWERING);
		else {
		  //
		  // cas residuel : apres 'beginnng' et avant floraison
		  //
		  result = App_GetFloatDecisionOption(pEA, OPT_SPRAY_PATH_REGULAR);
		}
	  }
	}
	else {  // chantier concurrentAntiMildewTreatments (non itéré)
	  // !!! choix arbitraire JPR 121113
	  result = App_GetFloatDecisionOption(pEA, OPT_SPRAY_PATH_REGULAR);
	}
	break;
  }
  default : {
	if(pOt->IsInstanceOf(TILLAGE_OPERATION)) {
	  Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
	  UserClassIdTab actParentIds = pInst->AncestorIds();
	  if(GetPosition(actParentIds, SPRING_TILLAGE_ACTIVITY) >= 0) {
		Entity* pPassedRow =  pPlot->GetElement(0); // par convention
		int management = pPassedRow->GetIntConstValue(SOIL_MANAGEMENT_MODALITY);
//printf("\n\tSPRING_TILLAGE_ACTIVITY pInst : '%s' management = %s", pInst->InstanceName(), App_EnumToChar(management));
		if(management == MIXED_TILLAGE_CW)
		  result = (float)1.0;
		else
		  result = (float)1. - q_plot;
	  }
	  else {
//printf("\n\tNOT SPRING_TILLAGE_ACTIVITY pInst : '%s'", pInst->InstanceName());
		result = (float)1. - q_plot;
		
	  }
	}
	else {
	  printf("\n!!! Tentative d'evaluation de la frequence de passage dans les inter-rangs\n    pour un type d'operation non pris en charge : '%s'.\n", ClassIdToCharTab[operationClassId]);
	exit(0);
	}
  }
  }
//printf("\n'%s'(q = %3.2f)::returnPassedRowsFrequency(%s) returns %3.2f", pPlot->InstanceName(), q_plot, ClassIdToCharTab[operationClassId], result);
  return result;
};
