// --------------------------------------------
// Initialisation de la simulation non autonome
// --------------------------------------------
void InitOpenSimulation(char* paramFile, char* simFile, char* osFile, char* structFile) {
  //InitChrono();
  //StartChrono()
  if(gTraceActionMode)
	printf("\nDEBUT UserFunction.cc/InitOpenSimulation()");

  // ---------------------------------------------
  // Plus grande tolerance pour l'egalite a 1 du degre de progression
  // des operations : -> 0.999 considere egal a 1
  // --------------------------------------------- 
  gFloatingPointEqualityTolerance = (float)0.001;
  //
  // Instantiation de la simulation DIESE
  //
  pCurrentSim = new OpenSimulation();
  pCurrentSim->AssignBeforeInAgendaMethod(BeforeByDatePriority);
  AssignEnumToInt(App_EnumToInt);
  //
  // Installation standard de la hierarchie des classes
  //
  App_AssignInstantiators();

  D_InstallMetaClasses();
  C_InstallMetaClasses();
  O_InstallMetaClasses();
  App_InstallMetaClasses();
  SetHierarchy();
  //
  // Installation des moniteurs
  //
  D_DefineMonitors();
  C_DefineMonitors();
  O_DefineMonitors();
  App_DefineMonitors();

  // -----------------------------------------------------------------------
  // Debut du code modifiable par le developpeur. Ne rien modifier ci-dessus
  // -----------------------------------------------------------------------

  // ---------------------------------------------
  // Lecture des parametres generaux du systeme
  // ---------------------------------------------
  ParseParameterFile(paramFile);
  //
  // Transformation des parametres du systeme en variables globales
  //
  Assign_InitParameterGlobalVariablesFunction(InitParameterGlobalVariables);
  pInitParameterGlobalVariables();
	
  // ---------------------------------------------
  // Lecture des parametres de la simulation
  // ---------------------------------------------
  ParseSimulationFile(simFile);

  // ---------------------------------------------
  // Instanciation superficielle du systeme de production
  // ---------------------------------------------
  Entity* pPS = new VineProducingRegion();
	
  Entity* pDS = new VirtualRegionManager();
					  
  Entity* pStrategy = new Strategy();
  Entity* pARBSet = new ActivitiesResourcesBlockSet();
  pStrategy->AddComponent(pARBSet);
  pDS->AddComponent(pStrategy);
  pPS->AddComponent(pDS);
  Entity* pOS = new VirtualRegionOperatingSystem();
  pPS->AddComponent(pOS);
  Entity* pCS = new VineGrowingTerritory();
  pPS->AddComponent(pCS);
  // ---------------------------------------------
  // Instanciation superficielle du systeme biophysique ...
  // ---------------------------------------------
  Entity* pBPS = new AgroClimaticSystem();
	
  Entity* pAgroSet = new VineyardSet();
  pBPS->AddComponent(pAgroSet);
  Entity* pClimSet = new WeatherStationSet();
  pBPS->AddComponent(pClimSet);
  // ... et attachement au ControlledSystem
  pCS->SetEntityConstValue(ATTACHED_ENTITY, pBPS);

  // ---------------------------------------------
  // Attachement des contraintes sur les ressources a l'OS
  // ---------------------------------------------
  Entity* pRSVC;
  pRSVC= new EquipmentMultiLocalization();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  pRSVC = new FarmWorkerMultiLocalization();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  pRSVC = new FarmWorkerMultiOccupation();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  
  Entity* pARIC = new PlotWithSeveralOperations();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pARIC);
  
  Entity* pAIC;
  pAIC = new TooLateInWorkDayForActivityBeginning();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pAIC);
  
  pAIC = new OnlyPhytoTreatmentsAtEarlyMorning();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pAIC);

  // ---------------------------------------------
  // Designation du systeme de production comme entite simulee
  // ---------------------------------------------
  pCurrentSim->SimulatedEntity(pPS);

  // ---------------------------------------------
  // exploitation des parametres de memoire et de prediction
  // ---------------------------------------------
  int memoryScope = gIntParam_virtualManager_memoryScope_;
  pDS->SetIntVarValue(MEMORY_SCOPE, memoryScope);
  int predictionScope = gIntParam_virtualManager_predictionScope_;
  pDS->SetIntVarValue(PREDICTION_SCOPE, predictionScope);
	
  // ---------------------------------------------
  // Lecture des autres fichiers-argument de la commande
  // ---------------------------------------------
  ParseOutputSpecFile(osFile);
  ParseStructureFile(structFile);

  // ---------------------------------------------
  // Construction profonde du systeme biophysique
  // ---------------------------------------------
  char* inDir = pCurrentSim->GetInDir();
  char* vineyardFile = gStringParam_install_vineyardFile_;
  char str[200] = {'\0'};
  strcat(str, inDir);
  strcat(str, vineyardFile);
  sprintf(str, "%s%s", inDir, vineyardFile);
  App_DepthBioPhysicalSettings(str);

  // ---------------------------------------------
  // Construction profonde du systeme operant
  // ---------------------------------------------
  char* rscFile = gStringParam_install_resourceFile_;
  char str2[200] = {'\0'};
  strcat(str2, inDir);
  strcat(str2, rscFile);
  sprintf(str2, "%s%s", inDir, rscFile);
  App_ResourceSettings(str2);

  // ---------------------------------------------
  // Construction profonde du systeme de d�cision
  // ---------------------------------------------
  char* planFile = gStringParam_install_planFile_;
  char str3[200] = {'\0'};
  strcat(str3, inDir);
  strcat(str3, rscFile);
  sprintf(str3, "%s%s", inDir, planFile);
  App_ControlPlanSettings(str3);

  // ---------------------------------------------
  // Etablissement des IsoAgeingEntity attachees aux parcelles,
  // une fois connues les varietes par parcelle (App_DepthBioPhysical.)
  // et les varietes-guides par conduite (App_ControlPlan.)
  // ---------------------------------------------
  App_AssignIsoAgeingEntityToPlots(pAgroSet);

  // ---------------------------------------------
  // Etablissement des operations exclues des SelectPlotsToX
  // ---------------------------------------------
  App_AssignTurnedOffActivitiesOnPlots(pAgroSet);	

  // -----------------------------------------------------------------------
  // Fin du code modifiable par le developpeur. Ne rien modifier ci-dessous
  // -----------------------------------------------------------------------

  if(gTraceActionMode)
	printf("\nFIN   UserFunction.cc/InitOpenSimulation()\n");
  //PauseChrono();
  //PrintChrono();
  //printf("\nduree InitOpenSimulation : '%s'\n", gTimeString);
  //CloseChrono(); // RAZ et fermeture chronometre InitOpenSimulation

  //InitChrono();  // re-ouverture du chronometre pour RunOpenSimulation
  // - les StartChrono/PauseChrono sont dans le code Openfluid
  //   . un StartChrono() juste avant l'appel � RunOpenSimulationToDate
  //                   et juste avant l'appel � RunOpenSimulation
  //   . un PauseChrono() juste apres l'appel � RunOpenSimulationToDate
  //                   et juste apres l'appel � RunOpenSimulation
  // - le CloseChrono est dans FinalizeOpenSimulation

}
