void hourlyWeatherFileReading_initializeProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'initialisation 
// du processus continu de lecture d'un fichier sequentiel.
//---------------------------------------------
{
  ReadBufferizedDataFileProcess* pCP = (ReadBufferizedDataFileProcess*)(pM->DescribedContinuousProcess());

  UserFileId fileClassId = pCP->FileClassId();
  BufferizedDataFile* pF =
    (BufferizedDataFile*)App_NewSequentialDataFileFromId(fileClassId);
  if(! pF)
    UnknownClassSymbol("BufferizedDataFile", fileClassId);
  pCP->SetFile(pF);

  //-------------------------------------------
  // installation tableau Memoire
  //-------------------------------------------
  int M = pF->BFSetMemoryScope(fileClassId);
  pF->SetMemoryTab(new Tableau<char*>());
  for(int k=0;k<M;k++) {
    pF->AddMemoryTabItem(strdup(""));
  }
  pF->SetMemoryCurrentRecordIndex(-1);

  //-------------------------------------------
  // installation tableau Prediction
  //-------------------------------------------
  int P = pF->BFSetPredictionScope(fileClassId);
  pF->SetPredictionTab(new Tableau<char*>());
  for(int k=0;k<P;k++) {
    pF->AddPredictionTabItem(strdup(""));
  }
  pF->SetPredictionCurrentRecordIndex(-1);

  //-------------------------------------------
  // recuperation des specifications du fichier
  //-------------------------------------------
  pCP->SetNamesFromFile(pF);
  //pCP->DisplayNames();

  //-------------------------------------------
  // etablissement du chemin complet du fichier
  // a partir des specifications du fichier et du processus
  //-------------------------------------------
  pCP->AssignFullNameToFile(pF);

  //-------------------------------------------
  // attachement fichier externe <-> station meteo
  //-------------------------------------------
  Entity* pWeatherStation = pCP->ProcessedEntity();
  char* fullPathName = pF->GetFullName();
//printf("\nhourlyWeatherFileReading_init fullPathName : '%s'\n", fullPathName);
  pWeatherStation->SetStringConstValue(PATH_NAME_ATTRIBUTE, fullPathName);

  // debut ajout JPR 130306 (5.5)
  Date* dateDebutHiver = GetDateParameterValue("debutCumulPluie", "declenchementLabour");
  dateDebutHiver->year = pCurrentSim->BeginningDate()->year;
  tm* tmDebutHiver = StructDateToTm(dateDebutHiver);
  int clockDebutHiver = StructTmToClockValue(tmDebutHiver); 
  pWeatherStation->SetIntConstValue(WINTER_BEGINNING_CLOCK, clockDebutHiver);
  delete tmDebutHiver;

  // debut ajout JPR 130425 (5.6)
  dateDebutHiver = GetDateParameterValue("debutCumulPluie", "declenchementLabourInit");
  dateDebutHiver->year = pCurrentSim->BeginningDate()->year;
  tmDebutHiver = StructDateToTm(dateDebutHiver);
  clockDebutHiver = StructTmToClockValue(tmDebutHiver); 
  pWeatherStation->SetIntConstValue(WINTER_BEGINNING_CLOCK_FOR_INITST, clockDebutHiver);
  delete tmDebutHiver;

  //---------------------
  // ouverture du fichier
  //---------------------
  pF->Open();

  //---------------------
  // saut des premieres lignes d'entete
  //---------------------
  pF->Skip(pF->GetNbTopCommentLines());

  //---------------------
  // positionnement sur le premier jour de la simulation
  //---------------------
  Date* dateDebutSim = pCurrentSim->BeginningDate();
  tm* pTm = StructDateToTm(dateDebutSim);
  //
  // transformation en date UTC, format en vigueur sur pF
  //
  //                                      début modif JPR 130206
  time_t time_begSim = mktime(pTm);
  tm* pTm_utc = gmtime(&time_begSim);
  pTm_utc->tm_hour--;
  Date* dateArretLecture =  StructTmToDate(pTm_utc);
//StructDateToChar(dateArretLecture);  printf("\ndateArretLecture : %s", gTimeString);
  //delete pTm_utc;  // Seg.Flt
  delete pTm;
  //                                      fin modif JPR 130206
  //
  // parcours des lignes et arret juste avant la premiere a considerer
  //
  char str_tmp[100];
  int i;
  char* line;
  Boolean onContinue = TRUE;
  while(onContinue) {
	line = fgets(str_tmp, 90, pF->GetPhysicalFile());
	i = sscanf (str_tmp,
				pF->GetFormat()
				, pF->mpFieldTab[FLD_DATE]->mFieldValue.stringValue
				, pF->mpFieldTab[FLD_HOUR]->mFieldValue.stringValue
                , &pF->mpFieldTab[FLD_RAINFALL]->mFieldValue.floatValue
				);
	if((i != EOF) && (i < pF->mpFieldTab.size()))
	  BadReadFile(pF);
	// ligne : 2004-01-01 19:00:00 0.5
	char* date = pF->GetStringFieldValue(FLD_DATE);
	//         2004-01-01
	char* heure = pF->GetStringFieldValue(FLD_HOUR);
	//                    19:00:00

	Date* dateRef = App_WeatherFileDateToDate(date, heure);
	
	if((*dateRef == *dateArretLecture)
	   ||
	   (line == NULL)) {
	  onContinue = FALSE;
	}
	delete dateRef;
  }
  delete dateArretLecture;

  //---------------------
  // Lecture des P premiers enregistrements simulant la prediction initiale
  //---------------------
  int predictionCurrentIndex = pF->GetPredictionCurrentRecordIndex();
  for(int k=0;k<P;k++) {
    //
    // lecture physique enregistrement sur fichier externe
    //
    char str[101] = "\0";
    int c, i=-1, onContinue = 1;
    while(onContinue) {
      c = fgetc(pF->GetPhysicalFile());
	  if(c == EOF) {
		if(! pF->IsEndOfFileReached()) {
		  if(gMyTraceWarnings)
			printf("\n!! EOF REACHED DURING INITIALIZATION : hourlyWeatherFileReading !!");
		  pF->IsEndOfFileReached(TRUE);
		}
		break;
	  }
	  else {
		if(c != 13) {                 // CR
		  if(c == 10) onContinue = 0; // LF
		  else str[++i] = (char)c;
		}
	  }
    }
    str[++i] = '\0';
    //
    // ... et ajout a la premiere place libre dans le tableau des predictions
    //
    int indexToLoad = ++predictionCurrentIndex;
    pF->SetPredictionTabItem(indexToLoad, strdup(str));

  }

  //---------------------
  // dans les predictions, on pointe toujours sur la plus lointaine
  //---------------------
  pF->SetPredictionCurrentRecordIndex(predictionCurrentIndex);
//pF->DisplayMemoryTab();
//pF->DisplayPredictionTab();
};
