float achievementDegree_whenSetFloat_Body(MonitorMethod* pM)
//---------------------------------------------
// Definition de la fonction qui sera declenchee lors de
// chaque modification de la variable surveillee.
//
// 0) appel du moniteur de la classe de base pour detection franchissement
//    du seuil de suspension de l'operation (cf. ontologie §5511)
//
// 1) trace des progressions journalieres d'execution
//
// 1bis) Ecriture sur le fichier 'base de donnees' du Manager Virtuel
//
// //2) si on termine une operation avant la mi-journee,
// //   on programme un examen intermediaire du plan
//
// //3) lorsqu'on vient de demarrer une operation, et qu'allonger un peu
// //   la journee de travail permettrait de la terminer, on le fait
//
// 4) quand une operation est terminee, on augmente la memoire des actions
//    sur l'entite appropriee (Row)
//    --> . creation d'un Last<X>MemoryItem, valuation de ses descripteurs
//        . ajout comme element du LastOperationsMemoryAttribute de l'entite 
//
// 4bis) quand une operation de Tillage est terminee, 
//    - on met à 0 le cumul de pluie, pour usage dans le pred. d'ouverture 
//    du Tillage suivant, dans les options appropriees (LOW_INTENSITY, ...)
//    - on regarde si le 'area scope' en vigueur est atteint, pour alors valuer
//    le PreviousReplicationRefDate de l'itération de SpringTillages
//
// 5) quand une operation est terminee, 
//    - on ajoute la parcelle concernee comme element de valeur de 
//      TERMINATED_PLOTS_ATTRIBUTE du chantier
//    - gere le descripteur PERCENT_<OPERATED> approprie sur la parcelle
//
// 5bis) quand une operation est terminee, et que ACTUAL_REALIZATION_END_DATE
//    du chantier est antérieure à l'instant courant, alors on lui donne 
//    la valeur courante de l'horloge (usage pour l'ecriture du fichier de
//    commande pour le 'Gantt chart')
//
// 6) quand une operation commence, et que ACTUAL_REALIZATION_BEG_DATE
//    du chantier n'est pas value, alors on lui donne la valeur courante 
//    de l'horloge (usage par exemple dans le test d'ouverture d'un
//    chantier ExtraPhytoTreatments)
//
// 6bis) quand une operation de Tillage commence, on value le
//    descripteur LastTillageOperationClock (utilisation dans le predicat
//    d'ouverture du chantier itere SpringTillage, actuellement) 
//
// 7) quand une operation progresse, on diminue le montant d'heures
//    restant a travailler dans la journee, la semaine, l'annee
//
// 8) quand une operation progresse, on augmente le cumul de pluie
//    recue par chaque unite de MO impliquée depuis le début de la journee
//
// 9) trace sur fichier des fins d'activités primitives (JPR 130325)
//
//---------------------------------------------
{
  Descriptor* pVD = pM->GetDescriptorArgValue(MONITOREDDESCRIPTOR_ARGUMENT);
  float oldValue = pM->GetFloatArgValue(CURRENTFLOATVALUE_ARGUMENT);
  //
  // invocation moniteur sur classe de base (+ JPR 110316)
  //
  //float newValue = pM->GetFloatArgValue(CANDIDATEFLOATVALUE_ARGUMENT);
  float newValue = operationPercentAchieved_whenSetFloat_Body(pM);
//printf("\nachievementDegree_whenSetFloat newValue = %f\n", newValue);
  //
  // on ne reagit pas s'il s'agit de l'initialisation du degre de progression
  //
  if(newValue < gFloatingPointEqualityTolerance)
    return newValue;

  int now = pCurrentSim->Clock();
  tm* pTm_now = ClockValueToTm(now);

  // --------------------------------------------------------
  // recuperation de la parcelle operee
  // --------------------------------------------------------
  Operation* pOt = (Operation*)pVD->DescribedEntity();
  Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);

  Entity* pV_alloc = pInst->GetEntityConstValue(ALLOCATION_ATTRIBUTE);
  pEntityTab* perfList = ((InstructionAllocation*)pV_alloc)->ExtractPerfSet();

  Entity* pCurrentOO = pOt->GetEntityConstValue(CURRENT_OPERATED_OBJECT);
  Entity* pPlot = NULL;
  if(pCurrentOO)
	pPlot = pCurrentOO->GetEntityConstValue(RESOURCE_ENTITY);
  else {
	printf("\n!!! achievementDegree_whenSetFloat : la parcelle operee n'a pas pu etre designee\n    pour l'operation '%s'\n", pOt->InstanceName());
	exit(0);
  }

  Entity* pChantier = NULL;
  int typeChantierPhyto = -9;
  pChantier = 
	(Entity*)pInst->GetFirstContainer(CONCURRENT_DISTRIBUTED_ACTIVITY_SET,
									  FALSE);
  if(pChantier == NULL) {
	pChantier = 
	  (Entity*)pInst->GetFirstContainer(SERIAL_DISTRIBUTED_ACTIVITY_SET,
										FALSE);
  }
  //else  - JPR 130227
  typeChantierPhyto = App_GetDistributedActivitySetType(pChantier);

  //
  // categorisation partielle de l'operation  (modif JPR 121113 pour BDD)
  // ---------------------------------------
  UserClassIdTab otParentIds = pOt->AncestorIds();
  otParentIds.push_front(pOt->ClassSymbol());;
  
  bool isTillageOT = (GetPosition(otParentIds, TILLAGE_OPERATION) >= 0);
  bool isInitSpringTillages = FALSE;
  bool isSpringTillages = FALSE;
  if(isTillageOT) {
	//
	// categorisation du chantier
	// -----------------------------
	UserClassIdTab chParentIds = pChantier->AncestorIds();
	chParentIds.push_front(pChantier->ClassSymbol());
	isInitSpringTillages = 
	  GetPosition(chParentIds, CONCURRENT_INIT_SPRING_TILLAGES) >= 0
	  || 
	  GetPosition(chParentIds, SERIAL_INIT_SPRING_TILLAGES) >= 0;
	isSpringTillages = 
	  GetPosition(chParentIds, CONCURRENT_SPRING_TILLAGES) >= 0
	  || 
	  GetPosition(chParentIds, SERIAL_SPRING_TILLAGES) >= 0;
  }
  
  // --------------------------------------------------------
  // tache 1) trace des progressions journalieres d'execution
  // --------------------------------------------------------
  //
  // gMyTraceOperations est vrai ssi on a utilise l'option '-o'
  //
  if(gMyTraceOperations) {
	char* tag = "";
	if(pInst->IsInstanceOf(PHYTO_TREATMENT_ACTIVITY)) {
	  Entity* pIteration = 
		(Entity*)pInst->GetFirstContainer(ACTIVITY_ITERATION, FALSE);
	  if(pIteration) {
		if(pIteration->IsInstanceOf(REGULAR_PHYTO_TREATMENTS_ITERATION)) {
		  tag = "REGULAR";
		}
		else if(pIteration->IsInstanceOf(EXTRA_PHYTO_TREATMENTS_ITERATION)) {
		  tag = "EXTRA";
		}
	  }
	}
	float totalArea = pPlot->GetFloatConstValue(AREA);
	printf("\n[t=%d] %s ", now, ShortWeekDayNameToCharTab[pTm_now->tm_wday]);
	TraceDateFromClock(now, DATE_HDM);
	printf(" '%s' sur '%s' -> %4.3f %s", pInst->InstanceName(), pPlot->InstanceName(), newValue, tag);
	// if(((PrimitiveActivity*)pInst)->IsOptional()) printf(" [optionnelle]");
	printf("\tperfs : ");perfList->display_names();

	if(pInst->IsInstanceOf(PHYTO_TREATMENT_ACTIVITY)) {
	  // etabli dans AddNewSituatedPhytoTreatments
	  TableauString* tmp = pInst->GetStringTabValue(PHYTO_TREATMENT_LIST);
	  printf("\t");tmp->display();
	}
  }
  //
  // gMyRecordOperations est vrai ssi on a utilise l'option '-f <fichier>'
  //
  if(gMyRecordOperations) {
	//
	// alimentation de la memoire des operations du manager 
	// exploitee par DumpOperationRecords en fn de simulation
	//
	Manager* lAgri = (Manager*)GetFirstEntity(MANAGER);
	Entity* pOtRecordingStructure = 
	  lAgri->GetEntityConstValue(OPERATION_RECORDING_ATTRIBUTE);
	int nbRecords = pOtRecordingStructure->GetNumberOfElements();
	//
	// recuperation YMD, plot, operation
	// 	
	int year_now = pTm_now->tm_year;
	int month_now = pTm_now->tm_mon;
	int day_now = pTm_now->tm_mday;
	int yday_now = pTm_now->tm_yday;
	// pPlot deja etablie
	// pInst deja etablie
	// pOt deja etablie 
//printf("\nt = %d yday = %d", now, yday_now);
	//
	// recuperation profils de MO et de materiel, et de produitPhyto
	// 	
	// perfList deja etablie
	pIntTab* profilMO = App_SummarizePerformerPool(perfList);

	pEntityTab* otRscList = 
	  ((InstructionAllocation*)pV_alloc)->ExtractOtRscSet();
	pIntTab* profilTools = App_SummarizeToolSet(otRscList);

	TableauString* phytoTrtList = NULL; // modif JPR 130325
	if(pInst->IsInstanceOf(PHYTO_TREATMENT_ACTIVITY)) {
	  // etabli dans AddNewSituatedPhytoTreatments
	  // copie du tableau parce que les chaines-elements vont etre
	  // desalloues avec le descripteur, lui-meme desalloue avec l'activite
	  phytoTrtList = new TableauString(*(pInst->GetStringTabValue(PHYTO_TREATMENT_LIST)));
	}
	//
	// on remonte d'un jour dans la memoire, a la recherche d'un enregistrement
	// a la meme date, sur la meme parcelle avec la meme activite,
	// et avec les mêmes profils de MO et de materiel, et de produitPhyto
	//
	Entity* pItemToUpdate = NULL;

	for(int k=nbRecords-1;k>=0;k--) {
	  Entity* pRecordItem_k = pOtRecordingStructure->GetElement(k);
	  int yday_k = pRecordItem_k->GetIntConstValue(RECORDED_JULIAN_DAY);
	  if(yday_k == yday_now) {
		char* plot_k = pRecordItem_k->GetStringConstValue(RECORDED_PLOT_NAME);
		char*act_k = pRecordItem_k->GetStringConstValue(RECORDED_ACTIVITY_CLASS_NAME);
		if(! strcmp(plot_k, pPlot->InstanceName()) 
	       && 
	       ! strcmp(act_k, pInst->ClassName())) {
		  // profil MO
		  pEntityTab* perfList_k = pRecordItem_k->GetEntityTabConstValue(RECORDED_PERFORMER_LIST);
		  pIntTab* profilMO_k = App_SummarizePerformerPool(perfList_k);
//printf("\n\tprofilMO_k : ");profilMO_k->display();printf(" vs. profilMO : ");profilMO->display();
		  if(*profilMO_k == *profilMO) {
			// profil Mat
			pEntityTab* otRscList_k = pRecordItem_k->GetEntityTabConstValue(RECORDED_RESOURCE_LIST);
			pIntTab* profilTools_k = App_SummarizeToolSet(otRscList_k);
//printf("\n\tprofilTools_k : ");profilTools_k->display();printf(" vs. profilTools : ");profilTools->display();
			if(*profilTools_k == *profilTools) {
			  // profil produits
			  TableauString* phytoTrtList_k = pRecordItem_k->GetStringTabConstValue(RECORDED_PHYTO_PRODUCT_LIST);
//printf("\n\tphytoTrtList_k : ");phytoTrtList_k->display();printf(" vs. phytoTrtList : ");phytoTrtList->display();
			  bool ok = TRUE;
			  int nbProd_k = phytoTrtList_k->size();
			  for(int i=0;i<nbProd_k;i++) {
				if(phytoTrtList && GetPosition(*phytoTrtList, phytoTrtList_k->get_nth(i)) < 0) { // modif JPR 130325
				  ok = FALSE;
				  break;
				}
			  }
			  if(ok) {
				pItemToUpdate = pRecordItem_k;
				delete profilTools_k; // + JPR 130325
				delete profilMO_k; // + JPR 130325
				break;
			  }
			}
			delete profilTools_k; // + JPR 130325
		  }
		  delete profilMO_k; // + JPR 130325
		}
	  }
	  else
		break;
	}
	delete profilTools; // + JPR 130325
	delete profilMO; // + JPR 130325

	if(pItemToUpdate) {
	  //
	  // s'il y avait un tel enrgt, on augmente seulement le nb d'heures
	  //
	  int oldValue = pItemToUpdate->GetIntConstValue(RECORDED_IN_DAY_DURATION);
	  int newValue = oldValue + pOt->GetStep();
	  pItemToUpdate->SetIntConstValue(RECORDED_IN_DAY_DURATION, newValue);
	}
	else {
	  //
	  // sinon, on cree un nouvel enregistrement, avec l'info complete
	  //
	  // recuperation listes de performers, ressources, produits phyto
	  //
	  // perfList deja etablie

	  Entity* pNewItem = new OperationRecordingItem();
	  Entity* pChantier = NULL; // (et ci-dessous) 120516 4.6
	  pChantier = 
		(Entity*)pInst->GetFirstContainer(CONCURRENT_DISTRIBUTED_ACTIVITY_SET,
										 FALSE);
	  if(! pChantier) {
		pChantier = 
		  (Entity*)pInst->GetFirstContainer(SERIAL_DISTRIBUTED_ACTIVITY_SET,
										   FALSE);
	  }
	  if(! pChantier) {
		printf("\ndumpOperationRecords_Body : chantier non teouvé pour '%s'\n", pInst->InstanceName());
		exit(0);
	  }
	  //
	  //  désignation du tour si SringTillages (JPR 121113
	  //
	  pNewItem->SetIntConstValue(RECORDED_YEAR, year_now);
	  pNewItem->SetIntConstValue(RECORDED_MONTH, month_now);
	  pNewItem->SetIntConstValue(RECORDED_DAY, day_now);
	  pNewItem->SetIntConstValue(RECORDED_JULIAN_DAY, yday_now);
	  pNewItem->SetStringConstValue(RECORDED_PLOT_NAME,
	                                pPlot->InstanceName());
	  if(isSpringTillages) {  // 121113
		Entity* pSTIteration = pChantier->GetSuperSet();
		int numTour = pSTIteration->GetIntVarValue(REPLICATION_COUNTER)+1;
		char str[30] = {'\0'};
		sprintf(str, "%s_%d", pChantier->ClassName(), numTour);
		pNewItem->SetStringConstValue(RECORDED_CHANTIER_CLASS_NAME,
									  str);
	  }
	  else
		pNewItem->SetStringConstValue(RECORDED_CHANTIER_CLASS_NAME,
									pChantier->ClassName()); // 120516
	  pNewItem->SetStringConstValue(RECORDED_ACTIVITY_CLASS_NAME,
	                                pInst->ClassName());
	  pNewItem->SetStringConstValue(RECORDED_OPERATION_CLASS_NAME,
	                                pOt->ClassName());
	  pNewItem->SetEntityTabConstValue(RECORDED_RESOURCE_LIST, 
									   new pEntityTab(*otRscList));
	  pNewItem->SetEntityTabConstValue(RECORDED_PERFORMER_LIST,
									   new pEntityTab(*perfList));
	  if(phytoTrtList) pNewItem->SetStringTabConstValue(RECORDED_PHYTO_PRODUCT_LIST, 
									   phytoTrtList); // modif JPR 130325
	  pNewItem->SetIntConstValue(RECORDED_IN_DAY_DURATION, pOt->GetStep());
//printf("\t **** IN_DAY_DURATION init = %d", pOt->GetStep());
	  //
	  // ... et on l'ajoute a la memoire des operations
	  //
	  pOtRecordingStructure->AddElement(pNewItem);
	}
	delete otRscList; // depl. JPR 130325
  }

  /*
  // --------------------------------------------------------
  // tache 2) on ne cherche a programmer un examen intermediaire du plan
  //          que si l'operation est terminee
  // --------------------------------------------------------
  if(newValue > (float)0.99999) {
	
	if(pCurrentSim->ClockTimeUnit() == HOUR) {
	  //
	  // prerequis : horloge = 1 HOUR
	  //
	  tm* tm_now = ClockValueToTm(now);
	  
	  int next_pm_update;
	  Manager* lAgri = app_getManager((Activity*)pInst);
	  int flagValue = lAgri->GetIntVarValue(INDICATEUR_RECOLTE_EN_COURS);
	  int heureMiJournee;
	  if(flagValue)
		heureMiJournee = gIntParam_revisionDuPlan_heureMiJourneePointe_;
	  else
		heureMiJournee = gIntParam_revisionDuPlan_heureMiJourneeNormale_;
	  next_pm_update = 
		DateToClockValue(tm_now->tm_mday,
						 tm_now->tm_mon+1,
						 tm_now->tm_year+1900,
						 heureMiJournee, 0, 0);
	  delete tm_now;
//printf("\n\tnext_pm_update = %d now+1 = %d", next_pm_update, now+1);
	  //
	  // ... seulement si l'operation s'est finie avant la mi-journee
	  //
	  if(next_pm_update > now) {
		//
		// programmation d'un examen du plan intermediaire
		//
		Event* pEv = new ExamenIntermediaireDuPlan();
		pEv->SetOccurrenceClockTime(next_pm_update);
//printf("\n    event examen intermediaire en %x programmï¿œ en %d end en %d", pEv, next_pm_update, pCurrentSim->EndClock());
		Boolean planned = pCurrentSim->InsertInAgendaEvents(pEv);
		if(! planned) {
		  ((DiscreteProcess*)pEv->FirstProcess())->DeleteInDepth();
		  pEv->DeleteInDepth();
		}
	  }
	  else {
		//
		// rien : on attend la revision matinale du lendemain
		//
	  }
	}
  }
  */

  // --------------------------------------------------------
  // tache 3) allongement de la journee de travail
  // --------------------------------------------------------
  if(oldValue < gFloatingPointEqualityTolerance) {
	//
	// duree de l'operation
	//
	pInst->SetEntityArgValue(OPERATION_NOMINAL_DURATION,
							 CANDIDATEENTITYVALUE_ARGUMENT, pV_alloc);
	float duration = pInst->ExecFloatMethod(OPERATION_NOMINAL_DURATION);
//printf("\n'%s' duration = %f", pOt->InstanceName(), duration);
	int dateFinOperation = now + duration;
	//
	//
	// date de fin de session de travail 
	// 
	int now_hour = pTm_now->tm_hour;
	int nextEndHour = now_hour;
	int supplementAccepte;
	Entity* perfSet = pV_alloc->GetComponent(ALLOCATED_PERFORMER_SET);
	pEntityTab perfList = perfSet->GetElementTab();
	int nbPerfs = perfList.size();
	//int nbPerfs = perfList->size();  // essai 110121
	for(int j=0;j<nbPerfs;j++) {
	  Entity* pPerf = (Entity*)perfList.get_element(j);
	  //Entity* pPerf = (Entity*)perfList->get_element(j);
	  Entity* pEmploymentPlan = 
		pPerf->GetEntityConstValue(ANNUAL_EMPLOYMENT_PLAN_ATTRIBUTE);
	  int seasonIndex = 
		pEmploymentPlan->ExecIntMethod(UPDATE_SEASON_CURRENT_INDEX);
	  Entity* pSeasonalPlan = pEmploymentPlan->GetElement(seasonIndex);
	  float normalMorningEnd = 
		pSeasonalPlan->GetFloatConstValue(WORK_DAY_MORNING_END_HOUR);
	  float normalAfternoonBeg = 
		pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_BEG_HOUR);
	  float normalAfternoonEnd = 
		pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_END_HOUR);
	  
//printf("\n\tnow_hour = %d", now_hour);
	  if(now_hour >= normalAfternoonBeg) {
		nextEndHour = normalAfternoonEnd;
		supplementAccepte = 
		  pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_EXTENDED_END_HOUR);
	  }
	  else {
		if(normalMorningEnd >= nextEndHour) {
		  nextEndHour = normalMorningEnd;
		  supplementAccepte = 
			pSeasonalPlan->GetFloatConstValue(WORK_DAY_MORNING_EXTENDED_END_HOUR);
		}
	  }
//printf("\n\tnextEndHour = %d", nextEndHour);
	
	  int nextImmobilisation = now + TimeIntervalToClockNumber((nextEndHour-now_hour)*3600);
	  //
	  // test : l'OT va se terminer un peu apres la fin de la journee
	  //
	  int delta = TimeIntervalToClockNumber((supplementAccepte-nextEndHour)*3600);
//printf("\n\tdateFinOperation = %d nextImmobilisation = %d supplementAccepte = %d delta = %d", dateFinOperation,  nextImmobilisation, supplementAccepte, delta);
	  if((dateFinOperation > nextImmobilisation)
		 &&
		 (dateFinOperation <= nextImmobilisation + delta)) {
		pSeasonalPlan->SetIntConstValue(CURRENT_SLOT_TO_BE_EXTENDED, TRUE);
	  }
	  else {
		pSeasonalPlan->SetIntConstValue(CURRENT_SLOT_TO_BE_EXTENDED, FALSE);
	  }
	}
  }	

  // --------------------------------------------------------
  // tache 4) creation d'IDI pour interface avec MHYDAS
  // --------------------------------------------------------
  //if(newValue > (1. - D_float_EPS)) {
  if(newValue > (float)0.99999) {

	//
	// categorisation complémentaire de l'operation
	// --------------------------------------------
	bool isSoilTillageOT = FALSE;
	bool isPesticideApplicationOT = FALSE;

	// --------------------------------------------------------
	// tache 4bis) pour un TILLAGE
	// --------------------------------------------------------
	if(isTillageOT) {
	  if(isInitSpringTillages || isSpringTillages) {
		Entity* pSTIteration = pChantier->GetSuperSet();
		if(! isInitSpringTillages) printf("\ttour %d", pSTIteration->GetIntVarValue(REPLICATION_COUNTER)+1); // + JPR 120904 
		//
		// RAZ du cumul de pluie
		//
		pPlot->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_LAST_TILLAGE, (float)0.);
		//
		// on regarde si le 'area scope' en vigueur est atteint
		//
		Entity* pEA = App_GetVineyard((Activity*)pChantier);
		int option = App_GetIntDecisionOption(pEA, OPT_SPRING_TILLAGES_O_P_SCOPE);
		float areaThreshold;
		switch (option) {
		case FARM_SCOPE : {
		  float totalArea = pEA->GetFloatConstValue(AREA_WITH_TILLED_GRAPEVINE);
		  areaThreshold = totalArea * (float)0.75;
		  break;
		}
		case PLOT_SCOPE : {
		  areaThreshold = (float)2.;
		  break;
		}
		case PLOT_GROUP_SCOPE : {
		  areaThreshold = (float)5.;
		  break;
		}
		}
		float operatedArea = pChantier->GetFloatVarValue(OPERATED_VINEYARD_SUB_AREA);
		operatedArea += pPlot->GetFloatConstValue(AREA);
		pChantier->SetFloatVarValue(OPERATED_VINEYARD_SUB_AREA, operatedArea);
		//
		// si oui, now --> RefDate de l'iteration des ST
		//
		if(isInitSpringTillages) {
		  Entity* pPhase = pChantier->GetSuperSet();
		  pSTIteration = (Entity*)pPhase->GetClassNamedElement(SPRING_TILLAGES_ITERATION);
		}
//printf("\n\t '%s' operatedArea = %f areaThreshold = %f", pOt->InstanceName(), operatedArea, areaThreshold);
		if(operatedArea >= areaThreshold) { 
		  // + JPR 120726
		  if(pSTIteration->GetIntConstValue(PREVIOUS_REPLICATION_REF_DATE) < 0) {
			printf("\tREF_DATE  = %d", now);
			pSTIteration->SetIntConstValue(PREVIOUS_REPLICATION_REF_DATE, now);
		  }
		}
	  }
	}

	bool isRotaryThinningOT = (GetPosition(otParentIds, ROTARY_OPERATED_SHOOT_THINNING) >= 0);
	if(isTillageOT || isRotaryThinningOT)
	  isSoilTillageOT = TRUE;
	else {
	  bool isBackSprayingOT = (GetPosition(otParentIds, BACK_SPRAYING_OPERATION) >= 0);
	  bool isMotorSprayingOT = (GetPosition(otParentIds, MOTORIZED_SPRAYING_OPERATION) >= 0);
	  if(isBackSprayingOT || isMotorSprayingOT)
		isPesticideApplicationOT = TRUE;
	}
	TableauString* treatmentList = NULL;
	bool isPhytoTreatmentAct = FALSE;
	bool isChemicalWeedingAct;
	bool isDebranchingAct = FALSE;
	int nbProducts = -9;
	if(isPesticideApplicationOT) {
	  //----------------------
	  //
	  // categorisation de l'activite
	  // -----------------------------
	  //
	  UserClassIdTab actParentIds = pInst->AncestorIds();
	  actParentIds.push_front(pInst->ClassSymbol());
	  isPhytoTreatmentAct = 
		(GetPosition(actParentIds, PHYTO_TREATMENT_ACTIVITY) >= 0);
	  isChemicalWeedingAct = 
		(GetPosition(actParentIds, CHEMICAL_WEEDING_ACTIVITY) >= 0);
	  isDebranchingAct = 
		(GetPosition(actParentIds, DEBRANCHING_ACTIVITY) >= 0);
	  
	  if(isPhytoTreatmentAct) {
		TableauString* tmp = 
		  pInst->GetStringTabConstValue(PHYTO_TREATMENT_LIST);
		treatmentList = new TableauString(*tmp);
	  }
	  else if(isChemicalWeedingAct) {
		treatmentList = new TableauString();
		char* nomChantier = pChantier->ClassName();
		char* nomPlot = pPlot->InstanceName();
	          if((strstr(nomChantier, "Spring"))
	             || (strstr(nomPlot, "MG16"))) {
		treatmentList->push_back(strdup("post_levee"));
	          }
	          else {
		treatmentList->push_back(strdup("pre_post_emergence"));}
	  }
	  else if(isDebranchingAct) {
		treatmentList = new TableauString();
		treatmentList->push_back(strdup("post_levee"));
	  }
//printf("\ntreatmentList : ");treatmentList->display();
	  nbProducts = treatmentList->size();
	}

	//
	// pour chaque rangee, creation IDI adapte a la categorie
	// ------------------------------------------------------
	Entity* pVegetal = pPlot->GetComponent(VEGETAL_COMPARTMENT);
	int nbRows = pVegetal->GetNumberOfElements();
//printf("\n!!! achievementDegree_WSF creation d'IDI nbRows = %d", nbRows);
	for(int i=0;i<nbRows;i++) {
	  //
	  // determination de l'entite touchee
	  // recuperation de son LastOperationMemory
	  //
	  Entity* pRow_i = pVegetal->GetElement(i);
	  Entity* pMemory 
		= pRow_i->GetEntityConstValue(LAST_OPERATIONS_MEMORY_ATTRIBUTE);
	  //
	  // recuparation modalite gestion sol
	  //
	  int modality = pRow_i->GetIntConstValue(SOIL_MANAGEMENT_MODALITY);
	  //
	  // instanciation d'une ss-classe de Last<X>MemoryItem en fonction de X
	  //
	  Entity* pMemoryItem = NULL;
	  if(isSoilTillageOT) {
		//---------------
//printf("\n\trow '%s' modality : %d", pRow_i->InstanceName(), modality); 
		if((modality == TILLAGE)
		   //|| (modality == MIXED_TILLAGE_CW)
		   ) {
		  pMemoryItem = new LastTillageOperationsMemoryItem();
		  if(pOt->IsInstanceOf(ROTARY_OPERATED_TILLAGE)
			 ||
			 isRotaryThinningOT) {
			pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "rotaryCultivator");
		  }
		  else {
			pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "springTineCultivator");
		  }
		  Entity* pLastOtMemory // quel usage ?
			= pRow_i->GetEntityConstValue(LAST_OPERATIONS_MEMORY_ATTRIBUTE);
		  //
		  // local -> UTC
		  //
		  StructLocalTmToUTCStringISO(pTm_now);
//printf("\nen UTC   : %s", gTimeString)
		  pMemoryItem->SetStringConstValue(DATE_ISO_ATTRIBUTE, gTimeString);
		  pMemoryItem->SetIntConstValue(INSTANT_ATTRIBUTE, now);
		  //
		  // ajout du Last<X>MemoryItem comme element du LastOperationMemory
		  //
		  pMemory->AddElement(pMemoryItem);
//printf("\nachievementDegree_WSF creation d'IDI sur '%s' : ", pRow_i->InstanceName());
//pMemory->DisplayElementNames();
//pMemoryItem->DisplayAttributes();
		}
	  }
	  else if(isPesticideApplicationOT) {
		//
		// determination caractere traite/non traite de la rangee
		//
		bool isTreated = FALSE;
		if(isPhytoTreatmentAct) {
		  isTreated = TRUE;
		}
		else if(isChemicalWeedingAct) {
		  switch (modality) {
		  case CHEMICAL_WEEDING: {
		  isTreated = TRUE;
			break;
		  }
		  default : {
		  }
		  } //fin switch
		}
		else if(isDebranchingAct) {
		  if((i == 2) || (i == 3))
			{isTreated = TRUE;}
		}
		if(isTreated) {
		  for(int k=0;k<nbProducts;k++) {
			pMemoryItem = new LastPesticideApplicationsMemoryItem();
		  
			if(pOt->IsInstanceOf(BACK_SPRAYING_OPERATION)) {
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "backSprayer");
			}
			else if(pOt->IsInstanceOf(AIR_ASSISTED_SPRAYING)){
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "airAssistedSprayer");
			}
			else if(pOt->IsInstanceOf(CHEMICAL_DEBRANCHER_OPERATED_SPRAYING)){
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "debranchingSprayer");
			}
			else if(pOt->IsInstanceOf(HAND_HELD_GUN_OPERATED_SPRAYING)){
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "handHeldSprayGun");
			}
			else if(pOt->IsInstanceOf(HORIZONTAL_RAIL_OPERATED_SPRAYING)){
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "horizontalRailSprayer");
			}
			else {
			  pMemoryItem->SetStringConstValue(TOOL_CLASS_NAME, "verticalRailSprayer");
			}
			float x = newValue;
			pMemoryItem->SetIntConstValue(ACTUALLY_OPERATED_AREA_PERCENTAGE, 
										  (int)(x*100.));

			if(isPhytoTreatmentAct) {
			  if(typeChantierPhyto == ANTI_MILDIOU) {
				pMemoryItem->SetStringConstValue(PESTICIDE_TYPE, "fongicide");
			  }
			  else if((typeChantierPhyto == REGULAR_PHYTO)
					  ||
					  (typeChantierPhyto == EXTRA_PHYTO)) {
				pMemoryItem->SetStringConstValue(PESTICIDE_TYPE, "fongicide");
			  }
			  else {
				if(gMyTraceWarnings)
				  printf("\n!!! achievementDegree_WSF creation IDI type chantier inconnu : '%s'", ClassIdToCharTab[pOt->ClassId()]);
			  }
			}
			else if(isChemicalWeedingAct) {
			  pMemoryItem->SetStringConstValue(PESTICIDE_TYPE, "herbicide");
			}
			else if(isDebranchingAct) {
			  pMemoryItem->SetStringConstValue(PESTICIDE_TYPE, "herbicide");
			}
			else {
			  if(gMyTraceWarnings)
				printf("\n!!! achievementDegree_WSF creation IDI activite non geree : '%s'", ClassIdToCharTab[pInst->ClassId()]);
			}
			char* product =  treatmentList->get_element(k);
			pMemoryItem->SetStringConstValue(PESTICIDE_SUBTYPE, 
	                                         product); // copie par strdup
			//
			// local -> UTC
			//
			StructLocalTmToUTCStringISO(pTm_now);
			//printf("\nen UTC   : %s", gTimeString)
			pMemoryItem->SetStringConstValue(DATE_ISO_ATTRIBUTE, gTimeString);
			pMemoryItem->SetIntConstValue(INSTANT_ATTRIBUTE, now);
			//
			// ajout du Last<X>MemoryItem comme element du LastOperationMemory
			//
			pMemory->AddElement(pMemoryItem);
//printf("\nachievementDegree_WSF creation d'IDI sur '%s' : ", pRow_i->InstanceName());
//pMemory->DisplayElementNames();
//pMemoryItem->DisplayAttributes();

		  } // fin boucle sur produits
		} // fin cas pRow_i isTreated
	  } // fin isPesticideApplicationOT
	  else {
		if(gMyTraceWarnings)
		  printf("\n!!! achievementDegree_WSF creation IDI operation non geree : '%s'", ClassIdToCharTab[pOt->ClassId()]);
	  }
	}
	if(treatmentList) {  // modif JPR 120216
	  for(int i=nbProducts-1;i>=0;i--) {
		free(treatmentList->get_nth(i)); 
	  }
	  delete treatmentList;
	}
  }

  // --------------------------------------------------------
  // tache 5) augmentation TERMINATED_PLOTS_ATTRIBUTE du chantier
  //          mise a jour du PERCENT_X descripteur de la parcelle
  // --------------------------------------------------------
  if(newValue > (float)0.99999) {	
	if(pChantier) {
	  pChantier->AddEntityValueElement(TERMINATED_PLOTS_ATTRIBUTE,
									   pPlot);
	  // tache 5bis)  JPR 130314 5.5
	  int currentValue = 
		pChantier->GetIntConstValue(ACTUAL_REALIZATION_END_DATE);
	  if(now > currentValue)
		pChantier->SetIntConstValue(ACTUAL_REALIZATION_END_DATE, now);
	}
  }
  UserClassIdTab classIds = pOt->AncestorIds();
  classIds.push_back(pOt->ClassId());

  Boolean isTillage = FALSE;
  Boolean isMotorizedSpraying = FALSE; 
  Boolean isBackSpraying = FALSE;
  Boolean isManualDebranching = FALSE;
  Boolean isChemicalDebrancherOperatedSpraying = FALSE;
  Boolean isVerticalRailOperatedSpraying = FALSE;
  Boolean isAirAssistedSpraying = FALSE;
  Boolean isHandHeldGunOperatedSpraying = FALSE;

  if(GetPosition(classIds, TILLAGE_OPERATION) >= 0)
	isTillage = TRUE;  
  else if(GetPosition(classIds, MOTORIZED_SPRAYING_OPERATION) >= 0)
	isMotorizedSpraying = TRUE;
  else if(GetPosition(classIds, BACK_SPRAYING_OPERATION) >= 0)
	isBackSpraying =  TRUE;
  else if(GetPosition(classIds, MANUAL_DEBRANCHING_OPERATION) >= 0)
	isManualDebranching = TRUE; 
  else if(GetPosition(classIds, CHEMICAL_DEBRANCHER_OPERATED_SPRAYING) >= 0)
	isChemicalDebrancherOperatedSpraying = TRUE; 
  else if(GetPosition(classIds, VERTICAL_RAIL_OPERATED_SPRAYING) >= 0)
	isVerticalRailOperatedSpraying = TRUE; 
  else if(GetPosition(classIds, AIR_ASSISTED_SPRAYING) >= 0) 
	isAirAssistedSpraying = TRUE;  
  else if(GetPosition(classIds, HAND_HELD_GUN_OPERATED_SPRAYING) >= 0)
	isHandHeldGunOperatedSpraying = TRUE;  

  char* nomChantier = pChantier->ClassName();
  if(isTillage) {
	if(strstr(nomChantier, "Autumn"))
	  pPlot->SetFloatVarValue(PERCENT_AUTUMN_TILLED, newValue);
	else if(strstr(nomChantier, "Winter"))
	  pPlot->SetFloatVarValue(PERCENT_WINTER_TILLED, newValue);
	else if(strstr(nomChantier, "InitSpring"))
	  pPlot->SetFloatVarValue(PERCENT_INIT_SPRING_TILLED, newValue);
	else if(strstr(nomChantier, "Spring"))
	  pPlot->SetFloatVarValue(PERCENT_SPRING_TILLED, newValue);
  }
  else if(isMotorizedSpraying
	      ||
	      isBackSpraying) {
	if(strstr(nomChantier, "SpringChemicalWeedings"))
	  pPlot->SetFloatVarValue(PERCENT_SPRING_WEEDED, newValue);
	else if(strstr(nomChantier, "ChemicalWeedings"))
	  pPlot->SetFloatVarValue(PERCENT_WEEDED, newValue);
	else if(strstr(nomChantier, "Debranchings"))
	  pPlot->SetFloatVarValue(PERCENT_DEBRANCHED, newValue);
	else if(strstr(nomChantier, "Treatments")) { // + JPR 130126
	  if(strstr(nomChantier, "Regular"))
		pPlot->SetFloatVarValue(PERCENT_REGULAR_PHYTO_TREATED, newValue);
	  else if(strstr(nomChantier, "Extra"))
		pPlot->SetFloatVarValue(PERCENT_EXTRA_PHYTO_TREATED, newValue);
	  else if(strstr(nomChantier, "Anti"))
		pPlot->SetFloatVarValue(PERCENT_ANTI_MILDEW_TREATED, newValue);
	}
  }
  else if(isManualDebranching
		  || // + JPR 130126
		  isChemicalDebrancherOperatedSpraying) {
	pPlot->SetFloatVarValue(PERCENT_DEBRANCHED, newValue);
  }
  else if(isVerticalRailOperatedSpraying
		  || // + JPR 130126
		  isAirAssistedSpraying
		  || // + JPR 130126
		  isHandHeldGunOperatedSpraying) {
	if(strstr(nomChantier, "Debranchings"))
	  pPlot->SetFloatVarValue(PERCENT_DEBRANCHED, newValue);
	else if(strstr(nomChantier, "Treatments")) { // + JPR 130126
	  if(strstr(nomChantier, "Regular"))
		pPlot->SetFloatVarValue(PERCENT_REGULAR_PHYTO_TREATED, newValue);
	  else if(strstr(nomChantier, "Extra"))
		pPlot->SetFloatVarValue(PERCENT_EXTRA_PHYTO_TREATED, newValue);
	  else if(strstr(nomChantier, "Anti"))
		pPlot->SetFloatVarValue(PERCENT_ANTI_MILDEW_TREATED, newValue);
	}
  }
  else if(GetPosition(classIds, PRUNING_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_PRUNED, newValue);
  else if(GetPosition(classIds, SHOOT_THINNING_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_SHOOT_THINNED, newValue);
  else if(GetPosition(classIds, FERTILIZATION_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_FERTILIZED, newValue);
  else if(GetPosition(classIds, PLANTATION_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_PLANTED, newValue);
  else if(GetPosition(classIds, TRELLISING_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_TRELLISED, newValue);
  else if(GetPosition(classIds, CLIPPING_OPERATION) >= 0)
	pPlot->SetFloatVarValue(PERCENT_CLIPPED, newValue);

  // --------------------------------------------------------
  // tache 6) valuation de ACTUAL_REALIZATION_BEG_DATE
  // --------------------------------------------------------
  if((oldValue < gFloatingPointEqualityTolerance)
	 &&
	 (newValue > gFloatingPointEqualityTolerance)) {	
	if(pChantier) {
	  int currentValue = 
		pChantier->GetIntConstValue(ACTUAL_REALIZATION_BEG_DATE);
	  if(currentValue < 0)
		pChantier->SetIntConstValue(ACTUAL_REALIZATION_BEG_DATE, now);
	}
	// --------------------------------------------------------
	// tache 6bis) valuation de LAST_TILLAGE_OPERATION_CLOCK
	// --------------------------------------------------------
	if(pInst->IsInstanceOf(TILLAGE_ACTIVITY)) {
	  pPlot->SetIntConstValue(LAST_TILLAGE_OPERATION_CLOCK, now);
	}
  }  
  
  // --------------------------------------------------------
  // tache 7) diminution des montants d'heures restant a travailler
  // --------------------------------------------------------
  int deltaHours = pOt->GetIntConstValue(STEP);
  int nbPerfs = perfList->size();
  for(int k=0;k<nbPerfs;k++) {
	Entity* pMO = (Entity*)perfList->get_element(k);
	int newDailyLeft = pMO->GetIntVarValue(WORKING_TIME_DAILY_LEFT)   - deltaHours;
	pMO->SetIntVarValue(WORKING_TIME_DAILY_LEFT, newDailyLeft);
	int newWeeklyLeft = pMO->GetIntVarValue(WORKING_TIME_WEEKLY_LEFT) - deltaHours;
	pMO->SetIntVarValue(WORKING_TIME_WEEKLY_LEFT, newWeeklyLeft);
	int newYearlyLeft = pMO->GetIntVarValue(WORKING_TIME_YEARLY_LEFT) - deltaHours;
	pMO->SetIntVarValue(WORKING_TIME_YEARLY_LEFT, newYearlyLeft);
	//
	// eventuelle mise en indisponibilite
	//
	if((newDailyLeft <= 0) || (newWeeklyLeft <= 0) || (newYearlyLeft <= 0)) {
	  pMO->SetIntVarValue(AVAILABILITY_STATUS, 0);
	  if(gMyTraceWarnings)
		printf("\n!!! %s mise en indisponibilité en t=%d (newDailyLeft=%d newWeeklyLeft=%d newYearlyLeft=%d)", pMO->InstanceName(), now+1, newDailyLeft, newWeeklyLeft, newYearlyLeft);
	  // on programme un MIL immédiat, sans déprogrammer les autres MIL
	  // on deprogramme en  meme temps les proceedOperationprocess
	  Entity* pArb = NULL;
	  ResourceSet* pPool = ((SingleResource*)pMO)->GetARBResourcePool();
	  if(pPool) pArb = pPool->GetSuperEntity();
	  if (! C_ExistMakeInstructionListEventAtDate(now+1, pArb)) {
		PostMakeInstructionListEvent2(now+1, pArb, FALSE);
	  }
	}
	/* 
	// incidemment, test de GetFutureAvailability
	//
	Entity* pEmploymentPlan = 
		pMO->GetEntityConstValue(ANNUAL_EMPLOYMENT_PLAN_ATTRIBUTE);
	int index = pEmploymentPlan->GetIntVarValue(CURRENT_ELEMENT_INDEX);
	Entity* pSeasonalPlan = pEmploymentPlan->GetElement(index);
	RecurrentAvailabilityTimetable* pRATT = (RecurrentAvailabilityTimetable*)pSeasonalPlan->GetEntityConstValue(TIMETABLE_ATTRIBUTE);
	
	int plus0 = pRATT->GetFutureAvailability(now);
	int plus1 = pRATT->GetFutureAvailability(now+1);
	int plus5 = pRATT->GetFutureAvailability(now+5);
	int plusA = pRATT->GetFutureAvailability(now+10);
	printf("\nFutureAvailability *0 : %d\t*1 = %d\t*5 = %d\t*10 = %d", plus0, plus1, plus5, plusA);
	*/
//printf("\nLEFT '%s' D:%d Y:%d 	 '%s' W:%d  -> status = %d", pMO->InstanceName(), newDailyLeft, newYearlyLeft, pSeasonalPlan->InstanceName(), newWeeklyLeft, pMO->GetIntVarValue(AVAILABILITY_STATUS));

	// --------------------------------------------------------
	// tache 8) cumul pluie recue
	// --------------------------------------------------------
	Entity* pWS = pPlot->GetEntityConstValue(WEATHER_STATION_ATTRIBUTE);
	float rain = pWS->GetFloatVarValue(RAINFALL);
	if(rain > 0) {
	  float oldValue = pMO->GetFloatVarValue(CUMULATIVE_RAINFALL_FROM_WORK_BEGINNING);
	  pMO->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_WORK_BEGINNING, 
						  oldValue + rain);  // no more monitored
	  // exploite par lowCumulativeRainSuffered_stateRelatedPredicate
	  pMO->SetIntConstValue(LAST_RAINFALL_CLOCK, now);
	}
  }

  // --------------------------------------------------------
  // tache 9) trace sur fichier des fins d'activités primitives
  //          
  // --------------------------------------------------------
  if(newValue > (float)0.99999) {
	char* tag = "";
	FILE* pF;
	if(gMyEndingTraceAllOperations || gMyEndingTraceSomeOperations) {
	  pF = gFileEndingTrace->GetPhysicalFile();
	  
	  if(pInst->IsInstanceOf(PHYTO_TREATMENT_ACTIVITY)) {
		Entity* pIteration = 
		  (Entity*)pInst->GetFirstContainer(ACTIVITY_ITERATION, FALSE);
		if(pIteration) {
		  if(pIteration->IsInstanceOf(REGULAR_PHYTO_TREATMENTS_ITERATION)) {
			tag = "REGULAR";
		  }
		  else if(pIteration->IsInstanceOf(EXTRA_PHYTO_TREATMENTS_ITERATION)) {
			tag = "EXTRA";
		  }
		}
	  }
	}
	if(gMyEndingTraceAllOperations) {
	  fprintf(pF, "\n[t=%d] %s ", now, ShortWeekDayNameToCharTab[pTm_now->tm_wday]);
	  PrintDateFromClock(pF, now, DATE_HDM);
	  fprintf(pF, " '%s' sur '%s'%s", pInst->InstanceName(), pPlot->InstanceName(), tag);
	}
	if(gMyEndingTraceSomeOperations) {
	  char* tmp = strdup(gMyEndingTraceStringTab);
	  char* nom = strtok(tmp, "_");
	  while(nom) {
		if(strstr(pInst->InstanceName(), nom)) {
		  fprintf(pF, "\n[t=%d] %s ", now, ShortWeekDayNameToCharTab[pTm_now->tm_wday]);
		  PrintDateFromClock(pF, now, DATE_HDM);
		  fprintf(pF, " '%s' sur '%s'%s", pInst->InstanceName(), pPlot->InstanceName(), tag);

		  break;
		}
		nom = strtok(NULL, "_");
	  }
	  free(tmp);
	}
  }

  delete perfList;
  delete pTm_now;
//printf("\nachievementDegree_whenSetFloat returns %f\n", newValue);
  return newValue;
};

