// ==============================================
// Fichier : UserConstantDescriptor.cc  
// Type    : source C++
// 
// Description : constructeurs des classes des 'ConstantDescriptor'.
// ==============================================
  
#include "UserConstantDescriptor.h"
  

// ==============================================
// Une classe de Descriptor : 'ActivityAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

ActivityAttribute::ActivityAttribute() {
	ClassName   ("activityAttribute");
	ClassComment   ("descripteur des DistributedActivitySetMemoryItem");
	ClassSymbol (ACTIVITY_ATTRIBUTE);
	AddValueClassId(ACTIVITY);
};



// ==============================================
// Une classe de Descriptor : 'ActivityNameAttribute'    (de sous-classe : 'STRING').
// ==============================================

ActivityNameAttribute::ActivityNameAttribute() {
	ClassName   ("activityNameAttribute");
	ClassComment   ("descripteur des DistributedActivitySetMemoryItem");
	ClassSymbol (ACTIVITY_NAME_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'ActivityNominalDate'    (de sous-classe : 'INT').
// ==============================================

ActivityNominalDate::ActivityNominalDate() {
	ClassName   ("activityNominalDate");
	ClassComment   ("instant optimal de debut de realisation d'un chantier");
	ClassSymbol (ACTIVITY_NOMINAL_DATE);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'ActualRealizationBegDate'    (de sous-classe : 'INT').
// ==============================================

ActualRealizationBegDate::ActualRealizationBegDate() {
	ClassName   ("actualRealizationBegDate");
	ClassComment   ("horloge lors de la premiere progression d'operation sur une des parcelles du chantier");
	ClassSymbol (ACTUAL_REALIZATION_BEG_DATE);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'ActualRealizationEndDate'    (de sous-classe : 'INT').
// ==============================================

ActualRealizationEndDate::ActualRealizationEndDate() {
	ClassName   ("actualRealizationEndDate");
	ClassComment   ("horloge lors de la derniere progression d'operation sur une des parcelles du chantier");
	ClassSymbol (ACTUAL_REALIZATION_END_DATE);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'ActuallyOperatedAreaPercentage'    (de sous-classe : 'INT').
// ==============================================

ActuallyOperatedAreaPercentage::ActuallyOperatedAreaPercentage() {
	ClassName   ("actuallyOperatedAreaPercentage");
	ClassComment   ("Lorsqu'une OT est terminee, pourcentage de la surface qui a reellement subi l'effet");
	ClassSymbol (ACTUALLY_OPERATED_AREA_PERCENTAGE);
	DescIntValueSpec* pDVS = new DescIntValueSpec(ACTUALLY_OPERATED_AREA_PERCENTAGE, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0, 100);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'AgeAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

AgeAttribute::AgeAttribute() {
	ClassName   ("ageAttribute");
	ClassComment   ("age exprime par la classe PlantAge");
	ClassSymbol (AGE_ATTRIBUTE);
	AddValueClassId(PLANT_AGE);
};



// ==============================================
// Une classe de Descriptor : 'AlleyWidth'    (de sous-classe : 'INT').
// ==============================================

AlleyWidth::AlleyWidth() {
	ClassName   ("alleyWidth");
	ClassComment   ("ecartement des rangs de vigne");
	ClassUnity   ("cm");
	ClassSymbol (ALLEY_WIDTH);
};



// ==============================================
// Une classe de Descriptor : 'AllocationPriorityOptionSetAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

AllocationPriorityOptionSetAttribute::AllocationPriorityOptionSetAttribute() {
	ClassName   ("allocationPriorityOptionSetAttribute");
	ClassComment   ("pointeur sur l'objet encapsulant les priorites d'allocation par activite et phase");
	ClassSymbol (ALLOCATION_PRIORITY_OPTION_SET_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'AnnualEmploymentPlanAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

AnnualEmploymentPlanAttribute::AnnualEmploymentPlanAttribute() {
	ClassName   ("annualEmploymentPlanAttribute");
	ClassComment   ("l'instance de AnnualEmploymentPlan attachee a un FarmWorker");
	ClassSymbol (ANNUAL_EMPLOYMENT_PLAN_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'AntiMildewScheduled'    (de sous-classe : 'INT').
// ==============================================

AntiMildewScheduled::AntiMildewScheduled() {
	ClassName   ("antiMildewScheduled");
	ClassComment   ("pour un chantier, vrai ssi un anti-mildiou a ete programme");
	ClassSymbol (ANTI_MILDEW_SCHEDULED);
};



// ==============================================
// Une classe de Descriptor : 'AntiMildewTreatmentJustDone'    (de sous-classe : 'INT').
// ==============================================

AntiMildewTreatmentJustDone::AntiMildewTreatmentJustDone() {
	ClassName   ("antiMildewTreatmentJustDone");
	ClassComment   ("passe � 1 quand un anti-mildiou est 'ouvert', passe � 0 qd un traitement 'ouvert' est sans anti-mildiou");
	ClassSymbol (ANTI_MILDEW_TREATMENT_JUST_DONE);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'Area'    (de sous-classe : 'FLOAT').
// ==============================================

Area::Area() {
	ClassName   ("area");
	ClassComment   ("superficie");
	ClassUnity   ("ha");
	ClassSymbol (AREA);
};



// ==============================================
// Une classe de Descriptor : 'AreaWithGrapevine'    (de sous-classe : 'FLOAT').
// ==============================================

AreaWithGrapevine::AreaWithGrapevine() {
	ClassName   ("areaWithGrapevine");
	ClassComment   ("superficie en vigne");
	ClassUnity   ("ha");
	ClassSymbol (AREA_WITH_GRAPEVINE);
};



// ==============================================
// Une classe de Descriptor : 'AreaWithTilledGrapevine'    (de sous-classe : 'FLOAT').
// ==============================================

AreaWithTilledGrapevine::AreaWithTilledGrapevine() {
	ClassName   ("areaWithTilledGrapevine");
	ClassComment   ("superficie en vigne entretenue par travail du sol");
	ClassUnity   ("ha");
	ClassSymbol (AREA_WITH_TILLED_GRAPEVINE);
};



// ==============================================
// Une classe de Descriptor : 'AutumnTillagesManagement'    (de sous-classe : 'STRING').
// ==============================================

AutumnTillagesManagement::AutumnTillagesManagement() {
	ClassName   ("autumnTillagesManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (AUTUMN_TILLAGES_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(AUTUMN_TILLAGES_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'BeginningCalendarDate'    (de sous-classe : 'STRING').
// ==============================================

BeginningCalendarDate::BeginningCalendarDate() {
	ClassName   ("beginningCalendarDate");
	ClassComment   ("date de debut d'une periode non travaillee, feriee ou de conge");
	ClassSymbol (BEGINNING_CALENDAR_DATE);
	SetToDefaultValue("");
};



// ==============================================
// Une classe de Descriptor : 'ChemicalWeedingsManagement'    (de sous-classe : 'STRING').
// ==============================================

ChemicalWeedingsManagement::ChemicalWeedingsManagement() {
	ClassName   ("chemicalWeedingsManagement");
	ClassComment   ("nécessairement en série (SER) ou possiblement en parallèle (PAR) sur les parcelles");
	ClassSymbol (CHEMICAL_WEEDINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(CHEMICAL_WEEDINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'ClippingsManagement'    (de sous-classe : 'STRING').
// ==============================================

ClippingsManagement::ClippingsManagement() {
	ClassName   ("clippingsManagement");
	ClassComment   ("nécessairement en série (SER) ou possiblement en parallèle (PAR) sur les parcelles");
	ClassSymbol (CLIPPINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(CLIPPINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'CommercialFormulation'    (de sous-classe : 'STRING').
// ==============================================

CommercialFormulation::CommercialFormulation() {
	ClassName   ("commercialFormulation");
	ClassComment   ("RoundUp, ...");
	ClassSymbol (COMMERCIAL_FORMULATION);
};



// ==============================================
// Une classe de Descriptor : 'CommercialFormulationDose'    (de sous-classe : 'FLOAT').
// ==============================================

CommercialFormulationDose::CommercialFormulationDose() {
	ClassName   ("commercialFormulationDose");
	ClassComment   ("quantite de produit commercial par hectare");
	ClassUnity   ("kg ou l selon le produit commercial");
	ClassSymbol (COMMERCIAL_FORMULATION_DOSE);
};



// ==============================================
// Une classe de Descriptor : 'CommercialFormulationList'    (de sous-classe : 'STRING_TAB').
// ==============================================

CommercialFormulationList::CommercialFormulationList() {
	ClassName   ("commercialFormulationList");
	ClassComment   ("(RoundUp, ...)");
	ClassSymbol (COMMERCIAL_FORMULATION_LIST);
};



// ==============================================
// Une classe de Descriptor : 'ControlPlanAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

ControlPlanAttribute::ControlPlanAttribute() {
	ClassName   ("controlPlanAttribute");
	ClassComment   ("la plan par lequel une EA est conduit");
	ClassSymbol (CONTROL_PLAN_ATTRIBUTE);
	AddValueClassId(ACTIVITY);
};



// ==============================================
// Une classe de Descriptor : 'ControlPlanClassNameAttribute'    (de sous-classe : 'STRING').
// ==============================================

ControlPlanClassNameAttribute::ControlPlanClassNameAttribute() {
	ClassName   ("controlPlanClassNameAttribute");
	ClassComment   ("descripteur de Vineyard pointant sur son plan de conduite");
	ClassSymbol (CONTROL_PLAN_CLASS_NAME_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'CropRowAlleyWidthRatio'    (de sous-classe : 'FLOAT').
// ==============================================

CropRowAlleyWidthRatio::CropRowAlleyWidthRatio() {
	ClassName   ("cropRowAlleyWidthRatio");
	ClassComment   ("largeur bande entretenue homogene autour des ceps / ecartement entre 2 rangs");
	ClassUnity   ("%");
	ClassSymbol (CROP_ROW_ALLEY_WIDTH_RATIO);
};



// ==============================================
// Une classe de Descriptor : 'CurrentSlotToBeExtended'    (de sous-classe : 'INT').
// ==============================================

CurrentSlotToBeExtended::CurrentSlotToBeExtended() {
	ClassName   ("currentSlotToBeExtended");
	ClassComment   ("1 ssi la terminaison d'une operation justifie de prolonger le TimeSlot en cours");
	ClassSymbol (CURRENT_SLOT_TO_BE_EXTENDED);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'DailyRainfallForecastFromCurrentDay'    (de sous-classe : 'FLOAT_TAB').
// ==============================================

DailyRainfallForecastFromCurrentDay::DailyRainfallForecastFromCurrentDay() {
	ClassName   ("dailyRainfallForecastFromCurrentDay");
	ClassComment   ("hauteurs de pluie (mm) pr�vues dans les jours a venir (index 0 pour aujourd'hui)");
	ClassSymbol (DAILY_RAINFALL_FORECAST_FROM_CURRENT_DAY);
};



// ==============================================
// Une classe de Descriptor : 'DayOfAchievement'    (de sous-classe : 'INT').
// ==============================================

DayOfAchievement::DayOfAchievement() {
	ClassName   ("dayOfAchievement");
	ClassComment   ("jour (julien) d'atteinte d'un stade");
	ClassSymbol (DAY_OF_ACHIEVEMENT);
	DescIntValueSpec* pDVS = new DescIntValueSpec(DAY_OF_ACHIEVEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(1, 366);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'DebranchingsManagement'    (de sous-classe : 'STRING').
// ==============================================

DebranchingsManagement::DebranchingsManagement() {
	ClassName   ("debranchingsManagement");
	ClassComment   ("nécessairement en série (SER) ou possiblement en parallèle (PAR) sur les parcelles");
	ClassSymbol (DEBRANCHINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(DEBRANCHINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'DecisionOptionSetAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

DecisionOptionSetAttribute::DecisionOptionSetAttribute() {
	ClassName   ("decisionOptionSetAttribute");
	ClassSymbol (DECISION_OPTION_SET_ATTRIBUTE);
	AddValueClassId(DECISION_OPTION_SET);
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewLevel'    (de sous-classe : 'INT').
// ==============================================

DownyMildewLevel::DownyMildewLevel() {
	ClassName   ("downyMildewLevel");
	ClassComment   ("0=no 1=scarse 2=frequNoDammage 3=freqAndDommage");
	ClassSymbol (DOWNY_MILDEW_LEVEL);
	DescIntValueSpec* pDVS = new DescIntValueSpec(DOWNY_MILDEW_LEVEL, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0, 3);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewManagementAttribute'    (de sous-classe : 'INT').
// ==============================================

DownyMildewManagementAttribute::DownyMildewManagementAttribute() {
	ClassName   ("downyMildewManagementAttribute");
	ClassComment   ("element de l'enum. DownyMildewManagementModalities");
	ClassSymbol (DOWNY_MILDEW_MANAGEMENT_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewPressure'    (de sous-classe : 'INT').
// ==============================================

DownyMildewPressure::DownyMildewPressure() {
	ClassName   ("downyMildewPressure");
	ClassComment   ("niveau de pression oidium calcule des que .....");
	ClassSymbol (DOWNY_MILDEW_PRESSURE);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewPrimaryInfection'    (de sous-classe : 'INT').
// ==============================================

DownyMildewPrimaryInfection::DownyMildewPrimaryInfection() {
	ClassName   ("downyMildewPrimaryInfection");
	ClassComment   ("0=NON 1 OUI");
	ClassSymbol (DOWNY_MILDEW_PRIMARY_INFECTION);
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewRiskLevel'    (de sous-classe : 'FLOAT').
// ==============================================

DownyMildewRiskLevel::DownyMildewRiskLevel() {
	ClassName   ("downyMildewRiskLevel");
	ClassComment   ("-5 : no risk +5 : high risk level");
	ClassSymbol (DOWNY_MILDEW_RISK_LEVEL);
	DescFloatValueSpec* pDVS = new DescFloatValueSpec(DOWNY_MILDEW_RISK_LEVEL, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(-5., +5.);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'DownyMildewToClose'    (de sous-classe : 'INT').
// ==============================================

DownyMildewToClose::DownyMildewToClose() {
	ClassName   ("downyMildewToClose");
	ClassComment   ("vrai s'il ya lieu d'arreter les traitements anti-mildiou");
	ClassSymbol (DOWNY_MILDEW_TO_CLOSE);
};



// ==============================================
// Une classe de Descriptor : 'EndCalendarDate'    (de sous-classe : 'STRING').
// ==============================================

EndCalendarDate::EndCalendarDate() {
	ClassName   ("endCalendarDate");
	ClassComment   ("date de fin d'une periode non travaillee, feriee ou de conge (vide si = debut)");
	ClassSymbol (END_CALENDAR_DATE);
	SetToDefaultValue("");
};



// ==============================================
// Une classe de Descriptor : 'FarmWorkerAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

FarmWorkerAttribute::FarmWorkerAttribute() {
	ClassName   ("farmWorkerAttribute");
	ClassComment   ("la FarmWorker dotee de l'AnnualEmploymentPlan par ce descripteur");
	ClassSymbol (FARM_WORKER_ATTRIBUTE);
	AddValueClassId(FARM_WORKER);
};



// ==============================================
// Une classe de Descriptor : 'FertilizationsManagement'    (de sous-classe : 'STRING').
// ==============================================

FertilizationsManagement::FertilizationsManagement() {
	ClassName   ("fertilizationsManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (FERTILIZATIONS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(FERTILIZATIONS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'FirstSpringTillageDate'    (de sous-classe : 'INT').
// ==============================================

FirstSpringTillageDate::FirstSpringTillageDate() {
	ClassName   ("firstSpringTillageDate");
	ClassComment   ("instant de debut du premier labour dans le chantier SpringTillages");
	ClassSymbol (FIRST_SPRING_TILLAGE_DATE);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'FlagMildew'    (de sous-classe : 'INT').
// ==============================================

FlagMildew::FlagMildew() {
	ClassName   ("flagMildew");
	ClassComment   ("vrai des que App_GetFlagMildew a renvoye vrai dans une campagne");
	ClassSymbol (FLAG_MILDEW);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'GrapeVarietyAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

GrapeVarietyAttribute::GrapeVarietyAttribute() {
	ClassName   ("grapeVarietyAttribute");
	ClassSymbol (GRAPE_VARIETY_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'GrapeVarietyListAttribute'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

GrapeVarietyListAttribute::GrapeVarietyListAttribute() {
	ClassName   ("grapeVarietyListAttribute");
	ClassComment   ("les varietes effectivement presentes sur une exploitation");
	ClassSymbol (GRAPE_VARIETY_LIST_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'GrassCoverVectorFromMhydas'    (de sous-classe : 'FLOAT_TAB').
// ==============================================

GrassCoverVectorFromMhydas::GrassCoverVectorFromMhydas() {
	ClassName   ("grassCoverVectorFromMhydas");
	ClassComment   ("pour une Row, vect. des couvertures sol sur les unit. hydro. identifiees dans Mhydas");
	ClassSymbol (GRASS_COVER_VECTOR_FROM_MHYDAS);
};



// ==============================================
// Une classe de Descriptor : 'GrassStripAlleyWidthRatio'    (de sous-classe : 'FLOAT').
// ==============================================

GrassStripAlleyWidthRatio::GrassStripAlleyWidthRatio() {
	ClassName   ("grassStripAlleyWidthRatio");
	ClassComment   ("largeur bande entretenue enherbee / ecartement entre 2 rangs");
	ClassUnity   ("%");
	ClassSymbol (GRASS_STRIP_ALLEY_WIDTH_RATIO);
};



// ==============================================
// Une classe de Descriptor : 'GrassTypeAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

GrassTypeAttribute::GrassTypeAttribute() {
	ClassName   ("grassTypeAttribute");
	ClassSymbol (GRASS_TYPE_ATTRIBUTE);
	AddValueClassId(GRASS_TYPE);
};



// ==============================================
// Une classe de Descriptor : 'GuidingEarlyVarietyAttribute'    (de sous-classe : 'STRING').
// ==============================================

GuidingEarlyVarietyAttribute::GuidingEarlyVarietyAttribute() {
	ClassName   ("guidingEarlyVarietyAttribute");
	ClassComment   ("nom d'instance du cepage precoce guide pour un viticulteur donne");
	ClassSymbol (GUIDING_EARLY_VARIETY_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'GuidingLateVarietyAttribute'    (de sous-classe : 'STRING').
// ==============================================

GuidingLateVarietyAttribute::GuidingLateVarietyAttribute() {
	ClassName   ("guidingLateVarietyAttribute");
	ClassComment   ("nom d'instance du cepage tardif guide pour un viticulteur donne");
	ClassSymbol (GUIDING_LATE_VARIETY_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'GuidingVarietiesAttribute'    (de sous-classe : 'STRING_TAB').
// ==============================================

GuidingVarietiesAttribute::GuidingVarietiesAttribute() {
	ClassName   ("guidingVarietiesAttribute");
	ClassComment   ("liste des cepages guides pour un viticulteur donne");
	ClassSymbol (GUIDING_VARIETIES_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'HasEarlyVariety'    (de sous-classe : 'INT').
// ==============================================

HasEarlyVariety::HasEarlyVariety() {
	ClassName   ("hasEarlyVariety");
	ClassComment   ("1 si au moins une variete precoce est presente sur l'exploitation, 0 sinon");
	ClassSymbol (HAS_EARLY_VARIETY);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'HasLateVariety'    (de sous-classe : 'INT').
// ==============================================

HasLateVariety::HasLateVariety() {
	ClassName   ("hasLateVariety");
	ClassComment   ("1 si au moins une variete tardive est presente sur l'exploitation, 0 sinon");
	ClassSymbol (HAS_LATE_VARIETY);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'InOutMode'    (de sous-classe : 'INT').
// ==============================================

InOutMode::InOutMode() {
	ClassName   ("inOutMode");
	ClassComment   ("C_IN=0 : l'element est dans l'ensemble ; C_OUT=1 : l'element n'est pas dans l'ensemble");
	ClassSymbol (IN_OUT_MODE);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'InitSpringTillagesManagement'    (de sous-classe : 'STRING').
// ==============================================

InitSpringTillagesManagement::InitSpringTillagesManagement() {
	ClassName   ("initSpringTillagesManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (INIT_SPRING_TILLAGES_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(INIT_SPRING_TILLAGES_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'IsAllocationExclusive'    (de sous-classe : 'INT').
// ==============================================

IsAllocationExclusive::IsAllocationExclusive() {
	ClassName   ("isAllocationExclusive");
	ClassComment   ("TRUE seulement si doit etre la seulle allouee dans un jeu d'activites (hors TrtPhyto)");
	ClassSymbol (IS_ALLOCATION_EXCLUSIVE);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'IsEarly'    (de sous-classe : 'INT').
// ==============================================

IsEarly::IsEarly() {
	ClassName   ("isEarly");
	ClassComment   ("1 : variete precoce ; 0 tardive");
	ClassSymbol (IS_EARLY);
};



// ==============================================
// Une classe de Descriptor : 'IsManager'    (de sous-classe : 'INT').
// ==============================================

IsManager::IsManager() {
	ClassName   ("isManager");
	ClassComment   ("1 ssi le FarmWorker est chef d'exploitation (plusieurs possibles), 0 sinon");
	ClassSymbol (IS_MANAGER);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'IsoAgeingEntityAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

IsoAgeingEntityAttribute::IsoAgeingEntityAttribute() {
	ClassName   ("isoAgeingEntityAttribute");
	ClassComment   ("entite d'iso-dvlpment attachee a une vigne");
	ClassSymbol (ISO_AGEING_ENTITY_ATTRIBUTE);
	AddValueClassId(ISO_AGEING_ENTITY);
};



// ==============================================
// Une classe de Descriptor : 'IsoAgeingEntitySetAttribute'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

IsoAgeingEntitySetAttribute::IsoAgeingEntitySetAttribute() {
	ClassName   ("isoAgeingEntitySetAttribute");
	ClassComment   ("ens. des entites d'iso-dvlpment identifiables dans un SimTerritory");
	ClassSymbol (ISO_AGEING_ENTITY_SET_ATTRIBUTE);
	AddValueClassId(ISO_AGEING_ENTITY);
};



// ==============================================
// Une classe de Descriptor : 'JulianTimeToAchieve'    (de sous-classe : 'INT').
// ==============================================

JulianTimeToAchieve::JulianTimeToAchieve() {
	ClassName   ("julianTimeToAchieve");
	ClassComment   ("nombre theorique de jours pour passer au stade suivant");
	ClassSymbol (JULIAN_TIME_TO_ACHIEVE);
};



// ==============================================
// Une classe de Descriptor : 'LaiForMowing'    (de sous-classe : 'FLOAT').
// ==============================================

LaiForMowing::LaiForMowing() {
	ClassName   ("laiForMowing");
	ClassComment   ("lai de reference pour la tonte");
	ClassSymbol (LAI_FOR_MOWING);
};



// ==============================================
// Une classe de Descriptor : 'LandUse'    (de sous-classe : 'STRING').
// ==============================================

LandUse::LandUse() {
	ClassName   ("landUse");
	ClassComment   ("Vigne, Champ");
	ClassSymbol (LAND_USE);
};



// ==============================================
// Une classe de Descriptor : 'LastOperationsMemoryAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

LastOperationsMemoryAttribute::LastOperationsMemoryAttribute() {
	ClassName   ("lastOperationsMemoryAttribute");
	ClassComment   ("valeur : LastOperationsMemory, sur-ensemble de Last<X>MemoryItem's");
	ClassSymbol (LAST_OPERATIONS_MEMORY_ATTRIBUTE);
	AddValueClassId(LAST_OPERATIONS_MEMORY);
};



// ==============================================
// Une classe de Descriptor : 'LastRainfallClock'    (de sous-classe : 'INT').
// ==============================================

LastRainfallClock::LastRainfallClock() {
	ClassName   ("lastRainfallClock");
	ClassComment   ("valeur d'horloge lors de la pluie la plus recente");
	ClassSymbol (LAST_RAINFALL_CLOCK);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'LastSevenDaysRainfall'    (de sous-classe : 'FLOAT').
// ==============================================

LastSevenDaysRainfall::LastSevenDaysRainfall() {
	ClassName   ("lastSevenDaysRainfall");
	ClassComment   ("hauteur de pluie dans les sept jours precedents");
	ClassUnity   ("mm");
	ClassSymbol (LAST_SEVEN_DAYS_RAINFALL);
};



// ==============================================
// Une classe de Descriptor : 'LastTillageOperationClock'    (de sous-classe : 'INT').
// ==============================================

LastTillageOperationClock::LastTillageOperationClock() {
	ClassName   ("lastTillageOperationClock");
	ClassComment   ("date du dernier labour sur la parcelle");
	ClassSymbol (LAST_TILLAGE_OPERATION_CLOCK);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'LastWeatherPrintClock'    (de sous-classe : 'INT').
// ==============================================

LastWeatherPrintClock::LastWeatherPrintClock() {
	ClassName   ("lastWeatherPrintClock");
	ClassComment   ("horloge au moment du dernier affichage ecran de la date et de la pluie");
	ClassSymbol (LAST_WEATHER_PRINT_CLOCK);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'Latitude'    (de sous-classe : 'FLOAT').
// ==============================================

Latitude::Latitude() {
	ClassName   ("latitude");
	ClassComment   ("latitude en ........");
	ClassSymbol (LATITUDE);
};



// ==============================================
// Une classe de Descriptor : 'LeafAreaIndexRate'    (de sous-classe : 'FLOAT').
// ==============================================

LeafAreaIndexRate::LeafAreaIndexRate() {
	ClassName   ("leafAreaIndexRate");
	ClassComment   ("taux de croissance foliaire journalier");
	ClassSymbol (LEAF_AREA_INDEX_RATE);
	SetDefaultValue(0.9);
	DescFloatValueSpec* pDVS = new DescFloatValueSpec(LEAF_AREA_INDEX_RATE, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0., 1.);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
	SetToDefaultValue(0.9);
};



// ==============================================
// Une classe de Descriptor : 'LeafLifeSpanMin'    (de sous-classe : 'INT').
// ==============================================

LeafLifeSpanMin::LeafLifeSpanMin() {
	ClassName   ("leafLifeSpanMin");
	ClassComment   ("LLSmin correspond � un gpe fonctionnel");
	ClassSymbol (LEAF_LIFE_SPAN_MIN);
	SetDefaultValue(600);
	SetToDefaultValue(600);
};



// ==============================================
// Une classe de Descriptor : 'Longitude'    (de sous-classe : 'FLOAT').
// ==============================================

Longitude::Longitude() {
	ClassName   ("longitude");
	ClassComment   ("longitude en .........");
	ClassSymbol (LONGITUDE);
};



// ==============================================
// Une classe de Descriptor : 'MaxReplicationNumberInDryPeriod'    (de sous-classe : 'INT').
// ==============================================

MaxReplicationNumberInDryPeriod::MaxReplicationNumberInDryPeriod() {
	ClassName   ("maxReplicationNumberInDryPeriod");
	ClassComment   ("descripteur de SpringTillageIteration");
	ClassSymbol (MAX_REPLICATION_NUMBER_IN_DRY_PERIOD);
	SetDefaultValue(99);
	SetToDefaultValue(99);
};



// ==============================================
// Une classe de Descriptor : 'MeanPriorityDegree'    (de sous-classe : 'FLOAT').
// ==============================================

MeanPriorityDegree::MeanPriorityDegree() {
	ClassName   ("meanPriorityDegree");
	ClassComment   ("degre moyen de priorite (d'allocation) sur les primitives d'un jeu");
	ClassSymbol (MEAN_PRIORITY_DEGREE);
};



// ==============================================
// Une classe de Descriptor : 'MembershipMode'    (de sous-classe : 'INT').
// ==============================================

MembershipMode::MembershipMode() {
	ClassName   ("membershipMode");
	ClassComment   ("IN : les elements sont supposes dans le conteneur ; NOT_IN : ... hors du conteneur");
	ClassSymbol (MEMBERSHIP_MODE);
	SetDefaultValue(1);
	SetToDefaultValue(1);
};



// ==============================================
// Une classe de Descriptor : 'MinPercentageToBeginOperation'    (de sous-classe : 'FLOAT').
// ==============================================

MinPercentageToBeginOperation::MinPercentageToBeginOperation() {
	ClassName   ("minPercentageToBeginOperation");
	ClassComment   ("pourcentage de realisation mini avant fin (demi)journee pour pouvoir demarrer");
	ClassSymbol (MIN_PERCENTAGE_TO_BEGIN_OPERATION);
	SetDefaultValue(0.);
	SetToDefaultValue(0.);
};



// ==============================================
// Une classe de Descriptor : 'NeedForNextFavourableDays'    (de sous-classe : 'INT').
// ==============================================

NeedForNextFavourableDays::NeedForNextFavourableDays() {
	ClassName   ("needForNextFavourableDays");
	ClassComment   ("besoin en nb mini de prochains jours en conditions favorables");
	ClassSymbol (NEED_FOR_NEXT_FAVOURABLE_DAYS);
	SetDefaultValue(2);
	SetToDefaultValue(2);
};



// ==============================================
// Une classe de Descriptor : 'NumberOfTractors'    (de sous-classe : 'INT').
// ==============================================

NumberOfTractors::NumberOfTractors() {
	ClassName   ("numberOfTractors");
	ClassComment   ("nombre de tracteurs dans l'exploitation");
	ClassSymbol (NUMBER_OF_TRACTORS);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'NumberOfWorkers'    (de sous-classe : 'INT').
// ==============================================

NumberOfWorkers::NumberOfWorkers() {
	ClassName   ("numberOfWorkers");
	ClassComment   ("nombre d'unités de MO dans l'exploitation");
	ClassSymbol (NUMBER_OF_WORKERS);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'OffWorkCalendarAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

OffWorkCalendarAttribute::OffWorkCalendarAttribute() {
	ClassName   ("offWorkCalendarAttribute");
	ClassComment   ("pointeur sur le calendrier des jours feries et de conge pour cet AnnualEmploymentPlan");
	ClassSymbol (OFF_WORK_CALENDAR_ATTRIBUTE);
	AddValueClassId(OFF_WORK_CALENDAR);
};



// ==============================================
// Une classe de Descriptor : 'OperatedPlotsAttribute'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

OperatedPlotsAttribute::OperatedPlotsAttribute() {
	ClassName   ("operatedPlotsAttribute");
	ClassComment   ("les parcelles en cours dans une IteratedActivityConjunction");
	ClassSymbol (OPERATED_PLOTS_ATTRIBUTE);
	AddValueClassId(PLOT);
};



// ==============================================
// Une classe de Descriptor : 'OperationPriorityOptionSetAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

OperationPriorityOptionSetAttribute::OperationPriorityOptionSetAttribute() {
	ClassName   ("operationPriorityOptionSetAttribute");
	ClassComment   ("pointeur sur l'objet encapsulant les priorites d'operation par activite et phase");
	ClassSymbol (OPERATION_PRIORITY_OPTION_SET_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'OperationRecordingAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

OperationRecordingAttribute::OperationRecordingAttribute() {
	ClassName   ("operationRecordingAttribute");
	ClassSymbol (OPERATION_RECORDING_ATTRIBUTE);
	AddValueClassId(OPERATION_RECORDING_STRUCTURE);
};



// ==============================================
// Une classe de Descriptor : 'OptActualDownyMildewRiskThreshold'    (de sous-classe : 'FLOAT').
// ==============================================

OptActualDownyMildewRiskThreshold::OptActualDownyMildewRiskThreshold() {
	ClassName   ("optActualDownyMildewRiskThreshold");
	ClassComment   ("risque observe minimum pour continuer a traiter anti mildiou");
	ClassSymbol (OPT_ACTUAL_DOWNY_MILDEW_RISK_THRESHOLD);
};



// ==============================================
// Une classe de Descriptor : 'OptAutumnTillageCP'    (de sous-classe : 'INT').
// ==============================================

OptAutumnTillageCP::OptAutumnTillageCP() {
	ClassName   ("optAutumnTillageCP");
	ClassComment   ("NOVEMBER ; DECEMBER ; BEG_PRUN");
	ClassSymbol (OPT_AUTUMN_TILLAGE_C_P);
};



// ==============================================
// Une classe de Descriptor : 'OptAutumnTillageOP'    (de sous-classe : 'INT').
// ==============================================

OptAutumnTillageOP::OptAutumnTillageOP() {
	ClassName   ("optAutumnTillageOP");
	ClassComment   ("END_HARV ; MID_OCT ; NONE");
	ClassSymbol (OPT_AUTUMN_TILLAGE_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptChemicalWeedingOP'    (de sous-classe : 'INT').
// ==============================================

OptChemicalWeedingOP::OptChemicalWeedingOP() {
	ClassName   ("optChemicalWeedingOP");
	ClassComment   ("END_PRUN_THIN ; MID_JAN ; MID_FEB ; EARLY_MAR ; MID_MAR");
	ClassSymbol (OPT_CHEMICAL_WEEDING_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptCropRowAlleyWidthRatio'    (de sous-classe : 'FLOAT').
// ==============================================

OptCropRowAlleyWidthRatio::OptCropRowAlleyWidthRatio() {
	ClassName   ("optCropRowAlleyWidthRatio");
	ClassComment   ("valeur de reference pour les CropRowAlleyWidthRatio des parcelles");
	ClassUnity   ("%");
	ClassSymbol (OPT_CROP_ROW_ALLEY_WIDTH_RATIO);
};



// ==============================================
// Une classe de Descriptor : 'OptDebranchingModality'    (de sous-classe : 'INT').
// ==============================================

OptDebranchingModality::OptDebranchingModality() {
	ClassName   ("optDebranchingModality");
	ClassComment   ("PART_CHEM ; PART_MECA ; PART_MECA_CHEM ; WHOLE_CHEM ; WHOLE_MECA ; WHOLE_MECA_CHEM");
	ClassSymbol (OPT_DEBRANCHING_MODALITY);
};



// ==============================================
// Une classe de Descriptor : 'OptDownyMildewAtLeastClosing'    (de sous-classe : 'INT').
// ==============================================

OptDownyMildewAtLeastClosing::OptDownyMildewAtLeastClosing() {
	ClassName   ("optDownyMildewAtLeastClosing");
	ClassComment   ("0:pea-sized, ...");
	ClassSymbol (OPT_DOWNY_MILDEW_AT_LEAST_CLOSING);
};



// ==============================================
// Une classe de Descriptor : 'OptDownyMildewManagement'    (de sous-classe : 'INT').
// ==============================================

OptDownyMildewManagement::OptDownyMildewManagement() {
	ClassName   ("optDownyMildewManagement");
	ClassComment   ("R0 ; R1 ; RFLO ; F2 ; F3 ; FFLO ; S");
	ClassSymbol (OPT_DOWNY_MILDEW_MANAGEMENT);
};



// ==============================================
// Une classe de Descriptor : 'OptDownyMildewUnderPressureClosing'    (de sous-classe : 'INT').
// ==============================================

OptDownyMildewUnderPressureClosing::OptDownyMildewUnderPressureClosing() {
	ClassName   ("optDownyMildewUnderPressureClosing");
	ClassComment   ("0: pea-sized; ...");
	ClassSymbol (OPT_DOWNY_MILDEW_UNDER_PRESSURE_CLOSING);
};



// ==============================================
// Une classe de Descriptor : 'OptExtraPhytoTreatmentsOP'    (de sous-classe : 'INT').
// ==============================================

OptExtraPhytoTreatmentsOP::OptExtraPhytoTreatmentsOP() {
	ClassName   ("optExtraPhytoTreatmentsOP");
	ClassComment   ("de 0 (pression nulle) a 3 (pression elevee)");
	ClassSymbol (OPT_EXTRA_PHYTO_TREATMENTS_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptFertilizationOP'    (de sous-classe : 'INT').
// ==============================================

OptFertilizationOP::OptFertilizationOP() {
	ClassName   ("optFertilizationOP");
	ClassComment   ("NONE ; END_HARVEST ; MID_OCT ; END_THIN ; END_THIN_CW ; EARLY_FEB ; EARLY_MAR");
	ClassSymbol (OPT_FERTILIZATION_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptFertilizationSeason'    (de sous-classe : 'INT').
// ==============================================

OptFertilizationSeason::OptFertilizationSeason() {
	ClassName   ("optFertilizationSeason");
	ClassComment   ("0 : no; 1 : Autumn; 2 : Spring; 3 : les deux");
	ClassSymbol (OPT_FERTILIZATION_SEASON);
};



// ==============================================
// Une classe de Descriptor : 'OptFirstSpringTillageOP'    (de sous-classe : 'INT').
// ==============================================

OptFirstSpringTillageOP::OptFirstSpringTillageOP() {
	ClassName   ("optFirstSpringTillageOP");
	ClassComment   ("INIT_SPRING_TILLAGE ; EARLY_MAR ; MID_MAR ; EARLY_APR ; MID_APR");
	ClassSymbol (OPT_FIRST_SPRING_TILLAGE_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptGrassStripAlleyWidthRatio'    (de sous-classe : 'FLOAT').
// ==============================================

OptGrassStripAlleyWidthRatio::OptGrassStripAlleyWidthRatio() {
	ClassName   ("optGrassStripAlleyWidthRatio");
	ClassComment   ("valeur de reference pour les GrassStripAlleyWidthRatio des parcelles");
	ClassUnity   ("%");
	ClassSymbol (OPT_GRASS_STRIP_ALLEY_WIDTH_RATIO);
};



// ==============================================
// Une classe de Descriptor : 'OptInitSpringTillageOP'    (de sous-classe : 'INT').
// ==============================================

OptInitSpringTillageOP::OptInitSpringTillageOP() {
	ClassName   ("optInitSpringTillageOP");
	ClassComment   ("MID_JAN ; EARLY_FEB ; MID_FEB ; END_THIN_FERT ; END_THIN_CW ; END_THIN_CW_FERT ; NONE");
	ClassSymbol (OPT_INIT_SPRING_TILLAGE_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptPastDownyMildewPressureThreshold'    (de sous-classe : 'FLOAT').
// ==============================================

OptPastDownyMildewPressureThreshold::OptPastDownyMildewPressureThreshold() {
	ClassName   ("optPastDownyMildewPressureThreshold");
	ClassComment   ("nivau d'attaque observe minimum pour continuer a traiter anti mildiou");
	ClassSymbol (OPT_PAST_DOWNY_MILDEW_PRESSURE_THRESHOLD);
};



// ==============================================
// Une classe de Descriptor : 'OptPhytoTreatmentCadence'    (de sous-classe : 'INT').
// ==============================================

OptPhytoTreatmentCadence::OptPhytoTreatmentCadence() {
	ClassName   ("optPhytoTreatmentCadence");
	ClassComment   ("delai de retour du prochain traitement phyto");
	ClassUnity   ("jour");
	ClassSymbol (OPT_PHYTO_TREATMENT_CADENCE);
};



// ==============================================
// Une classe de Descriptor : 'OptPhytoTreatmentRainAnticipation'    (de sous-classe : 'INT').
// ==============================================

OptPhytoTreatmentRainAnticipation::OptPhytoTreatmentRainAnticipation() {
	ClassName   ("optPhytoTreatmentRainAnticipation");
	ClassComment   ("on passe ... BEFORE : avant une pluie prevue ; AFTER : ... apres ...");
	ClassSymbol (OPT_PHYTO_TREATMENT_RAIN_ANTICIPATION);
};



// ==============================================
// Une classe de Descriptor : 'OptPlantationOP'    (de sous-classe : 'INT').
// ==============================================

OptPlantationOP::OptPlantationOP() {
	ClassName   ("optPlantationOP");
	ClassComment   ("MID_FEB ; MID_MAR ; LATE_MAR ; EARLY_APR");
	ClassSymbol (OPT_PLANTATION_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptPowderyMildewClosing'    (de sous-classe : 'INT').
// ==============================================

OptPowderyMildewClosing::OptPowderyMildewClosing() {
	ClassName   ("optPowderyMildewClosing");
	ClassComment   ("0:berry_touch ; 1:beg_ripening ; 2:end_ripening ; 3:august");
	ClassSymbol (OPT_POWDERY_MILDEW_CLOSING);
};



// ==============================================
// Une classe de Descriptor : 'OptPowderyMildewManagement'    (de sous-classe : 'INT').
// ==============================================

OptPowderyMildewManagement::OptPowderyMildewManagement() {
	ClassName   ("optPowderyMildewManagement");
	ClassComment   ("E1 ; E2 ; E3 ; G ; SG1 ; SG4");
	ClassSymbol (OPT_POWDERY_MILDEW_MANAGEMENT);
};



// ==============================================
// Une classe de Descriptor : 'OptPrePruningModality'    (de sous-classe : 'INT').
// ==============================================

OptPrePruningModality::OptPrePruningModality() {
	ClassName   ("optPrePruningModality");
	ClassComment   ("NONE ; ALL_PLOTS ; ALL_BUT_GOB ; LARGE ROYAT ; VIGOROUS ; ROYAT_VIGOROUS");
	ClassSymbol (OPT_PRE_PRUNING_MODALITY);
};



// ==============================================
// Une classe de Descriptor : 'OptPruningMinBegDate'    (de sous-classe : 'INT').
// ==============================================

OptPruningMinBegDate::OptPruningMinBegDate() {
	ClassName   ("optPruningMinBegDate");
	ClassComment   ("jj/mm/aaaa");
	ClassSymbol (OPT_PRUNING_MIN_BEG_DATE);
};



// ==============================================
// Une classe de Descriptor : 'OptPruningOP'    (de sous-classe : 'INT').
// ==============================================

OptPruningOP::OptPruningOP() {
	ClassName   ("optPruningOP");
	ClassComment   ("0:earlyNov ; 1:midNov ; 2:endNov ; 3:earlyDec");
	ClassSymbol (OPT_PRUNING_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptRegularPhytoTreatmentsOP'    (de sous-classe : 'INT').
// ==============================================

OptRegularPhytoTreatmentsOP::OptRegularPhytoTreatmentsOP() {
	ClassName   ("optRegularPhytoTreatmentsOP");
	ClassComment   ("de 1 (demarrage. precoce) a 6 (dem. tardif)");
	ClassSymbol (OPT_REGULAR_PHYTO_TREATMENTS_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptRidingOrientedInterRowProportion'    (de sous-classe : 'FLOAT').
// ==============================================

OptRidingOrientedInterRowProportion::OptRidingOrientedInterRowProportion() {
	ClassName   ("optRidingOrientedInterRowProportion");
	ClassComment   ("valeur de reference pour les RidingOrientedInterRowProportion des parcelles");
	ClassUnity   ("%");
	ClassSymbol (OPT_RIDING_ORIENTED_INTER_ROW_PROPORTION);
};



// ==============================================
// Une classe de Descriptor : 'OptShootThinningsOP'    (de sous-classe : 'INT').
// ==============================================

OptShootThinningsOP::OptShootThinningsOP() {
	ClassName   ("optShootThinningsOP");
	ClassComment   ("ONE_PLOT_PRUNED ; SURF_PRUNED_3 ; SURF_PRUNED_12 ; SURF_PRUNED_17 ; SURF_PRUNED_MAJ");
	ClassSymbol (OPT_SHOOT_THINNINGS_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayNumberAtBeginning'    (de sous-classe : 'INT').
// ==============================================

OptSprayNumberAtBeginning::OptSprayNumberAtBeginning() {
	ClassName   ("optSprayNumberAtBeginning");
	ClassComment   ("1 ou 2, typiquement");
	ClassSymbol (OPT_SPRAY_NUMBER_AT_BEGINNING);
	SetDefaultValue(0);
	SetToDefaultValue(0);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayPathAtBeginning'    (de sous-classe : 'FLOAT').
// ==============================================

OptSprayPathAtBeginning::OptSprayPathAtBeginning() {
	ClassName   ("optSprayPathAtBeginning");
	ClassComment   ("frequence de passage dans les IR au debut de l'iteration de traitements");
	ClassSymbol (OPT_SPRAY_PATH_AT_BEGINNING);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayPathRegular'    (de sous-classe : 'FLOAT').
// ==============================================

OptSprayPathRegular::OptSprayPathRegular() {
	ClassName   ("optSprayPathRegular");
	ClassComment   ("frequence de passage dans les IR pour les traitements dans le cas general");
	ClassSymbol (OPT_SPRAY_PATH_REGULAR);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayPathSinceFlowering'    (de sous-classe : 'FLOAT').
// ==============================================

OptSprayPathSinceFlowering::OptSprayPathSinceFlowering() {
	ClassName   ("optSprayPathSinceFlowering");
	ClassComment   ("frequence de passage dans les IR pour les traitements apres la floraison");
	ClassSymbol (OPT_SPRAY_PATH_SINCE_FLOWERING);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayerTypeAtBeginning'    (de sous-classe : 'INT').
// ==============================================

OptSprayerTypeAtBeginning::OptSprayerTypeAtBeginning() {
	ClassName   ("optSprayerTypeAtBeginning");
	ClassComment   ("cf. Enum_SprayerTypeOptions");
	ClassSymbol (OPT_SPRAYER_TYPE_AT_BEGINNING);
};



// ==============================================
// Une classe de Descriptor : 'OptSprayerTypeRegular'    (de sous-classe : 'INT').
// ==============================================

OptSprayerTypeRegular::OptSprayerTypeRegular() {
	ClassName   ("optSprayerTypeRegular");
	ClassComment   ("cf. Enum_SprayerTypeOptions");
	ClassSymbol (OPT_SPRAYER_TYPE_REGULAR);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringChemicalWeedingOP'    (de sous-classe : 'INT').
// ==============================================

OptSpringChemicalWeedingOP::OptSpringChemicalWeedingOP() {
	ClassName   ("optSpringChemicalWeedingOP");
	ClassComment   ("EARLY_JUN ; END_DEBR ; NONE");
	ClassSymbol (OPT_SPRING_CHEMICAL_WEEDING_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringFertMaxBegDate'    (de sous-classe : 'INT').
// ==============================================

OptSpringFertMaxBegDate::OptSpringFertMaxBegDate() {
	ClassName   ("optSpringFertMaxBegDate");
	ClassComment   ("jj/mm/aaaa");
	ClassSymbol (OPT_SPRING_FERT_MAX_BEG_DATE);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringFertMinBegDate'    (de sous-classe : 'INT').
// ==============================================

OptSpringFertMinBegDate::OptSpringFertMinBegDate() {
	ClassName   ("optSpringFertMinBegDate");
	ClassComment   ("jj/mm/aaaa");
	ClassSymbol (OPT_SPRING_FERT_MIN_BEG_DATE);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringTillagesIterationCP'    (de sous-classe : 'INT').
// ==============================================

OptSpringTillagesIterationCP::OptSpringTillagesIterationCP() {
	ClassName   ("optSpringTillagesIterationCP");
	ClassComment   ("EARLY_JUNE ; EARLY_JUL ; MID_JUL ; TEN_AUGUST");
	ClassSymbol (OPT_SPRING_TILLAGES_ITERATION_C_P);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringTillagesOP'    (de sous-classe : 'INT').
// ==============================================

OptSpringTillagesOP::OptSpringTillagesOP() {
	ClassName   ("optSpringTillagesOP");
	ClassComment   ("WEED_COVER ; OPTIMIZATION ; OPPORTUNITY");
	ClassSymbol (OPT_SPRING_TILLAGES_O_P);
};



// ==============================================
// Une classe de Descriptor : 'OptSpringTillagesOPScope'    (de sous-classe : 'INT').
// ==============================================

OptSpringTillagesOPScope::OptSpringTillagesOPScope() {
	ClassName   ("optSpringTillagesOPScope");
	ClassComment   ("FARM_SCOPE ; PLOT_SCOPE ; PLOT_GROUP_SCOPE");
	ClassSymbol (OPT_SPRING_TILLAGES_O_P_SCOPE);
};



// ==============================================
// Une classe de Descriptor : 'PathNameAttribute'    (de sous-classe : 'STRING').
// ==============================================

PathNameAttribute::PathNameAttribute() {
	ClassName   ("pathNameAttribute");
	ClassComment   ("chemin complet du fichier externe des donnees meteo de cette station");
	ClassSymbol (PATH_NAME_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'PestInfoMemoryAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

PestInfoMemoryAttribute::PestInfoMemoryAttribute() {
	ClassName   ("pestInfoMemoryAttribute");
	ClassComment   ("attribut de VineyardSet : memoire des avertissements maladie");
	ClassSymbol (PEST_INFO_MEMORY_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'PesticideSubtype'    (de sous-classe : 'STRING').
// ==============================================

PesticideSubtype::PesticideSubtype() {
	ClassName   ("pesticideSubtype");
	ClassComment   ("(preLevee, postLevee), (AntiMildiou, AntiOidium), ...");
	ClassSymbol (PESTICIDE_SUBTYPE);
};



// ==============================================
// Une classe de Descriptor : 'PesticideType'    (de sous-classe : 'STRING').
// ==============================================

PesticideType::PesticideType() {
	ClassName   ("pesticideType");
	ClassComment   ("herbicide, fongicide, nematicide, ...");
	ClassSymbol (PESTICIDE_TYPE);
	DescStringValueSpec* pDVS = new DescStringValueSpec(PESTICIDE_TYPE, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("herbicide"));
	pDVS->AddSubDomain(strdup("fongicide"));
};



// ==============================================
// Une classe de Descriptor : 'PhytoManagement'    (de sous-classe : 'STRING').
// ==============================================

PhytoManagement::PhytoManagement() {
	ClassName   ("phytoManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (PHYTO_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(PHYTO_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'PhytoTreatmentList'    (de sous-classe : 'STRING_TAB').
// ==============================================

PhytoTreatmentList::PhytoTreatmentList() {
	ClassName   ("phytoTreatmentList");
	ClassComment   ("elements : anti-oidium anti-mildiou");
	ClassSymbol (PHYTO_TREATMENT_LIST);
};



// ==============================================
// Une classe de Descriptor : 'PlaceName'    (de sous-classe : 'STRING').
// ==============================================

PlaceName::PlaceName() {
	ClassName   ("placeName");
	ClassComment   ("lieu dit ou est situe la parcelle");
	ClassSymbol (PLACE_NAME);
};



// ==============================================
// Une classe de Descriptor : 'PlantAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

PlantAttribute::PlantAttribute() {
	ClassName   ("plantAttribute");
	ClassComment   ("pointeur sur une instance de vegetation");
	ClassSymbol (PLANT_ATTRIBUTE);
	AddValueClassId(VEGETATION);
};



// ==============================================
// Une classe de Descriptor : 'PlantSpacing'    (de sous-classe : 'INT').
// ==============================================

PlantSpacing::PlantSpacing() {
	ClassName   ("plantSpacing");
	ClassComment   ("ecartement entre les plants de vignes (sur le rang)");
	ClassSymbol (PLANT_SPACING);
};



// ==============================================
// Une classe de Descriptor : 'PlotPattern'    (de sous-classe : 'P_ENTITY').
// ==============================================

PlotPattern::PlotPattern() {
	ClassName   ("plotPattern");
	ClassComment   ("instance de PlotInternalPatternOptionSubset");
	ClassSymbol (PLOT_PATTERN);
};



// ==============================================
// Une classe de Descriptor : 'PlotPatternOptionSetAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

PlotPatternOptionSetAttribute::PlotPatternOptionSetAttribute() {
	ClassName   ("plotPatternOptionSetAttribute");
	ClassSymbol (PLOT_PATTERN_OPTION_SET_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'PlotProfile'    (de sous-classe : 'DESCVALUESPEC_TAB').
// ==============================================

PlotProfile::PlotProfile() {
	ClassName   ("plotProfile");
	ClassComment   ("conjonction de specifications de domaines de valeurs");
	ClassSymbol (PLOT_PROFILE);
	SetToDefaultValue();
};



// ==============================================
// Une classe de Descriptor : 'PlotProfileOperator'    (de sous-classe : 'INT').
// ==============================================

PlotProfileOperator::PlotProfileOperator() {
	ClassName   ("plotProfileOperator");
	ClassComment   ("IN=1 ou NOT_IN=0");
	ClassSymbol (PLOT_PROFILE_OPERATOR);
	SetDefaultValue(1);
	SetToDefaultValue(1);
};



// ==============================================
// Une classe de Descriptor : 'PowderyMildewFlagShoot'    (de sous-classe : 'INT').
// ==============================================

PowderyMildewFlagShoot::PowderyMildewFlagShoot() {
	ClassName   ("powderyMildewFlagShoot");
	ClassComment   ("0=NON 1=OUI");
	ClassSymbol (POWDERY_MILDEW_FLAG_SHOOT);
};



// ==============================================
// Une classe de Descriptor : 'PowderyMildewLevel'    (de sous-classe : 'INT').
// ==============================================

PowderyMildewLevel::PowderyMildewLevel() {
	ClassName   ("powderyMildewLevel");
	ClassComment   ("0=no 1=scarse 2=frequNoDammage 3=freqAndDommage");
	ClassSymbol (POWDERY_MILDEW_LEVEL);
	DescIntValueSpec* pDVS = new DescIntValueSpec(POWDERY_MILDEW_LEVEL, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0, 3);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'PowderyMildewManagementAttribute'    (de sous-classe : 'INT').
// ==============================================

PowderyMildewManagementAttribute::PowderyMildewManagementAttribute() {
	ClassName   ("powderyMildewManagementAttribute");
	ClassComment   ("element de l'enum. PowderyMildewManagementModalities");
	ClassSymbol (POWDERY_MILDEW_MANAGEMENT_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'PowderyMildewToClose'    (de sous-classe : 'INT').
// ==============================================

PowderyMildewToClose::PowderyMildewToClose() {
	ClassName   ("powderyMildewToClose");
	ClassComment   ("vrai s'il ya lieu d'arreter les traitements anti-oidium");
	ClassSymbol (POWDERY_MILDEW_TO_CLOSE);
};



// ==============================================
// Une classe de Descriptor : 'PreviousReplicationRefDate'    (de sous-classe : 'INT').
// ==============================================

PreviousReplicationRefDate::PreviousReplicationRefDate() {
	ClassName   ("previousReplicationRefDate");
	ClassComment   ("dans une iteration, au-dela de la premiere, date de la precedente activite iteree");
	ClassSymbol (PREVIOUS_REPLICATION_REF_DATE);
};



// ==============================================
// Une classe de Descriptor : 'PruningMode'    (de sous-classe : 'STRING').
// ==============================================

PruningMode::PruningMode() {
	ClassName   ("pruningMode");
	ClassComment   ("R, G, GOB");
	ClassSymbol (PRUNING_MODE);
};



// ==============================================
// Une classe de Descriptor : 'PruningsManagement'    (de sous-classe : 'STRING').
// ==============================================

PruningsManagement::PruningsManagement() {
	ClassName   ("pruningsManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (PRUNINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(PRUNINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'RecordedActivityClassName'    (de sous-classe : 'STRING').
// ==============================================

RecordedActivityClassName::RecordedActivityClassName() {
	ClassName   ("recordedActivityClassName");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_ACTIVITY_CLASS_NAME);
};



// ==============================================
// Une classe de Descriptor : 'RecordedChantierClassName'    (de sous-classe : 'STRING').
// ==============================================

RecordedChantierClassName::RecordedChantierClassName() {
	ClassName   ("recordedChantierClassName");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_CHANTIER_CLASS_NAME);
};



// ==============================================
// Une classe de Descriptor : 'RecordedDay'    (de sous-classe : 'INT').
// ==============================================

RecordedDay::RecordedDay() {
	ClassName   ("recordedDay");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_DAY);
};



// ==============================================
// Une classe de Descriptor : 'RecordedInDayDuration'    (de sous-classe : 'INT').
// ==============================================

RecordedInDayDuration::RecordedInDayDuration() {
	ClassName   ("recordedInDayDuration");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_IN_DAY_DURATION);
};



// ==============================================
// Une classe de Descriptor : 'RecordedJulianDay'    (de sous-classe : 'INT').
// ==============================================

RecordedJulianDay::RecordedJulianDay() {
	ClassName   ("recordedJulianDay");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_JULIAN_DAY);
};



// ==============================================
// Une classe de Descriptor : 'RecordedMonth'    (de sous-classe : 'INT').
// ==============================================

RecordedMonth::RecordedMonth() {
	ClassName   ("recordedMonth");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_MONTH);
};



// ==============================================
// Une classe de Descriptor : 'RecordedOperationClassName'    (de sous-classe : 'STRING').
// ==============================================

RecordedOperationClassName::RecordedOperationClassName() {
	ClassName   ("recordedOperationClassName");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_OPERATION_CLASS_NAME);
};



// ==============================================
// Une classe de Descriptor : 'RecordedPerformerList'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

RecordedPerformerList::RecordedPerformerList() {
	ClassName   ("recordedPerformerList");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_PERFORMER_LIST);
};



// ==============================================
// Une classe de Descriptor : 'RecordedPhytoProductList'    (de sous-classe : 'STRING_TAB').
// ==============================================

RecordedPhytoProductList::RecordedPhytoProductList() {
	ClassName   ("recordedPhytoProductList");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_PHYTO_PRODUCT_LIST);
};



// ==============================================
// Une classe de Descriptor : 'RecordedPlotName'    (de sous-classe : 'STRING').
// ==============================================

RecordedPlotName::RecordedPlotName() {
	ClassName   ("recordedPlotName");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_PLOT_NAME);
};



// ==============================================
// Une classe de Descriptor : 'RecordedResourceList'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

RecordedResourceList::RecordedResourceList() {
	ClassName   ("recordedResourceList");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_RESOURCE_LIST);
};



// ==============================================
// Une classe de Descriptor : 'RecordedYear'    (de sous-classe : 'INT').
// ==============================================

RecordedYear::RecordedYear() {
	ClassName   ("recordedYear");
	ClassComment   ("attribut de OperationRecordedItem");
	ClassSymbol (RECORDED_YEAR);
};



// ==============================================
// Une classe de Descriptor : 'RegularPhytoTreatmentIndex'    (de sous-classe : 'INT').
// ==============================================

RegularPhytoTreatmentIndex::RegularPhytoTreatmentIndex() {
	ClassName   ("regularPhytoTreatmentIndex");
	ClassComment   ("numero d'ordre du chantier 'Regular' auquel est attache ce chantier 'Extra'");
	ClassSymbol (REGULAR_PHYTO_TREATMENT_INDEX);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'RelativeArea'    (de sous-classe : 'FLOAT').
// ==============================================

RelativeArea::RelativeArea() {
	ClassName   ("relativeArea");
	ClassComment   ("pour une rangee, surface relative p.r. a la parcelle");
	ClassSymbol (RELATIVE_AREA);
	DescFloatValueSpec* pDVS = new DescFloatValueSpec(RELATIVE_AREA, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0., 1.);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'RidingOrientedInterRowProportion'    (de sous-classe : 'FLOAT').
// ==============================================

RidingOrientedInterRowProportion::RidingOrientedInterRowProportion() {
	ClassName   ("ridingOrientedInterRowProportion");
	ClassComment   ("nombre d'inter-rangs avec fonction de passage / nombre d'inter-rangs");
	ClassUnity   ("%");
	ClassSymbol (RIDING_ORIENTED_INTER_ROW_PROPORTION);
};



// ==============================================
// Une classe de Descriptor : 'RowManagement'    (de sous-classe : 'INT').
// ==============================================

RowManagement::RowManagement() {
	ClassName   ("rowManagement");
	ClassComment   ("GRASS_COVERING, etc");
	ClassSymbol (ROW_MANAGEMENT);
};



// ==============================================
// Une classe de Descriptor : 'SeasonalEmploymentPlanAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

SeasonalEmploymentPlanAttribute::SeasonalEmploymentPlanAttribute() {
	ClassName   ("seasonalEmploymentPlanAttribute");
	ClassComment   ("l'instance de SeasonalEmploymentPlan attachee a une Timetable");
	ClassSymbol (SEASONAL_EMPLOYMENT_PLAN_ATTRIBUTE);
	AddValueClassId(SEASONAL_EMPLOYMENT_PLAN);
};



// ==============================================
// Une classe de Descriptor : 'ShootIntensity'    (de sous-classe : 'INT').
// ==============================================

ShootIntensity::ShootIntensity() {
	ClassName   ("shootIntensity");
	ClassComment   ("niveau de production des pampres");
	ClassSymbol (SHOOT_INTENSITY);
	DescIntValueSpec* pDVS = new DescIntValueSpec(SHOOT_INTENSITY, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(0, 2);
	pDVS->SortOnLowBound();
	pDVS->ReduceToDisjunction();
};



// ==============================================
// Une classe de Descriptor : 'ShootThinningsManagement'    (de sous-classe : 'STRING').
// ==============================================

ShootThinningsManagement::ShootThinningsManagement() {
	ClassName   ("shootThinningsManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (SHOOT_THINNINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(SHOOT_THINNINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'SkippedPlotsAttribute'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

SkippedPlotsAttribute::SkippedPlotsAttribute() {
	ClassName   ("skippedPlotsAttribute");
	ClassComment   ("les parcelles non traitees lors d'une iteration d'un chantier");
	ClassSymbol (SKIPPED_PLOTS_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'SoilHumidityVectorFromMhydas'    (de sous-classe : 'FLOAT_TAB').
// ==============================================

SoilHumidityVectorFromMhydas::SoilHumidityVectorFromMhydas() {
	ClassName   ("soilHumidityVectorFromMhydas");
	ClassComment   ("pour une Row, vecteur des humidites sur les unit. hydro. identifiees dans Mhydas");
	ClassSymbol (SOIL_HUMIDITY_VECTOR_FROM_MHYDAS);
};



// ==============================================
// Une classe de Descriptor : 'SoilManagementGroupName'    (de sous-classe : 'STRING').
// ==============================================

SoilManagementGroupName::SoilManagementGroupName() {
	ClassName   ("soilManagementGroupName");
	ClassComment   ("nom d'une instance de SoilManagementGroup, dscripteur de Plot");
	ClassSymbol (SOIL_MANAGEMENT_GROUP_NAME);
	SetToDefaultValue("");
};



// ==============================================
// Une classe de Descriptor : 'SoilManagementGroupSetName'    (de sous-classe : 'STRING').
// ==============================================

SoilManagementGroupSetName::SoilManagementGroupSetName() {
	ClassName   ("soilManagementGroupSetName");
	ClassComment   ("nom d'une instance de SoilManagementGroupSet, descripteur d'ARB");
	ClassSymbol (SOIL_MANAGEMENT_GROUP_SET_NAME);
};



// ==============================================
// Une classe de Descriptor : 'SoilManagementModality'    (de sous-classe : 'INT').
// ==============================================

SoilManagementModality::SoilManagementModality() {
	ClassName   ("soilManagementModality");
	ClassComment   ("symbole de la strategie d'entretien du sol d'un IR/R (Enum_SoilManagementModalities)");
	ClassSymbol (SOIL_MANAGEMENT_MODALITY);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'SoilManagementOption'    (de sous-classe : 'INT').
// ==============================================

SoilManagementOption::SoilManagementOption() {
	ClassName   ("soilManagementOption");
	ClassComment   ("symbole de la strategie d'entretien du sol d'un ARB (Enum_SoilManagementOptions)");
	ClassSymbol (SOIL_MANAGEMENT_OPTION);
};



// ==============================================
// Une classe de Descriptor : 'SoilType'    (de sous-classe : 'STRING').
// ==============================================

SoilType::SoilType() {
	ClassName   ("soilType");
	ClassComment   ("TR, TB, TN, calcosol");
	ClassSymbol (SOIL_TYPE);
};



// ==============================================
// Une classe de Descriptor : 'SpringChemicalWeedingsManagement'    (de sous-classe : 'STRING').
// ==============================================

SpringChemicalWeedingsManagement::SpringChemicalWeedingsManagement() {
	ClassName   ("springChemicalWeedingsManagement");
	ClassComment   ("nécessairement en série (SER) ou possiblement en parallèle (PAR) sur les parcelles");
	ClassSymbol (SPRING_CHEMICAL_WEEDINGS_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(SPRING_CHEMICAL_WEEDINGS_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'SpringTillagesManagement'    (de sous-classe : 'STRING').
// ==============================================

SpringTillagesManagement::SpringTillagesManagement() {
	ClassName   ("springTillagesManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (SPRING_TILLAGES_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(SPRING_TILLAGES_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'StageName'    (de sous-classe : 'STRING').
// ==============================================

StageName::StageName() {
	ClassName   ("stageName");
	ClassComment   ("nom du stade phenologique");
	ClassSymbol (STAGE_NAME);
};



// ==============================================
// Une classe de Descriptor : 'StageTab'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

StageTab::StageTab() {
	ClassName   ("stageTab");
	ClassComment   ("tableau de stades phenologiques");
	ClassSymbol (STAGE_TAB);
};



// ==============================================
// Une classe de Descriptor : 'StandardValuePriorityTab'    (de sous-classe : 'INT_TAB').
// ==============================================

StandardValuePriorityTab::StandardValuePriorityTab() {
	ClassName   ("standardValuePriorityTab");
	ClassComment   ("priorites d'allocation par phase pour une classe d'activites primitives");
	ClassSymbol (STANDARD_VALUE_PRIORITY_TAB);
};



// ==============================================
// Une classe de Descriptor : 'Stoniness'    (de sous-classe : 'STRING').
// ==============================================

Stoniness::Stoniness() {
	ClassName   ("stoniness");
	ClassComment   ("N, P, B");
	ClassSymbol (STONINESS);
};



// ==============================================
// Une classe de Descriptor : 'TerminatedPlotsAttribute'    (de sous-classe : 'ENTITY_TAB').
// ==============================================

TerminatedPlotsAttribute::TerminatedPlotsAttribute() {
	ClassName   ("terminatedPlotsAttribute");
	ClassComment   ("les parcelles terminees dans un chantier");
	ClassSymbol (TERMINATED_PLOTS_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'ThermalTimeFromBbkToAchieve'    (de sous-classe : 'FLOAT').
// ==============================================

ThermalTimeFromBbkToAchieve::ThermalTimeFromBbkToAchieve() {
	ClassName   ("thermalTimeFromBbkToAchieve");
	ClassComment   ("nombre thorique de deg.j entre le debourrement de la vigne et le stade X");
	ClassSymbol (THERMAL_TIME_FROM_BBK_TO_ACHIEVE);
};



// ==============================================
// Une classe de Descriptor : 'ThermalTimeToAchieve'    (de sous-classe : 'FLOAT').
// ==============================================

ThermalTimeToAchieve::ThermalTimeToAchieve() {
	ClassName   ("thermalTimeToAchieve");
	ClassComment   ("nombre theorique de degres.jour pour passer au stade suivant depuis le semis");
	ClassSymbol (THERMAL_TIME_TO_ACHIEVE);
};



// ==============================================
// Une classe de Descriptor : 'TimetableAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

TimetableAttribute::TimetableAttribute() {
	ClassName   ("timetableAttribute");
	ClassComment   ("instance de RecurrentAvailabilityTimetable attachee a un SeasonalEmploymentPlan");
	ClassSymbol (TIMETABLE_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'ToolClassName'    (de sous-classe : 'STRING').
// ==============================================

ToolClassName::ToolClassName() {
	ClassName   ("toolClassName");
	ClassComment   ("le nom de classe d'un materiel lie a l'entite decrite");
	ClassSymbol (TOOL_CLASS_NAME);
};



// ==============================================
// Une classe de Descriptor : 'TrellisMode'    (de sous-classe : 'STRING').
// ==============================================

TrellisMode::TrellisMode() {
	ClassName   ("trellisMode");
	ClassComment   ("1.1 , 1.1.2, 1.2.2");
	ClassSymbol (TRELLIS_MODE);
};



// ==============================================
// Une classe de Descriptor : 'Village'    (de sous-classe : 'STRING').
// ==============================================

Village::Village() {
	ClassName   ("village");
	ClassComment   ("commune ou est situee la parcelle");
	ClassSymbol (VILLAGE);
};



// ==============================================
// Une classe de Descriptor : 'VineyardAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

VineyardAttribute::VineyardAttribute() {
	ClassName   ("vineyardAttribute");
	ClassComment   ("descripteur d'un VineyardControlPlan pointant sur le Vineyard");
	ClassSymbol (VINEYARD_ATTRIBUTE);
};



// ==============================================
// Une classe de Descriptor : 'WeatherStationAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

WeatherStationAttribute::WeatherStationAttribute() {
	ClassName   ("weatherStationAttribute");
	ClassSymbol (WEATHER_STATION_ATTRIBUTE);
	AddValueClassId(WEATHER_STATION);
};



// ==============================================
// Une classe de Descriptor : 'WeekDayNumber'    (de sous-classe : 'INT').
// ==============================================

WeekDayNumber::WeekDayNumber() {
	ClassName   ("weekDayNumber");
	ClassComment   ("0 : dimanche, 1 : lundi, ... 6 : samedi pour un AvailabilityTimeSlot");
	ClassSymbol (WEEK_DAY_NUMBER);
};



// ==============================================
// Une classe de Descriptor : 'Width'    (de sous-classe : 'INT').
// ==============================================

Width::Width() {
	ClassName   ("width");
	ClassComment   ("largeur de quelque chose");
	ClassSymbol (WIDTH);
};



// ==============================================
// Une classe de Descriptor : 'WinterBeginningClock'    (de sous-classe : 'INT').
// ==============================================

WinterBeginningClock::WinterBeginningClock() {
	ClassName   ("winterBeginningClock");
	ClassComment   ("descripteur de WeatherStation ; valeur parametree");
	ClassSymbol (WINTER_BEGINNING_CLOCK);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'WinterBeginningClockForInitst'    (de sous-classe : 'INT').
// ==============================================

WinterBeginningClockForInitst::WinterBeginningClockForInitst() {
	ClassName   ("winterBeginningClockForInitst");
	ClassComment   ("descripteur de WeatherStation ; valeur parametree");
	ClassSymbol (WINTER_BEGINNING_CLOCK_FOR_INITST);
	SetDefaultValue(-9);
	SetToDefaultValue(-9);
};



// ==============================================
// Une classe de Descriptor : 'WinterTillagesManagement'    (de sous-classe : 'STRING').
// ==============================================

WinterTillagesManagement::WinterTillagesManagement() {
	ClassName   ("winterTillagesManagement");
	ClassComment   ("necessairement en serie (SER) ou possiblement en parallele (PAR) sur les parcelles");
	ClassSymbol (WINTER_TILLAGES_MANAGEMENT);
	DescStringValueSpec* pDVS = new DescStringValueSpec(WINTER_TILLAGES_MANAGEMENT, IN);
	SetDomain(pDVS);
	pDVS->AddSubDomain(strdup("SER"));
	pDVS->AddSubDomain(strdup("PAR"));
};



// ==============================================
// Une classe de Descriptor : 'WorkBeginningHour'    (de sous-classe : 'INT').
// ==============================================

WorkBeginningHour::WorkBeginningHour() {
	ClassName   ("workBeginningHour");
	ClassComment   ("heure a laquelle un premier travail a ete fait ce jour");
	ClassSymbol (WORK_BEGINNING_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayAfternoonBegHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayAfternoonBegHour::WorkDayAfternoonBegHour() {
	ClassName   ("workDayAfternoonBegHour");
	ClassComment   ("heure d'embauche l'apres-midi (14h15 : 14,25)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_AFTERNOON_BEG_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayAfternoonEndHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayAfternoonEndHour::WorkDayAfternoonEndHour() {
	ClassName   ("workDayAfternoonEndHour");
	ClassComment   ("heure de fin de travail l'apres-midi (17h : 17,0)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_AFTERNOON_END_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayAfternoonExtendedEndHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayAfternoonExtendedEndHour::WorkDayAfternoonExtendedEndHour() {
	ClassName   ("workDayAfternoonExtendedEndHour");
	ClassComment   ("heure maximale de fin de travail l'apres-midi (17h30 : 17,5)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_AFTERNOON_EXTENDED_END_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayMorningBegHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayMorningBegHour::WorkDayMorningBegHour() {
	ClassName   ("workDayMorningBegHour");
	ClassComment   ("heure d'embauche matinale (8h45 : 8,75)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_MORNING_BEG_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayMorningEndHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayMorningEndHour::WorkDayMorningEndHour() {
	ClassName   ("workDayMorningEndHour");
	ClassComment   ("heure de fin de travail en matinee (12h : 12,0)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_MORNING_END_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkDayMorningExtendedEndHour'    (de sous-classe : 'FLOAT').
// ==============================================

WorkDayMorningExtendedEndHour::WorkDayMorningExtendedEndHour() {
	ClassName   ("workDayMorningExtendedEndHour");
	ClassComment   ("heure maximale de fin de travail en matinee (12h30 : 12,5)");
	ClassUnity   ("heures decimales");
	ClassSymbol (WORK_DAY_MORNING_EXTENDED_END_HOUR);
};



// ==============================================
// Une classe de Descriptor : 'WorkOrganizationOptionSetAttribute'    (de sous-classe : 'P_ENTITY').
// ==============================================

WorkOrganizationOptionSetAttribute::WorkOrganizationOptionSetAttribute() {
	ClassName   ("workOrganizationOptionSetAttribute");
	ClassComment   ("pointeur sur l'objet encapsulant les options de conduite");
	ClassSymbol (WORK_ORGANIZATION_OPTION_SET_ATTRIBUTE);
	AddValueClassId(WORK_ORGANIZATION_OPTION_SET);
};



// ==============================================
// Une classe de Descriptor : 'WorkingTimeDailyUpperBound'    (de sous-classe : 'INT').
// ==============================================

WorkingTimeDailyUpperBound::WorkingTimeDailyUpperBound() {
	ClassName   ("workingTimeDailyUpperBound");
	ClassComment   ("nb d'heures max de travail par jour");
	ClassSymbol (WORKING_TIME_DAILY_UPPER_BOUND);
};



// ==============================================
// Une classe de Descriptor : 'WorkingTimeWeeklyUpperBound'    (de sous-classe : 'INT').
// ==============================================

WorkingTimeWeeklyUpperBound::WorkingTimeWeeklyUpperBound() {
	ClassName   ("workingTimeWeeklyUpperBound");
	ClassComment   ("nb d'heures max de travail par semaine");
	ClassSymbol (WORKING_TIME_WEEKLY_UPPER_BOUND);
};



// ==============================================
// Une classe de Descriptor : 'WorkingTimeYearlyUpperBound'    (de sous-classe : 'INT').
// ==============================================

WorkingTimeYearlyUpperBound::WorkingTimeYearlyUpperBound() {
	ClassName   ("workingTimeYearlyUpperBound");
	ClassComment   ("nb d'heures max de travail par an");
	ClassSymbol (WORKING_TIME_YEARLY_UPPER_BOUND);
};



// ==============================================
// Une classe de Descriptor : 'YearOfPlantation'    (de sous-classe : 'INT').
// ==============================================

YearOfPlantation::YearOfPlantation() {
	ClassName   ("yearOfPlantation");
	ClassComment   ("millesime (yyyy) de l'annee de plantation");
	ClassSymbol (YEAR_OF_PLANTATION);
};


