// ==============================================
// Fichier : UserContinuousProcess.cc  
// Type    : source C++
// 
// Description : constructeurs des classes des 'ContinuousProcess'.
// ==============================================

#include "UserContinuousProcess.h"

// ==============================================
// Une classe de Process : 'GrapevineAgeingProcess
// ==============================================

void grapevineAgeingProcess_initializeProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'initialisation du processus continu.
//---------------------------------------------
{
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pIsoAgeingEntity = pCP->ProcessedEntity();
  //
  // etablissement du jour julien courant
  //
  int currentClock = pCurrentSim->Clock();
  tm* pTm = ClockValueToTm(currentClock);
  int currentJulianDay = pTm->tm_yday+1;
  delete pTm;
  //
  // parcours des stades et arret sur le premier dont le jour d'atteinte pleine
  // est superieur au jour julien courant
  //
  int initialStageIndex = 0;
  Entity* pAge = pIsoAgeingEntity->GetEntityConstValue(AGE_ATTRIBUTE);
  pEntityTab* pStageTab = pAge->GetEntityTabConstValue(STAGE_TAB);
  int nbStages = pStageTab->size();
  for(int k=0;k<nbStages;k++) {
    Entity* pStage_k = (Entity*)pStageTab->get_element(k);
    int achievementDay_k = pStage_k->GetIntConstValue(DAY_OF_ACHIEVEMENT);

    if( achievementDay_k <= currentJulianDay) {
      initialStageIndex++;
    }
    else {
      break;
    }
  }
  int firstStage = initialStageIndex-1;
  if(initialStageIndex==0) {
    firstStage = nbStages-1;
  }
  pAge->SetIntVarValue(CURRENT_STAGE_INDEX, firstStage);
};

void grapevineAgeingProcess_goOneStepForwardProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'avancee du processus continu.
//---------------------------------------------
{
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pVegetation = pCP->ProcessedEntity();
  //
  // etablissement du jour julien courant
  //
  int currentClock = pCurrentSim->Clock();
  tm* pTm = ClockValueToTm(currentClock);
  int currentJulianDay = pTm->tm_yday;
  //
  // examen du stade courant ...
  //
  Entity* pAge = pVegetation->GetEntityConstValue(AGE_ATTRIBUTE);
  pEntityTab* pStageTab = pAge->GetEntityTabConstValue(STAGE_TAB);

  int currentIndex = pAge->GetIntVarValue(CURRENT_STAGE_INDEX);
  Entity* pCurrentStage = (Entity*)pStageTab->get_element(currentIndex);
  int jourAtteinteCurrentStage = pCurrentStage->GetIntConstValue(DAY_OF_ACHIEVEMENT);
//printf("\njourAtteinteCurrentStage = %d", jourAtteinteCurrentStage);

  int nextStageIndex = currentIndex + 1;
  if(currentIndex == pStageTab->size()-1) {
    nextStageIndex = 0;
  }
  Entity* pNextStage = (Entity*)pStageTab->get_element(nextStageIndex);
  int jourAtteinteNextStage = pNextStage->GetIntConstValue(DAY_OF_ACHIEVEMENT);
//printf("\njourAtteinteNextStage    = %d", jourAtteinteNextStage);
  //
  // culture perenne : retour au premier stade si le dernier est atteint
  //
  int seuilAtteinte = jourAtteinteNextStage;
  if(currentJulianDay > jourAtteinteNextStage)
    seuilAtteinte = jourAtteinteNextStage + 366;
  //
  // recherche de la somme de degres-jours a atteindre quand on est : 
  //  au stade FLO, VER ou MAT 
  // et recherche de la somme de degres-jours actuelle
  //

  if((currentIndex >= 0) && (currentIndex<3)){

    float thermalTimeFromBbkToAchieve = pNextStage->GetFloatConstValue(THERMAL_TIME_FROM_BBK_TO_ACHIEVE);
    float currentThermalTimeFromBbk = pAge->GetFloatVarValue(THERMAL_TIME_FROM_BBK);

    if((currentIndex == 2) 
       && 
       (currentJulianDay>335)){
      pAge->SetIntVarValue(CURRENT_STAGE_INDEX, nextStageIndex);
//printf("\n nextStageIndex= %d", nextStageIndex);
    }
    if(currentThermalTimeFromBbk > thermalTimeFromBbkToAchieve) {
      pAge->SetIntVarValue(CURRENT_STAGE_INDEX, nextStageIndex);
    }
  }
  //
  // ou si stade maturite, stade recolte atteint quand j = jMat+1
  //
  else if(currentIndex == 3) {
    int dayWhenAchieved = pCurrentStage->GetIntVarValue(DAY_WHEN_ACHIEVED);
    if(currentJulianDay == dayWhenAchieved + 1){
//printf("\n currentJulianDay = %d dayWhenAchieved =%d", currentJulianDay,dayWhenAchieved);
	pAge->SetIntVarValue(CURRENT_STAGE_INDEX, nextStageIndex);
    }
  }
  //
  // ... ou changement de stade si sa date d'atteinte pleine est depassee
  // ou si somme degres-jour atteinte
  //

//printf("\ncurrentJulianDay = %d seuil = %d (jourAtteinteNextStage = %d)\n", currentJulianDay, seuilAtteinte, jourAtteinteNextStage);
  else if((currentIndex == 4)
	  &&
	  (currentJulianDay>jourAtteinteNextStage)) {
    int dayWhenAchieved = pCurrentStage->GetIntVarValue(DAY_WHEN_ACHIEVED);
    if(currentJulianDay == dayWhenAchieved + 1){
	pAge->SetIntVarValue(CURRENT_STAGE_INDEX, nextStageIndex);
    }
  }
  else if(currentJulianDay >= seuilAtteinte) {
    pAge->SetIntVarValue(CURRENT_STAGE_INDEX, nextStageIndex);
    

//printf("\n%d/%d/%d (jour %d) passage du stade '%s' au stade '%s'",
//       pTm->tm_mday, pTm->tm_mon+1, pTm->tm_year+1900, currentJulianDay,
//       pCurrentStage->GetStringConstValue(STAGE_NAME),
//       pNextStage->GetStringConstValue(STAGE_NAME));
  }
  delete pTm;
};


// ==============================================

GrapevineAgeingProcess::GrapevineAgeingProcess() : ContinuousProcess(GRAPEVINE_AGEING_PROCESS, "grapevineAgeingProcess") {
  Step(1);
  ProcessedEntityClassId(ISO_AGEING_ENTITY);
  SetInitialize(grapevineAgeingProcess_initializeProcess_Body);
  SetGoOneStepForward(grapevineAgeingProcess_goOneStepForwardProcess_Body);
};


// ==============================================
// Une classe de Process : 'HourlyWeatherFileReading
// ==============================================

void hourlyWeatherFileReading_initializeProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'initialisation 
// du processus continu de lecture d'un fichier sequentiel.
//---------------------------------------------
{
  ReadBufferizedDataFileProcess* pCP = (ReadBufferizedDataFileProcess*)(pM->DescribedContinuousProcess());

  UserFileId fileClassId = pCP->FileClassId();
  BufferizedDataFile* pF =
    (BufferizedDataFile*)App_NewSequentialDataFileFromId(fileClassId);
  if(! pF)
    UnknownClassSymbol("BufferizedDataFile", fileClassId);
  pCP->SetFile(pF);

  //-------------------------------------------
  // installation tableau Memoire
  //-------------------------------------------
  int M = pF->BFSetMemoryScope(fileClassId);
  pF->SetMemoryTab(new Tableau<char*>());
  for(int k=0;k<M;k++) {
    pF->AddMemoryTabItem(strdup(""));
  }
  pF->SetMemoryCurrentRecordIndex(-1);

  //-------------------------------------------
  // installation tableau Prediction
  //-------------------------------------------
  int P = pF->BFSetPredictionScope(fileClassId);
  pF->SetPredictionTab(new Tableau<char*>());
  for(int k=0;k<P;k++) {
    pF->AddPredictionTabItem(strdup(""));
  }
  pF->SetPredictionCurrentRecordIndex(-1);

  //-------------------------------------------
  // recuperation des specifications du fichier
  //-------------------------------------------
  pCP->SetNamesFromFile(pF);
  //pCP->DisplayNames();

  //-------------------------------------------
  // etablissement du chemin complet du fichier
  // a partir des specifications du fichier et du processus
  //-------------------------------------------
  pCP->AssignFullNameToFile(pF);

  //-------------------------------------------
  // attachement fichier externe <-> station meteo
  //-------------------------------------------
  Entity* pWeatherStation = pCP->ProcessedEntity();
  char* fullPathName = pF->GetFullName();
//printf("\nhourlyWeatherFileReading_init fullPathName : '%s'\n", fullPathName);
  pWeatherStation->SetStringConstValue(PATH_NAME_ATTRIBUTE, fullPathName);

  // debut ajout JPR 130306 (5.5)
  Date* dateDebutHiver = GetDateParameterValue("debutCumulPluie", "declenchementLabour");
  dateDebutHiver->year = pCurrentSim->BeginningDate()->year;
  tm* tmDebutHiver = StructDateToTm(dateDebutHiver);
  int clockDebutHiver = StructTmToClockValue(tmDebutHiver); 
  pWeatherStation->SetIntConstValue(WINTER_BEGINNING_CLOCK, clockDebutHiver);
  delete tmDebutHiver;

  // debut ajout JPR 130425 (5.6)
  dateDebutHiver = GetDateParameterValue("debutCumulPluie", "declenchementLabourInit");
  dateDebutHiver->year = pCurrentSim->BeginningDate()->year;
  tmDebutHiver = StructDateToTm(dateDebutHiver);
  clockDebutHiver = StructTmToClockValue(tmDebutHiver); 
  pWeatherStation->SetIntConstValue(WINTER_BEGINNING_CLOCK_FOR_INITST, clockDebutHiver);
  delete tmDebutHiver;

  //---------------------
  // ouverture du fichier
  //---------------------
  pF->Open();

  //---------------------
  // saut des premieres lignes d'entete
  //---------------------
  pF->Skip(pF->GetNbTopCommentLines());

  //---------------------
  // positionnement sur le premier jour de la simulation
  //---------------------
  Date* dateDebutSim = pCurrentSim->BeginningDate();
  tm* pTm = StructDateToTm(dateDebutSim);
  //
  // transformation en date UTC, format en vigueur sur pF
  //
  //                                      début modif JPR 130206
  time_t time_begSim = mktime(pTm);
  tm* pTm_utc = gmtime(&time_begSim);
  pTm_utc->tm_hour--;
  Date* dateArretLecture =  StructTmToDate(pTm_utc);
//StructDateToChar(dateArretLecture);  printf("\ndateArretLecture : %s", gTimeString);
  //delete pTm_utc;  // Seg.Flt
  delete pTm;
  //                                      fin modif JPR 130206
  //
  // parcours des lignes et arret juste avant la premiere a considerer
  //
  char str_tmp[100];
  int i;
  char* line;
  Boolean onContinue = TRUE;
  while(onContinue) {
	line = fgets(str_tmp, 90, pF->GetPhysicalFile());
	i = sscanf (str_tmp,
				pF->GetFormat()
				, pF->mpFieldTab[FLD_DATE]->mFieldValue.stringValue
				, pF->mpFieldTab[FLD_HOUR]->mFieldValue.stringValue
                , &pF->mpFieldTab[FLD_RAINFALL]->mFieldValue.floatValue
				);
	if((i != EOF) && (i < pF->mpFieldTab.size()))
	  BadReadFile(pF);
	// ligne : 2004-01-01 19:00:00 0.5
	char* date = pF->GetStringFieldValue(FLD_DATE);
	//         2004-01-01
	char* heure = pF->GetStringFieldValue(FLD_HOUR);
	//                    19:00:00

	Date* dateRef = App_WeatherFileDateToDate(date, heure);
	
	if((*dateRef == *dateArretLecture)
	   ||
	   (line == NULL)) {
	  onContinue = FALSE;
	}
	delete dateRef;
  }
  delete dateArretLecture;

  //---------------------
  // Lecture des P premiers enregistrements simulant la prediction initiale
  //---------------------
  int predictionCurrentIndex = pF->GetPredictionCurrentRecordIndex();
  for(int k=0;k<P;k++) {
    //
    // lecture physique enregistrement sur fichier externe
    //
    char str[101] = "\0";
    int c, i=-1, onContinue = 1;
    while(onContinue) {
      c = fgetc(pF->GetPhysicalFile());
	  if(c == EOF) {
		if(! pF->IsEndOfFileReached()) {
		  if(gMyTraceWarnings)
			printf("\n!! EOF REACHED DURING INITIALIZATION : hourlyWeatherFileReading !!");
		  pF->IsEndOfFileReached(TRUE);
		}
		break;
	  }
	  else {
		if(c != 13) {                 // CR
		  if(c == 10) onContinue = 0; // LF
		  else str[++i] = (char)c;
		}
	  }
    }
    str[++i] = '\0';
    //
    // ... et ajout a la premiere place libre dans le tableau des predictions
    //
    int indexToLoad = ++predictionCurrentIndex;
    pF->SetPredictionTabItem(indexToLoad, strdup(str));

  }

  //---------------------
  // dans les predictions, on pointe toujours sur la plus lointaine
  //---------------------
  pF->SetPredictionCurrentRecordIndex(predictionCurrentIndex);
//pF->DisplayMemoryTab();
//pF->DisplayPredictionTab();
};

void hourlyWeatherFileReading_postReadProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'exploitation des valeurs
// lues sur un enregistrement.
//---------------------------------------------
{
  //
  // 1) cumul pluie depuis interaction Mhydas
  // 2) cumul pluie depuis dernier tillage (RAZ dans AchievementDegreeMonitor)
  // 3) cumul pluie depuis debut hiver (130308, 5.5) 
  // 3bis) cumul pluie depuis debut hiver pour InitST (130425, 5.6) 
  //
  ReadBufferizedDataFileProcess* pCP 
	= (ReadBufferizedDataFileProcess*)(pM->DescribedContinuousProcess());
  Entity* pWS = pCP->ProcessedEntity();
  BufferizedDataFile* pF = (BufferizedDataFile*)pCP->GetFile();

  // ligne : 2004-01-01 19:00:00 0.5
  char* date = pF->GetStringFieldValue(FLD_DATE);
  //         2004-01-01
  char* heure = pF->GetStringFieldValue(FLD_HOUR);
  //                    19:00:00
  float rain_i = pF->GetFloatFieldValue(FLD_RAINFALL);
  //                     0.5
  pWS->SetFloatValue(RAINFALL, rain_i);
//printf("\ndateLue     : '%s' '%s' rain = %f", date, heure, rain_i);

  //
  // Traduction heure UTC -> heure locale
  //
  char* tmpDate = new char[50];
  sprintf(tmpDate, "%s", date);
  int y = atoi(strtok(tmpDate,"-"));
  int m = atoi(strtok(NULL,"-"));
  int d = atoi(strtok(NULL,"-"));
  char* tmpHeure = new char[50];
  sprintf(tmpHeure, "%s", heure);
  int h = atoi(strtok(tmpHeure,":"));
  int n = atoi(strtok(NULL,":"));
  int s = atoi(strtok(NULL,":"));
  delete[] tmpDate;
  delete[] tmpHeure;

  time_t absT = UTCDateToAbsoluteTime(d, m, y, h, n, s); // modif JPR 130306
  //
  // transformation en date locale, langage de la décision
  //
  tm* pTm = new tm();
  localtime_r(&absT, pTm);
  StructTmToStringISO(pTm);
//printf("\n----pTm_local : '%s' timezone = %d tzname : '%s' '%s' daylight = %d", gTimeString, timezone, tzname[0], tzname[1], daylight);
  int wday = pTm->tm_wday;
  
  int now = pCurrentSim->Clock(); // modif JPR 130306
  int clock = now; // modif JPR 130306

  Entity* pWSSet = pWS->GetSuperSet();
  if(pWSSet->GetIntConstValue(LAST_WEATHER_PRINT_CLOCK) != clock) {
	  printf("\n---- le %s %s (clock = %d) ---- rain %s : %4.1f", ShortWeekDayNameToCharTab[wday], gTimeString, clock, pWS->InstanceName(), rain_i);
	  pWSSet->SetIntConstValue(LAST_WEATHER_PRINT_CLOCK, clock);
  }
  else
	printf(" %s : %4.1f", pWS->InstanceName(), rain_i);

  //
  // tache 2) cumul pluie depuis depuis dernier tillage
  //       3) cumul pluie depuis debut hiver
  //
  pEntityTab* pPlotForThisWS = new pEntityTab();
  Entity* pSystem = pWSSet->GetSuperEntity();
  Entity* pVineyardSet = pSystem->GetComponent(VINEYARD_SET);
  int nbEA = pVineyardSet->GetNumberOfElements();
  for(int i=0;i<nbEA;i++) {
	Entity* pEA_i = pVineyardSet->GetElement(i);
	pEntityTab pPlotList = pEA_i->GetElementTab();
	int nbPlots = pPlotList.size();
	for(int j=0;j<nbPlots;j++) {
	  Entity* pPlot_j = (Entity*)pPlotList[j];
	  Entity* pWS_j = pPlot_j->GetEntityConstValue(WEATHER_STATION_ATTRIBUTE);
	  if(pWS_j == pWS) {
		pPlotForThisWS->push_back(pPlot_j);
		//
		// tache 2)
		//
		float oldValue = 
		  pPlot_j->GetFloatVarValue(CUMULATIVE_RAINFALL_FROM_LAST_TILLAGE);
		float newValue = oldValue + rain_i;
//printf("\nplot '%s' new cumul pluie depuis dernier tillage = %f", pPlot_j->InstanceName(), newValue);
		pPlot_j->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_LAST_TILLAGE,
								  newValue);
		//
		// tache 3)
		//
		int clockDebutHiver = pWS_j->GetIntConstValue(WINTER_BEGINNING_CLOCK);
//printf("\nplot '%s' clockDebutHiver = %d", pPlot_j->InstanceName(), clockDebutHiver);
		if(now >= clockDebutHiver) {
		  oldValue = 
			pPlot_j->GetFloatVarValue(CUMULATIVE_RAINFALL_FROM_EARLY_WINTER);
		  newValue = oldValue + rain_i;
//printf("\nplot '%s' new cumul pluie depuis debut hiver = %f", pPlot_j->InstanceName(), newValue);
		  pPlot_j->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_EARLY_WINTER,
									newValue);
		}
		//
		// tache 3bis) JPR 130425 (5.6)
		//
		int clockDebutHiverPourInit = pWS_j->GetIntConstValue(WINTER_BEGINNING_CLOCK_FOR_INITST);
//printf("\nplot '%s' clockDebutHiver = %d", pPlot_j->InstanceName(), clockDebutHiverPourInit);
		if(now >= clockDebutHiverPourInit) {
		  oldValue = 
			pPlot_j->GetFloatVarValue(CUMULATIVE_RAINFALL_FROM_EARLY_WINTER_FOR_INITST);
		  newValue = oldValue + rain_i;
//printf("\nplot '%s' new cumul pluie depuis debut hiver pour initst = %f", pPlot_j->InstanceName(), newValue);
		  pPlot_j->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_EARLY_WINTER_FOR_INITST,
									newValue);
		}
	  }
	}
  }
  pTm->tm_hour = 21;
  mktime(pTm);
  int julianDay = pTm->tm_yday;
  int leSoir = StructTmToClockValue(pTm);
  delete pTm;
  //
  // recherche dans l'agenda d'un event MhydasInteraction
  //
  Event* pMhydasInteractionEvent = NULL;
  pCurrentSim->GoHeadInAgenda();
  while(pCurrentSim->IsCurrentInAgenda()) {
	Event* pEvent = pCurrentSim->GetCurrentInAgenda();
	if(! strcmp(pEvent->ClassName(), "morningMhydasInteractionEvent")) {
	  pMhydasInteractionEvent = pEvent;
	  break;
	}
	pCurrentSim->GoNextInAgenda();
  };
  if(pMhydasInteractionEvent) {
	//
	// cumul pluie depuis la derniere interaction Mhydas 
	// et debrayage apres 21h jusqu'a l'interaction du matin
	//
	int nextInteractionClock = pMhydasInteractionEvent->OccurrenceClockTime();
//printf("\n\tnextInteractionClock = %d   leSoir = %d", nextInteractionClock, leSoir);
	tm* pTm_interaction = ClockValueToTm(nextInteractionClock);
	int julianDay_interaction = pTm_interaction->tm_yday;
	delete pTm_interaction;
	bool nextInteractionCeJour = FALSE;
	if(julianDay_interaction == julianDay)
	  nextInteractionCeJour = TRUE;
	int nbPlots = pPlotForThisWS->size();
	for(int j=0;j<nbPlots;j++) {
	  Entity* pPlot_j = (Entity*)pPlotForThisWS->get_element(j);
	  if((clock > leSoir)
		 ||
		 ((clock < nextInteractionClock)
		  &&
		  nextInteractionCeJour)) {
		// no action
	  }
	  else {
		float oldValue = 
		  pPlot_j->GetFloatVarValue(CUMULATIVE_RAINFALL_FROM_MHYDAS_INTERACTION);
		float newValue = oldValue + rain_i;
//printf("\nplot '%s' new cumul pluie depuis Mhydas = %f", pPlot_j->InstanceName(), newValue);
		pPlot_j->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_MHYDAS_INTERACTION,
								  newValue);
	  }
	}
  }
  delete pPlotForThisWS;
};


// ==============================================

HourlyWeatherFileReading::HourlyWeatherFileReading() : ReadBufferizedDataFileProcess() {
  ClassSymbol(HOURLY_WEATHER_FILE_READING);
  ClassName("hourlyWeatherFileReading");
  ProcessedEntityClassId(BD_ENTITY);
  Step(1);
  FileClassId(HOURLY_WEATHER_FILE);
  SetInitialize(hourlyWeatherFileReading_initializeProcess_Body);
  SetPostRead(hourlyWeatherFileReading_postReadProcess_Body);
};


