// ==============================================
// Fichier : UserDiscreteProcess.cc  
// Type    : source C++
// 
// Description : constructeurs des classes des 'DiscreteProcess'.
// ==============================================
  
#include "UserDiscreteProcess.h"
  
// ==============================================
// Une classe de Process : 'ClearPerfAllocationAndUpdateSituation
// ==============================================

void clearPerfAllocationAndUpdateSituation_execProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'execution du processus ponctuel.
//---------------------------------------------
// La validité de la spécification SAME_PREVIOUS_ENTITIES sur les spéc. de
// performers des activités primitives est limitée à la demi-journée. 
// Autrement dit, qd SAME_PREVIOUS_ENTITIES est spécifié, les performers
// doivent rester les mêmes jusqu'à la fin de la demi-journée en cours, mais
// pourront changer dans la prochaine.
// Il faut donc, lors des débuts des demi-journées, effacer la trace des
// performers alloués lors de la demi-journée précédente. C'est le descr.
// developpedList de la spéc.de performers de l'activité. (ajout 121211 5.1)
//---------------------------------------------
{
  int now = pCurrentSim->Clock();
  tm* pTm_now = ClockValueToTm(now);
  int hour = pTm_now->tm_hour;
  int mon = pTm_now->tm_mon;
  int day = pTm_now->tm_mday;
  delete pTm_now;
  //
  // la date a atteindre pour entrer dans la periode speciale
  bool clearToDo = FALSE;
  bool inSpecialHours = FALSE;
  if(((mon == 3) && (day == 30))  // 30 avril -> specialHour pour le 1er mai
	 ||
	 ((mon > 3) && (mon < 7))     // mai, ... ,juillet
	 ||
	 (mon == 7) && (day < 31)) {  // 31 aout -> normalHour pour 1er sept
	inSpecialHours = TRUE;
  }
  if((inSpecialHours
	  &&
	  (hour == gIntParam_morningUpdateSituation_specialHour_))
	 ||
	 (hour == gIntParam_morningUpdateSituation_normalHour_)
	 ||
	 (hour == gIntParam_afternoonUpdateSituation_normalHour_))
	clearToDo = TRUE;
	  
  if(clearToDo) {
	DiscreteProcess* pCP = pM->DescribedDiscreteProcess();
	Manager* pManager = (Manager*)(pCP->ProcessedEntity());
	Strategy* pStrategy = (Strategy*)(pManager->GetComponent(STRATEGY));
	Entity* 
	  pArbSet = pStrategy->GetComponent(ACTIVITIES_RESOURCES_BLOCK_SET);
	int nbArb = pArbSet->GetNumberOfElements();
//printf("\n>> t=%d nbArb = %d\n", now, nbArb);
	for (int k=0;k<nbArb;k++) {
	  ActivitiesResourcesBlock* 
		pArb = (ActivitiesResourcesBlock*)(pArbSet->GetElement(k));
//printf("\n>> t=%d nbArb = %d pArb '%s'\n", now, nbArb, pArb->InstanceName());
	  Activity* 
		pAct = (Activity*)(pArb->GetComponent(ACTIVITY));
	  pEntityTab tmp = pAct->GetDepthElementTab(PRIMITIVE_ACTIVITY);
//printf("\ntmp :: ");tmp.display_names();
	  pEntityTab* tmp2 = new pEntityTab();
	  int nbInst = tmp.size();
//printf("\tnbInst = %d", nbInst);
	  for (int k=0;k<nbInst;k++) {
		Entity* pInst = (Entity*)tmp[k];
		if(pInst->GetIntVarValue(SITUATION) == IN_COURSE) {
		  Entity* perfSpec 
			= pInst->GetEntityVarValue(PERFORMER_SPEC_ATTRIBUTE);
		  if(perfSpec) {
			pEntityTab* devList = perfSpec->GetEntityTabVarValue(DEVELOPPED_LIST);
//devList->display_names(); 
			int nbElem = devList->size();
			for(int m=nbElem-1;m>=0;m--) {
			  BasicEntity* pElem_m_tmp = devList->get_nth(m);
			  Entity* pElem_m = (Entity*)pElem_m_tmp;
			  devList->remove(m);
			  DeleteEntity(pElem_m);
			}
//devList->display_names(); 
		  }
		  tmp2->push_back(pInst);
		}
	  }
//printf("\ntmp2 :: ");tmp2->display_names();
	  delete tmp2; // + JPR 130114 <- MLK
	}
	//
	// ajout: RAZ PrimitiveActivityAttribute des performers JPR 130226 
	//
	pEntityTab* perfList = GetEntityList(FARM_WORKER);
//printf("\ndans clearPerfAllocationAndUpdateSituation_execProcess_Body perfPool : ");perfList->display_names();
	int nbPerf = perfList->size();
	for(int i=0;i<nbPerf;i++) {
	  Entity* pPerf_i = (Entity*)perfList->get_nth(i);
	  pPerf_i->SetEntityConstValue(PRIMITIVE_ACTIVITY_ATTRIBUTE, NULL);
	}	
  }
  //
  // corps prédéfini de l'event 
  //
  updateSituationProcess_execProcess_Body(pM);
};


// ==============================================

ClearPerfAllocationAndUpdateSituation::ClearPerfAllocationAndUpdateSituation() : DiscreteProcess(CLEAR_PERF_ALLOCATION_AND_UPDATE_SITUATION, "clearPerfAllocationAndUpdateSituation") {
  BasicEntity* pObj = GetFirstBasicEntity(MANAGER);
  if(! pObj) exit(0);
  ProcessedEntity(pObj);
  SetExec(clearPerfAllocationAndUpdateSituation_execProcess_Body);
};


// ==============================================
// Une classe de Process : 'MemoryItemGarbagingProcess
// ==============================================

void memoryItemGarbagingProcess_execProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'execution du processus ponctuel.
//---------------------------------------------
{
  //DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  //Entity* pVineyardSet = pDP->ProcessedEntity();

  //
  // Les informations relatives � un episode de simulation, et envoyees a 
  // Mhydas/OpenFluid dans le tableau 'OUT' (argument de RunOpenSimulation),
  // sont supposees etre captees (et exploitees) opportunement par Mhydas.
  // Au debut de l'episode suivant, et si ces informations pointaient sur
  // des instances d'objets non permanents (et non seulement des nombres), 
  // alors la memoire allouee a ces instances peut etre recuperee.
  // Dans Dhivine, c'est le cas des LastOperationsMemoryItem's, qui sont
  // les elements de la valeur du descripteur LastOperationsMemoryAttribute
  // de la classe Row.
  // Pour memoire, les InterfaceDataItem's du tableau 'OUT' ont :
  // . le nom d'une Row comme valeur de EntityNameAttribute, 
  // . LAST_OPERATIONS_MEMORY_ATTRIBUTE comme valeur du DescriptorIdAttribute.
  //

  pEntityTab* pMemoryItemTab = GetEntityList(LAST_OPERATIONS_MEMORY_ITEM);
//printf("\nt = %d entree memoryItemGarbagingProcess_exec MemoryItemTab : ", pCurrentSim->Clock()); pMemoryItemTab->display_names();
  
  int nbItems = pMemoryItemTab->size();
  for(int i=nbItems-1;i>=0;i--) {
	Entity* pItem = (Entity*)pMemoryItemTab->get_element(i);
	DeleteEntity(pItem);
  }
};


// ==============================================

MemoryItemGarbagingProcess::MemoryItemGarbagingProcess() : DiscreteProcess(MEMORY_ITEM_GARBAGING_PROCESS, "memoryItemGarbagingProcess") {
  ProcessedEntityClassId(BD_ENTITY);
  SetExec(memoryItemGarbagingProcess_execProcess_Body);
};


// ==============================================
// Une classe de Process : 'MildewWarningProcess
// ==============================================

void mildewWarningProcess_execProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'execution du processus ponctuel.
//---------------------------------------------
{
  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  Entity* pVineyardSet = pDP->ProcessedEntity();
  int nbEA = pVineyardSet->GetNumberOfElements();
  for(int i=0;i<nbEA;i++) {
	Entity* pEA_i = pVineyardSet->GetElement(i);
	Entity* pConduite_i = pEA_i->GetEntityConstValue(CONTROL_PLAN_ATTRIBUTE);
	pConduite_i->ExecIntMethod(SET_FLAG_MILDEW);
  }
};


// ==============================================

MildewWarningProcess::MildewWarningProcess() : DiscreteProcess(MILDEW_WARNING_PROCESS, "mildewWarningProcess") {
  ProcessedEntityClassId(BD_ENTITY);
  SetExec(mildewWarningProcess_execProcess_Body);
};


// ==============================================
// Une classe de Process : 'MorningMhydasInteractionProcess
// ==============================================

void morningMhydasInteractionProcess_execProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'execution du processus ponctuel.
//---------------------------------------------
{
  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  Entity* pVineyardSet = pDP->ProcessedEntity();

  pCurrentSim->Stop(); // a 'commenter' pour simulation autonome

  //
  // pour chaque parcelle de ttes EA, initialisation du cumul de pluie 
  // depuis la derni�re interaction avec Mhydas
  // (exploite dans  hourlyWeatherFileReading_postReadProcess)
  //
  int nbEA = pVineyardSet->GetNumberOfElements();
  for(int i=0;i<nbEA;i++) {
	Entity* pEA_i = pVineyardSet->GetElement(i);
	int nbPlots = pEA_i->GetNumberOfElements();
	for(int j=0;j<nbPlots;j++) {
	  Entity* pPlot_i_j = pEA_i->GetElement(j);
	  pPlot_i_j->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_MHYDAS_INTERACTION,
								  (float)0.);
//printf("\n'%s' morningMhydasInteractionProcess RAZ de CUMULATIVE_RAINFALL", pPlot_i_j->InstanceName());
	}
  }  
  //
  // programmation event de desallocation des LastOperationsMemoryItem
  //     (relatifs a l'episode qui se termine maintenant)
  // en l'instant de debut du prochain episode de simulation,
  // c'est-a-dire l'instant present.
  //
  Event* pEvt = new MemoryItemGarbagingEvent();
  pEvt->SetOccurrenceClockTime(pCurrentSim->Clock());
  pCurrentSim->InsertInAgendaEvents(pEvt);
};


// ==============================================

MorningMhydasInteractionProcess::MorningMhydasInteractionProcess() : DiscreteProcess(MORNING_MHYDAS_INTERACTION_PROCESS, "morningMhydasInteractionProcess") {
  ProcessedEntityClassId(BD_ENTITY);
  SetExec(morningMhydasInteractionProcess_execProcess_Body);
};


// ==============================================
// Une classe de Process : 'UpdateSeasonCurrentIndexProcess
// ==============================================

void updateSeasonCurrentIndexProcess_execProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'execution du processus ponctuel.
//---------------------------------------------
{
  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  BasicEntity* pVineyardSet = pDP->ProcessedEntity();
//printf("\nentree updateSeasonCurrentIndexProcess_exec sur '%s'", pVineyardSet->InstanceName());
  // ----------------------------------------------------------------------
  // T�che 1 (nominale) : mise a jour indice de la periode courante
  // T�che 2 (nominale) : initialisation des compteurs d'heures travaillees
  // T�che 3 (opportuniste) : pointage sur nouvelle OffWorkPeriod courante
  // T�che 4 (opportuniste) : rechargement des compteurs d'heures travaillees
  // T�che 5 (opportuniste) : RAZ cumul pluie depuis le debut du travail ce jour
  //                          + re-init de l'heure de debut du travail ce jour
  // ----------------------------------------------------------------------
  //
  // T�che 1 (nominale) : mise a jour de l'indice de la periode courante
  // sur chaque unite de MO, puisque chaque unite de MO pointe sur sa propre
  // instance de AnnualEmploymentPlan.
  // Chaque unite de MO a son propre AnnualEmploymentPlan parce que, 
  // . pour une m�me structure en 5 periodes, les 5 types de periodes
  //   ne sont pas les m�mes pour toutes les unites de MO
  // . pour un m�me jeu de 5 types de periodes, les periodes n'ont pas
  //   m�mes heures d'embauche et d'arret pour toutes les unites de MO
  //
  int now = pCurrentSim->Clock();
  tm* pTm_now = ClockValueToTm(now);
  int yday_now = pTm_now->tm_yday;
  int wday_now = pTm_now->tm_wday;
//printf("\nyday_now = %d wday_now = %d", yday_now, wday_now);

  int nbVineyards = pVineyardSet->GetNumberOfElements();
  for(int i=0;i<nbVineyards;i++) {
	Entity* pEA = pVineyardSet->GetElement(i);
	Entity* pPlan = pEA->GetEntityConstValue(CONTROL_PLAN_ATTRIBUTE);
	Entity* pARB = pPlan->GetSuperEntity();
	Entity* pMultiPool = pARB->GetComponent(RESOURCE_POOL_CONJUNCTION);
	Entity* pWorkerPool = pMultiPool->GetElement(0); // 0 : par construction
	
	int nbMOunits = pWorkerPool->GetNumberOfElements();
	for(int j=0;j<nbMOunits;j++) {
	  Entity* pMO = pWorkerPool->GetElement(j);
	  int yearlyUpperBound = pMO->GetIntConstValue(WORKING_TIME_YEARLY_UPPER_BOUND);
	  int dailyUpperBound = pMO->GetIntConstValue(WORKING_TIME_DAILY_UPPER_BOUND);
	  
	  Entity* pEmploymentPlan = 
		pMO->GetEntityConstValue(ANNUAL_EMPLOYMENT_PLAN_ATTRIBUTE);
	  //
	  // recherche de la nouvelle Timetable courante
	  //
	  int oldIndex =
		pEmploymentPlan->GetIntVarValue(CURRENT_ELEMENT_INDEX);
	  int newIndex = 
		pEmploymentPlan->ExecIntMethod(UPDATE_SEASON_CURRENT_INDEX);
	  Entity* pSeasonalPlan = pEmploymentPlan->GetElement(newIndex);
	  int weeklyUpperBound = pSeasonalPlan->GetIntConstValue(WORKING_TIME_WEEKLY_UPPER_BOUND);

	  if(newIndex != oldIndex) {
		//
		// attachement de la nouvelle Timetable courante a l'unite de MO
		//
		Entity* pRATT = pSeasonalPlan->GetEntityConstValue(TIMETABLE_ATTRIBUTE);
		
		pMO->SetEntityConstValue(AVAILABILITY_TIMETABLE_ATTRIBUTE,
								 pRATT);
//printf("\nupdateSeasonCurrentIndex %d -> %d ==> attach '%s' a '%s'", oldIndex, newIndex, pRATT->InstanceName(), pMO->InstanceName());
		//
		// recherche du TimeSlot courant et son initialisation 
		//
		int nbSlots = pRATT->GetNumberOfElements();
		Entity* pCurrentSlot = NULL;
		int currentSlotIndex = -9;
		float begHour, endHour;
		int begDate, endDate;
		for(int j=0;j<nbSlots;j=j+2) {  // on ne considere que les Morning's
		  Entity* pTimeSlot_j = pRATT->GetElement(j);
		  int weekDay_j = pTimeSlot_j->GetIntConstValue(WEEK_DAY_NUMBER);
//printf("\n\t\tweekDay_j = %d wday_now = %d", weekDay_j, wday_now);
		  if(weekDay_j == wday_now) {
			// now tombe un jour travaille
			//
			// modif JPR 121001 : le timeslot coutant n'est ce Morning que si
			//                    la date courante est dedans
			begHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_MORNING_BEG_HOUR);
			tm* pTm = new tm(*pTm_now);
			pTm->tm_hour = (int)ceil(begHour);
			mktime(pTm);
			begDate = StructTmToClockValue(pTm);
//printf("\t\t\tnow = %d begDate = %d", now, begDate);
			if(now >= begDate) {
			  currentSlotIndex = j;
			  pCurrentSlot = pTimeSlot_j;
			  endHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_MORNING_END_HOUR);
			  pTm->tm_hour = (int)floor(endHour);
			  mktime(pTm);
			  endDate = StructTmToClockValue(pTm);
			}
			else {  // modif JPR 121009
			  currentSlotIndex = j-1;
			  if(currentSlotIndex < 0) currentSlotIndex = nbSlots-1;
			  pCurrentSlot = pRATT->GetElement(currentSlotIndex);
			  pTm->tm_mday = pTm->tm_mday - 1;
			  begHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_BEG_HOUR);
			  pTm->tm_hour = (int)ceil(begHour);
			  mktime(pTm);
			  begDate = StructTmToClockValue(pTm);
			  endHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_END_HOUR);
			  pTm->tm_hour = (int)floor(endHour);
			  mktime(pTm);
			  endDate = StructTmToClockValue(pTm);
			}
			delete pTm;
			break;
		  }
		}
		if(! pCurrentSlot) {
		  //
		  // now tombe un jour non travaille
		  //  -> le slot courant est le dernier travaille
		  //
		  currentSlotIndex = nbSlots-1;
//printf("\n\tnow tombe un jour non travaille => currentSlotIndex = %d nbSlots = %d", currentSlotIndex, nbSlots);
		  pCurrentSlot = pRATT->GetElement(currentSlotIndex);  // Afternoon
		  // calcul du dernier jour de semaine (LUN, ...) travaille
		  // (on recule de 1 ou 2 jours selon le cas)
		  int deltaDays = 0;
		  if((nbSlots == 10) && (wday_now == DIM))
			deltaDays = 2;
		  else if(((nbSlots == 12) && ((wday_now == DIM)))
				  ||
				  ((nbSlots == 10) && ((wday_now == SAM))))
			deltaDays = 1;
//printf("\tdeltaDays = %d",deltaDays);

		  tm* pTm = new tm(*pTm_now);
		  // calcul [beg end] : on copie pTm_now, pour le conserver intact pour
		  // l'examen des pMO suivantes
		  pTm->tm_mday = pTm->tm_mday - deltaDays;
		  begHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_BEG_HOUR);
		  pTm->tm_hour = (int)ceil(begHour);
		  mktime(pTm);
		  begDate = StructTmToClockValue(pTm);
		  endHour = pSeasonalPlan->GetFloatConstValue(WORK_DAY_AFTERNOON_END_HOUR);
		  pTm->tm_hour = (int)floor(endHour);
		  mktime(pTm);
		  endDate = StructTmToClockValue(pTm);
		  delete pTm;
		}
//printf("\n\ttimeSlot initial '%s' index=%d [%d %d]", pCurrentSlot->InstanceName(), currentSlotIndex, begDate, endDate);
		pCurrentSlot->SetIntVarValue(BEG_DATE, begDate);
		pCurrentSlot->SetIntVarValue(END_DATE, endDate);
		//
		// mesures derogatoires sur le temps de tavail
		//
		// - trait. phyto. en ete
		if(pEmploymentPlan->IsInstanceOf(SPECIAL_WORKER_ANNUAL_EMPLOYMENT_PLAN)) {
//printf("\nmesure derogatoire ? newIndex = %d", newIndex);
		  if(newIndex == 4) {// MayAugust...Plan
			dailyUpperBound = gIntParam_phytoTreatmentsWorkingTime_day_;
			pMO->SetIntConstValue(WORKING_TIME_DAILY_UPPER_BOUND, dailyUpperBound);
			weeklyUpperBound = gIntParam_phytoTreatmentsWorkingTime_week_;
			pSeasonalPlan->SetIntConstValue(WORKING_TIME_WEEKLY_UPPER_BOUND, weeklyUpperBound);
//printf("\nmesure derogatoire '%s' dailyUpperBound = %d weeklyUpperBound =  %d", pMO->InstanceName(), dailyUpperBound, weeklyUpperBound);
		  }
		}
		//
		//  T�che 2 (nominale) : initialisation des compteurs d'heures travaillees
		//                       (poses sur l'unite de MO)

		// - annuel : prorata du WorkingTimeYearlyUpperBound du FarmWorker
		if(now < 24) {   // seulement lors du premier appel
		  int initialYearlyStock = yearlyUpperBound/365*(365-yday_now);
		  pMO->SetIntVarValue(WORKING_TIME_YEARLY_LEFT, initialYearlyStock);
//printf("\n'%s'WORKING_TIME_YEARLY_LEFT initialise a %d", pMO->InstanceName(), initialYearlyStock);

		// - hebdo : prorata du WorkingTimeWeeklyUpperBound du SeasonalEmploymentPlan
		  int daysInWeek = pRATT->GetNumberOfElements() / 2;
		  int daysToWorkInInitialWeek = daysInWeek - wday_now + 1;
		  int initialWeeklyStock = weeklyUpperBound/daysInWeek*daysToWorkInInitialWeek;
		  pMO->SetIntVarValue(WORKING_TIME_WEEKLY_LEFT, initialWeeklyStock);
//printf("\n'%s'WORKING_TIME_WEEKLY_LEFT initialise a %d", pMO->InstanceName(), initialWeeklyStock);

		// - journalier : le WorkingTimeDailyUpperBound du FarmWorker
		  int initialDailyStock = dailyUpperBound;
		  pMO->SetIntVarValue(WORKING_TIME_DAILY_LEFT, initialDailyStock);
//printf("\n'%s'WORKING_TIME_DAILY_LEFT initialise a %d", pMO->InstanceName(), initialDailyStock);
		}
		//
		// affectation de la nouvelle valeur d'index
		//
		pRATT->SetIntVarValue(CURRENT_ELEMENT_INDEX, currentSlotIndex);
//printf("\n\ttimeSlot initial '%s' index=%d", pCurrentSlot->InstanceName(), currentSlotIndex);
	  }
	  //
	  // T�che 4 (opportuniste) : rechargement des compteurs d'heures travaillees
	  //
	  // - annuel : WorkingTimeYearlyUpperBound du FarmWorker
	  if(yday_now == 0) {                // 1er janvier
		pMO->SetIntVarValue(WORKING_TIME_YEARLY_LEFT, yearlyUpperBound);
//printf("\nreload '%s'WORKING_TIME_YEARLY_LEFT avec %d", pMO->InstanceName(), yearlyUpperBound);
		pMO->SetIntVarValue(AVAILABILITY_STATUS, 1);
	  }
	  // - hebdo : WorkingTimeWeeklyUpperBound du SeasonalEmploymentPlan
	  if(pMO->GetIntVarValue(WORKING_TIME_YEARLY_LEFT) > 0) {
		if(wday_now == 1) {                // LUNDI
		  pMO->SetIntVarValue(WORKING_TIME_WEEKLY_LEFT, weeklyUpperBound);
//printf("\nreload '%s'WORKING_TIME_WEEKLY_LEFT avec %d", pMO->InstanceName(), weeklyUpperBound);
		  pMO->SetIntVarValue(AVAILABILITY_STATUS, 1);
		}
		if(pMO->GetIntVarValue(WORKING_TIME_WEEKLY_LEFT) > 0) {
	  // - journalier : WorkingTimeDailyUpperBound du FarmWorker
		  pMO->SetIntVarValue(WORKING_TIME_DAILY_LEFT, dailyUpperBound);
//printf("\nreload '%s'WORKING_TIME_DAILY_LEFT avec %d", pMO->InstanceName(), dailyUpperBound);
		  pMO->SetIntVarValue(AVAILABILITY_STATUS, 1);
		}
	  }
	  //
	  // T�che 5 (opportuniste) : init debut du travail ce jour
	  //
	  pMO->SetIntConstValue(WORK_BEGINNING_HOUR, D_INTMAX);
	  pMO->SetFloatVarValue(CUMULATIVE_RAINFALL_FROM_WORK_BEGINNING, (float)0.);

	}   // fin boucle sur les unites de MO
  }   // fin boucle sur les vineyards
  delete pTm_now;
  //
  // T�che 3 (opportuniste) : 
  // pointage sur la nouvelle OffWorkPeriod courante de chaque instance de
  // OffWorkCalendar creee dans un des fichiers en entree de la simulation
  //
  pEntityTab* calendarTab = GetEntityList(OFF_WORK_CALENDAR);
  int nbCalendars = calendarTab->size();
  for(int k=0;k<nbCalendars;k++) {
	Entity* pCalendar_k = (Entity*)calendarTab->get_element(k);
	int oldIndex =
	  pCalendar_k->GetIntVarValue(CURRENT_ELEMENT_INDEX);
	int newIndex = 
	  pCalendar_k->ExecIntMethod(UPDATE_OFF_PERIOD_CURRENT_INDEX);
	//if(TRUE) {
	if(FALSE) {
	  if(newIndex != oldIndex) {
		Entity* pCurrentPeriod = pCalendar_k->GetElement(newIndex);
		printf("'%s' periode 'off' courante : '%s' ['%s' '%s']", pCalendar_k->InstanceName(), pCurrentPeriod->InstanceName(), pCurrentPeriod->GetStringConstValue(BEGINNING_CALENDAR_DATE), pCurrentPeriod->GetStringConstValue(END_CALENDAR_DATE));
	  } 
	}
  }
};


// ==============================================

UpdateSeasonCurrentIndexProcess::UpdateSeasonCurrentIndexProcess() : DiscreteProcess(UPDATE_SEASON_CURRENT_INDEX_PROCESS, "updateSeasonCurrentIndexProcess") {
  ProcessedEntityClassId(VINEYARD_SET);
  SetExec(updateSeasonCurrentIndexProcess_execProcess_Body);
};


// ==============================================
// Une classe de Process : 'InitPestInfoFileReading
// ==============================================

void initPestInfoFileReading_postReadProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'exploitation des valeurs
// lues sur un enregistrement.
//---------------------------------------------
{
  ReadFactualDataRecordProcess* pDP
    = (ReadFactualDataRecordProcess*)(pM->DescribedDiscreteProcess());
  Entity* pVineyardSet = pDP->ProcessedEntity();
  FactualDataFile* pF = pDP->GetFile();

  char* fldFlagShoot = pF->GetStringFieldValue(FLD_FLAGSHOOT);
  int fldLevelPM = pF->GetIntFieldValue(FLD_LEVEL_PM);
  char* fldPrimInfectDM = pF->GetStringFieldValue(FLD_PRIM_INFECT_DM);
  int fldLevelDM = pF->GetIntFieldValue(FLD_LEVEL_DM);
  float fldRiskLevelDM = pF->GetFloatFieldValue(FLD_RISK_DM);

  Entity* pestInfoItem = new PestInfoMemoryItem();
  //--------------------------------------------------
  int index = FLD_DMY;
  char* champDate = strdup(pF->GetStringFieldValue(index));

  if(strlen(champDate) == 0)
	return;

//printf("\ninitPestInfoFileReading_PR champDate : '%s'", champDate);
  char* s_d = strtok(champDate,"/");
  char* s_m = strtok(NULL,"/");
  char* s_y = strtok(NULL,"/");
  int d = atoi(s_d);
  int m = atoi(s_m);
  int y = atoi(s_y);
  free(champDate);
  int clockValue = DateToClockValue(d, m, y, 0, 0, 0);
  pestInfoItem->SetIntConstValue(INSTANT_ATTRIBUTE, 
								 clockValue);
  //--------------------------------------------------
  int flagShoot = NON;
  if(! strcmp(fldFlagShoot, "O"))
	flagShoot = OUI;
  pestInfoItem->SetIntConstValue(POWDERY_MILDEW_FLAG_SHOOT,
								 flagShoot);
  //--------------------------------------------------

  pestInfoItem->SetIntConstValue(POWDERY_MILDEW_LEVEL,
								 fldLevelPM);
  //--------------------------------------------------
  int primInfectDM = NON;
  if(! strcmp(fldPrimInfectDM, "O"))
	primInfectDM = OUI;
  pestInfoItem->SetIntConstValue(DOWNY_MILDEW_PRIMARY_INFECTION,
								 primInfectDM);
  //--------------------------------------------------

  pestInfoItem->SetIntConstValue(DOWNY_MILDEW_LEVEL,
								 fldLevelDM);
  //--------------------------------------------------

  pestInfoItem->SetFloatConstValue(DOWNY_MILDEW_RISK_LEVEL,
								   fldRiskLevelDM);
  //--------------------------------------------------

  Entity* pMemory = pVineyardSet->GetEntityConstValue(PEST_INFO_MEMORY_ATTRIBUTE);
  pMemory->InsertElement(0, pestInfoItem);

//printf("\nPestInfo : t:'%s' (clock = %d) %d %d %d %d %4.2f", pF->GetStringFieldValue(FLD_DMY), clockValue, flagShoot, fldLevelPM, primInfectDM, fldLevelDM, fldRiskLevelDM);
//printf("\n\t--> ");pMemory->DisplayElementNames();

};


// ==============================================

InitPestInfoFileReading::InitPestInfoFileReading() : InitFactualDataFileReadingProcess() {
  ClassSymbol(INIT_PEST_INFO_FILE_READING);
  ClassName("initPestInfoFileReading");
  ProcessedEntityClassId(VINEYARD_SET);
  FileClassId(PEST_INFO_FILE);
  SetPostRead(initPestInfoFileReading_postReadProcess_Body);
};


// ==============================================
// Une classe de Process : 'InitPhenoStageFileReading
// ==============================================

void initPhenoStageFileReading_postReadProcess_Body(ProcessMethod* pM)
//---------------------------------------------
// Definition de la fonction qui code l'exploitation des valeurs
// lues sur un enregistrement.
//---------------------------------------------
{
  ReadFactualDataRecordProcess* pDP
    = (ReadFactualDataRecordProcess*)(pM->DescribedDiscreteProcess());
  //Entity* pEntity = pDP->ProcessedEntity();
  FactualDataFile* pF = pDP->GetFile();

  int codeStade = pF->GetIntFieldValue(FLD_CODE_STADE);
  char* nomVariete = pF->GetStringFieldValue(FLD_VARIETE);

  pEntityTab* isoAgeingEntityTab = GetEntityList(ISO_AGEING_ENTITY);
  int nbElem = isoAgeingEntityTab->size();
  for(int k=0;k<nbElem;k++) {
	Entity* pIsoAgeingEntity = (Entity*)isoAgeingEntityTab->get_element(k);
	Entity* pVariety = pIsoAgeingEntity->GetEntityConstValue(GRAPE_VARIETY_ATTRIBUTE);
	if(! strcmp(pVariety->InstanceName(), nomVariete)) {
	  Entity* pAge = pIsoAgeingEntity->GetEntityConstValue(AGE_ATTRIBUTE);
	  pAge->SetIntVarValue(CURRENT_STAGE_INDEX, codeStade);
	  pEntityTab* stageTab = pAge->GetEntityTabConstValue(STAGE_TAB);
	  Entity* pCurrentStage = (Entity*)stageTab->get_element(codeStade);
//printf("\nPhenoStage : '%s' passe au stade %d", nomVariete, codeStade);
	  tm* pTm = ClockValueToTm(pCurrentSim->Clock());
	  pCurrentStage->SetIntVarValue(DAY_WHEN_ACHIEVED, pTm->tm_yday);
	  delete pTm;

	  //break;  // !! - JPR 101020
	}
  }
};


// ==============================================

InitPhenoStageFileReading::InitPhenoStageFileReading() : InitFactualDataFileReadingProcess() {
  ClassSymbol(INIT_PHENO_STAGE_FILE_READING);
  ClassName("initPhenoStageFileReading");
  ProcessedEntityClassId(BD_ENTITY);
  FileClassId(PHENO_STAGE_FILE);
  SetPostRead(initPhenoStageFileReading_postReadProcess_Body);
};


