// ==============================================
// Fichier : UserFile.cc  
// Type    : source C++
// 
// Description : constructeurs des classes des fichiers sequentiels.
// ==============================================
  
#include "UserFile.h"
  
// ==============================================
// Une classe de fichier séquentiel : 'HourlyWeatherFile'
// ==============================================

HourlyWeatherFile::HourlyWeatherFile() : BufferizedDataFile(HOURLY_WEATHER_FILE, READ) {

	AddField (FLD_DATE,        STRING);
	AddField (FLD_HOUR,        STRING);
	AddField (FLD_RAINFALL,    FLOAT);
  
	AssignFormat ("%s%s%f");
	SetNbTopCommentLines (10);
  
	AssignScanNextEnrgtMethod(hourlyWeatherFile_ScanNextEnrgt);

	AssignSetMemoryScopeMethod(hourlyWeatherFile_SetMemoryScope);
	AssignSetPredictionScopeMethod(hourlyWeatherFile_SetPredictionScope);
};


int hourlyWeatherFile_ScanNextEnrgt(BufferizedDataFile* pF, char* str)
//---------------------------------------------
// Fonction de scanning d'un enregistrement en memoire.
//---------------------------------------------
{
  int i;
  i = sscanf (str,
              pF->GetFormat()
              , pF->mpFieldTab[FLD_DATE]->mFieldValue.stringValue
              , pF->mpFieldTab[FLD_HOUR]->mFieldValue.stringValue
              , &pF->mpFieldTab[FLD_RAINFALL]->mFieldValue.floatValue
             );
  if((i != EOF) && (i < pF->mpFieldTab.size()))
    BadFileRead(pF);
  return i;
};

int hourlyWeatherFile_SetMemoryScope(UserFileId fileId)
//------------------------------------------------------
// Fonction de sp�cification de la port�e de la m�moire.
//------------------------------------------------------
{
  int result = 0;

  Entity* myManager = GetFirstEntity(VIRTUAL_REGION_MANAGER);
  result = myManager->GetIntVarValue(MEMORY_SCOPE);

//printf("\nhourlyWeatherFile_SetMemoryScope myManager : '%s' memoryScope = %d", myManager->InstanceName(), result);

  return result;
};


int hourlyWeatherFile_SetPredictionScope(UserFileId fileId)
//---------------------------------------------------------
// Fonction de sp�cification de la port�e de la pr�diction.
//---------------------------------------------------------
{
  int result = 0;

  Entity* myManager = GetFirstEntity(VIRTUAL_REGION_MANAGER);
  result = myManager->GetIntVarValue(PREDICTION_SCOPE);
//printf("\nhourlyWeatherFile_SetPredictionScope myManager : '%s' predictionScope = %d", myManager->InstanceName(), result);

  return result;
};



// ==============================================
// Une classe de fichier séquentiel : 'PestInfoFile'
// ==============================================

PestInfoFile::PestInfoFile() : FactualDataFile(PEST_INFO_FILE, READ) {

	AddField (FLD_DMY,        STRING);
	AddField (FLD_FLAGSHOOT,  STRING);
	AddField (FLD_LEVEL_PM,   INT);
	AddField (FLD_PRIM_INFECT_DM, STRING);
	AddField (FLD_LEVEL_DM,   INT);
	AddField (FLD_RISK_DM,    FLOAT);
  
	AssignFormat ("%s%s%d%s%d%f");
	SetNbTopCommentLines (1);
  
	AssignReadNextEnrgtMethod(pestInfoFile_ReadNextEnrgt);
	AssignGetRecordClockValueMethod(pestInfoFile_GetRecordClockValue);
};

int pestInfoFile_ReadNextEnrgt(FactualDataFile* pF)
//---------------------------------------------
// Fonction de lecture d'un seul enregistrement.
//---------------------------------------------
{
  int i;
  i = fscanf (pF->GetPhysicalFile(),
              pF->GetFormat()
              , pF->mpReadFieldTab[FLD_DMY]->mFieldValue.stringValue
              , pF->mpReadFieldTab[FLD_FLAGSHOOT]->mFieldValue.stringValue
              , &pF->mpReadFieldTab[FLD_LEVEL_PM]->mFieldValue.intValue
              , pF->mpReadFieldTab[FLD_PRIM_INFECT_DM]->mFieldValue.stringValue
              , &pF->mpReadFieldTab[FLD_LEVEL_DM]->mFieldValue.intValue
              , &pF->mpReadFieldTab[FLD_RISK_DM]->mFieldValue.floatValue
             );
  if((i != EOF) && (i < pF->mpFieldTab.size()))
    BadFileRead(pF);
 
  DiscreteProcess* pDP = (DiscreteProcess*)pF->GetReaderProcess();
    ReadFactualDataRecordProcess* pDPcopy
  	= new ReadFactualDataRecordProcess(*((ReadFactualDataRecordProcess*)pDP));
  pF->SetReaderProcess(pDPcopy);
  int readClock = pF->GetRecordClockValue(pF);
  int now = pCurrentSim->Clock();
  
  if(i == EOF) {
    pF->BufferTabToDataTab();
  }
  else {
    if(pF->IsBufferEmpty()) {
      pF->ReadTabToBufferTab();
      pF->BufferTabToDataTab();
    }
    else {
      pF->BufferTabToDataTab();
      pF->ReadTabToBufferTab();
    }
    int eventClock = readClock;
    if(eventClock < now)
      eventClock = now;
    Event* pEv = new DiscreteExecution(eventClock, pDPcopy);
    pEv->SetPriorityDegree(0);
    Boolean planned = pCurrentSim->InsertInAgendaEvents(pEv);
    if(! planned) pEv->DeleteInDepth();
  }
  return i;
};

int pestInfoFile_GetRecordClockValue(FactualDataFile* pF)
//---------------------------------------------
// D�finition de la fonction qui code la reconnaissance
// de la date dans un enregistrement, et son renvoi.
//---------------------------------------------
{
  int clockValue;
  // 
  // cas d'un seul champ date de la forme '24/09/2001'
  // 
  int index = FLD_DMY;
  char* champDate = strdup(pF->GetStringFieldReadValue(index));
//printf("\npestInfoFile_GetRecordClockValue champDate : '%s'", champDate);
  char* s_d = strtok(champDate,"/");
  char* s_m = strtok(NULL,"/");
  char* s_y = strtok(NULL,"/");
  int d = atoi(s_d);
  int m = atoi(s_m);
  int y = atoi(s_y);
  free(champDate);
  clockValue = DateToClockValue(d, m, y, 0, 0, 0);
  
//printf("\t --> clock = %d", clockValue);
  return clockValue;

};



// ==============================================
// Une classe de fichier séquentiel : 'PhenoStageFile'
// ==============================================

PhenoStageFile::PhenoStageFile() : FactualDataFile(PHENO_STAGE_FILE, READ) {

	AddField (FLD_CODE_STADE,        INT);
	AddField (FLD_JMA,               STRING);
	AddField (FLD_VARIETE,           STRING);
  
	AssignFormat ("%d%s%s");
  
	AssignReadNextEnrgtMethod(phenoStageFile_ReadNextEnrgt);
	AssignGetRecordClockValueMethod(phenoStageFile_GetRecordClockValue);
};

int phenoStageFile_ReadNextEnrgt(FactualDataFile* pF)
//---------------------------------------------
// Fonction de lecture d'un seul enregistrement.
//---------------------------------------------
{
  int i;
  i = fscanf (pF->GetPhysicalFile(),
              pF->GetFormat()
              , &pF->mpReadFieldTab[FLD_CODE_STADE]->mFieldValue.intValue
              , pF->mpReadFieldTab[FLD_JMA]->mFieldValue.stringValue
              , pF->mpReadFieldTab[FLD_VARIETE]->mFieldValue.stringValue
             );
  if((i != EOF) && (i < pF->mpFieldTab.size()))
    BadFileRead(pF);
 
  DiscreteProcess* pDP = (DiscreteProcess*)pF->GetReaderProcess();
    ReadFactualDataRecordProcess* pDPcopy
  	= new ReadFactualDataRecordProcess(*((ReadFactualDataRecordProcess*)pDP));
  pF->SetReaderProcess(pDPcopy);
  int readClock = pF->GetRecordClockValue(pF);
  int now = pCurrentSim->Clock();
  
  if(i == EOF) {
    pF->BufferTabToDataTab();
  }
  else {
    if(pF->IsBufferEmpty()) {
      pF->ReadTabToBufferTab();
      pF->BufferTabToDataTab();
    }
    else {
      pF->BufferTabToDataTab();
      pF->ReadTabToBufferTab();
    }
    int eventClock = readClock;
    if(eventClock < now)
      eventClock = now;
    Event* pEv = new DiscreteExecution(eventClock, pDPcopy);
    pEv->SetPriorityDegree(0);
    Boolean planned = pCurrentSim->InsertInAgendaEvents(pEv);
    if(! planned) pEv->DeleteInDepth();
  }
  return i;
};

int phenoStageFile_GetRecordClockValue(FactualDataFile* pF)
//---------------------------------------------
// D�finition de la fonction qui code la reconnaissance
// de la date dans un enregistrement, et son renvoi.
//---------------------------------------------
{
  int clockValue;
  // 
  // cas d'un seul champ date de la forme '24/09/2001'
  // 
  int index = FLD_JMA;// = 1
  char* champDate = strdup(pF->GetStringFieldReadValue(index));
//printf("\nphenoStageFile_GetRecordClockValue champdate = '%s'", champDate);
  char* s_d = strtok(champDate,"/");
  char* s_m = strtok(NULL,"/");
  char* s_y = strtok(NULL,"/");
  int d = atoi(s_d);
  int m = atoi(s_m);
  int y = atoi(s_y);
  free(champDate);
  clockValue = DateToClockValue(d, m, y, 0, 0, 0);
  //
  // +1 pour diferencier la date d'occurence de cet event
  // de l'event de lecture du fichier pestInfo (modif 101220).
  // Sans cela, Simulation::InsertInAgendaEvents ne programme pas
  // un des deux events, parce qu'un est deja programme au meme clock
  //
  // --> 17/04/2004 dans ce fichier est interprete comme 17/04/2004 1h00
  
  return clockValue+1;

};



