// ==============================================
// Fichier : UserOperation.cc
// Type    : source C++
// 
// Description : constructeurs des classes des 'Operation'.
// ==============================================
  
#include "UserOperation.h"
  
bool myFloatAreaOperation_feasibilityCondition_Body(EntityMethod* pM) {
//---------------------------------------------------------------------
// M�thode invoqu�e alors que l'activit� est ouverte.
// Doit renvoyer vrai si l'op�ration est maintenant faisable.
//---------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  bool result = TRUE; 
//printf("\nentree '%s'::FC", pOt->InstanceName());

  //
  // Condition générale de traficabilité pour opérations mécanisées
  //
  if(pOt->IsInstanceOf(MOTORIZED_OPERATION)) {
	Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
	Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
	pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
	Entity* pPlot = (Entity*)ooList->get_element(0);
	Entity* pWS = pPlot->GetEntityConstValue(WEATHER_STATION_ATTRIBUTE);
	char* pathName = pWS->GetStringConstValue(PATH_NAME_ATTRIBUTE);
	tm* pTm = ClockValueToTm(pCurrentSim->Clock());
	int nearestMem = pTm->tm_hour;
	delete pTm;
	int farthestMem = nearestMem + 7*24;
	float value =  // pluie des 7j precedents
	  SumUpMemoryFloatFieldValue(pathName, FLD_RAINFALL, nearestMem, farthestMem);
	if(pOt->IsInstanceOf(TILLAGE_OPERATION)) {
	  pOt->SetFloatConstValue(LAST_SEVEN_DAYS_RAINFALL, value);
	}
	//
	// test
	//
	float threshold = gRealParam_trafficabilityRainMaxThreshold_mechanicalOperation_;
//printf("\nmyFloatAreaOperation_FC '%s' pluie[j-1 j-7] = %f / seuil = %f", pWS->InstanceName(), value, threshold);
	if(value >= threshold) {
	  result = FALSE;
	}
  }

  if(! result) 
	if(gMyTraceWarnings)
	  printf("\n'%s'::FC returns FALSE / non trafficalility", pOt->InstanceName());
//printf("\n'%s'::FC returns %d", pOt->InstanceName(), result);
  return result;
};

MyFloatAreaOperation::MyFloatAreaOperation() {
// =========================================================
// Une classe de Entity : 'MyFloatAreaOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(MY_FLOAT_AREA_OPERATION);
  //SetParentClassId(FLOAT_QUANTITY_GRANULATED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  SetIntConstValue(QUANTITY_DESCRIPTOR_ID, AREA);
  AssignBody(FEASIBILITY_CONDITION, myFloatAreaOperation_feasibilityCondition_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
  AchievementDegreeMonitor->AttachToDescriptor(this, ACHIEVEMENT_DEGREE);

  SetIntConstValue(PRIORITY_DEGREE, 99);   // + JPR 120907
  // corrolairement et à titre de modèle, la priorité du Trellising est mise à 1
};

float manualOperation_setUserFloatQuantitySpeed_Body(EntityMethod* pM) {
//
// Retourne la vitesse de l'op�ration.
// (surcharge une �ventuelle valeur du descripteur FloatQuantitySpeed).
//
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  
  float currentValue = pOt->GetFloatConstValue(FLOAT_QUANTITY_SPEED);
  if(currentValue > (float)-8.)
	return currentValue;
  //
  // FloatQuantitySpeed est -9. par construction de l'operation,
  // puis devient > 0. dans ce corps de methode.
  // Pour une instance d'operation, on ne poursuit donc compl�tement 
  // le present calcul de la vitesse qu'une seule fois.
  //
  float result;

  Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);

  Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
  pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
  Entity* pPlot = (Entity*)ooList->get_element(0);
//printf("\nentree '%s'::setUserFloatQuantitySpeed plot '%s'", pOt->InstanceName(), pPlot->InstanceName());

  float alleyWidth = pPlot->GetIntConstValue(ALLEY_WIDTH) / (float)100.;
  float plantSpacing = pPlot->GetIntConstValue(PLANT_SPACING) / (float)100.;
  //
  // vitesse nominale en nombre de souches par heure :
  // - Pruning : NominalIntQuantitySpeed est dote d'un moniteur
  // - autres : valeur affectee par construction
  //
  int nominalSpeed = pOt->GetIntVarValue(NOMINAL_INT_QUANTITY_SPEED);
//printf("\n\talleyWidth = %3.2f plantSpacing = %3.2f nominalSpeed = %d", alleyWidth, plantSpacing, nominalSpeed);

  result = alleyWidth * plantSpacing * nominalSpeed / (float)10000.;

  pOt->SetFloatConstValue(FLOAT_QUANTITY_SPEED, result);

//printf("\n'%s'::setUserFloatQuantitySpeed plot '%s' returns %f", pOt->InstanceName(), pPlot->InstanceName(), result);
  return result;
};

ManualOperation::ManualOperation() {
// =========================================================
// Une classe de Entity : 'ManualOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(MANUAL_OPERATION);
  //SetParentClassId(MY_FLOAT_AREA_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  Descriptor* pD;
  pD = new NominalIntQuantitySpeed();     AddVariableDescriptor(pD);
  SetFloatConstValue(SUSPENSION_THRESHOLD, 1.0);
  AssignBody(SET_USER_FLOAT_QUANTITY_SPEED, manualOperation_setUserFloatQuantitySpeed_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  // Pour economiser des reconstitutions de la valeur de la vitesse
  // par la m�thode manualOperation_setUserFloatQuantitySpeed
  // on n'y fait cette reconstitution que si la valeur courante est < 0.
  // La presente initialisation � -9. permet cela.
  // Le calcul de la vitesse est fait une seule fois, puisque 
  // setUserFloatQuantitySpeed affecte une valeur > 0. lors de ce calcul.
  //
  SetFloatConstValue(FLOAT_QUANTITY_SPEED, (float)-9.);
};

float motorizedOperation_setUserFloatQuantitySpeed_Body(EntityMethod* pM) {
//
// Retourne la vitesse de l'op�ration.
// (surcharge une �ventuelle valeur du descripteur FloatQuantitySpeed).
//
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  
  float currentValue = pOt->GetFloatConstValue(FLOAT_QUANTITY_SPEED);
  if(currentValue > (float)-8.)
	return currentValue;
  //
  // FloatQuantitySpeed est -9. par construction de l'operation,
  // puis devient > 0. dans ce corps de methode.
  // Pour une instance d'operation, on ne poursuit donc compl�tement 
  // le present calcul de la vitesse qu'une seule fois.
  //
  float result;

  Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);

  Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
  pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
  Entity* pPlot = (Entity*)ooList->get_element(0);
//printf("\nentree '%s'::setUserFloatQuantitySpeed plot '%s'", pOt->InstanceName(), pPlot->InstanceName());

  float alleyWidth = pPlot->GetIntConstValue(ALLEY_WIDTH) / (float)100.;
  //
  // vitesse nominale en km par heure :
  // - valeur affectee par construction
  //
  float nominalSpeed = pOt->GetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED);
//printf("\n\talleyWidth = %3.2f nominalSpeed = %f", alleyWidth, nominalSpeed);

  // cas particulier TILLAGE
  //
  if(pOt->IsInstanceOf(TILLAGE_OPERATION)) {
	//Entity* pConduite = ((Activity*)pInst)->RootActivity();
	//Entity* pARB      = pConduite->GetSuperEntity();
	//int sm = pARB->GetIntConstValue(SOIL_MANAGEMENT_OPTION);  // JPR 1312 : desc. supprimé
	int sm = pPlot->GetIntConstValue(ROW_MANAGEMENT);  // modif JPR 1312
	if((sm == GRASS_COVERING_TILLAGE)
	   ||
	   (sm == TOTAL_TILLAGE)) {
	  float reductionRate = gRealParam_tillageOperation_speedReductionRate_;
	  nominalSpeed = nominalSpeed * reductionRate;
	}
  }
  //
  // Calcul fr�quence de passage dans les inter-rangs :
  // - 0.5 si le tracteur ne passe qu'un IR /2
  // - 1.0 si le tracteur ne passe dans tous les IR
  //
  pPlot->SetEntityArgValue(RETURN_PASSED_ROWS_FREQUENCY,
						   OPERATION_ARGUMENT, pOt);
  float passedRowsFrequency = 
	pPlot->ExecFloatMethod(RETURN_PASSED_ROWS_FREQUENCY);

  result = alleyWidth * nominalSpeed / (float)12. / passedRowsFrequency;

  pOt->SetFloatConstValue(FLOAT_QUANTITY_SPEED, result);

//printf("\n'%s'::setUserFloatQuantitySpeed plot '%s' returns %f", pOt->InstanceName(), pPlot->InstanceName(), result);
  return result;
};

MotorizedOperation::MotorizedOperation() {
// =========================================================
// Une classe de Entity : 'MotorizedOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(MOTORIZED_OPERATION);
  //SetParentClassId(MY_FLOAT_AREA_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  Descriptor* pD;
  pD = new NominalFloatQuantitySpeed();   AddVariableDescriptor(pD);
  SetFloatConstValue(SUSPENSION_THRESHOLD, 1.0);
  AssignBody(SET_USER_FLOAT_QUANTITY_SPEED, motorizedOperation_setUserFloatQuantitySpeed_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
//
// SUSPENSION_THRESHOLD : la valeur idéale 0 forcerait la continuation sans 
// suspension de ces opérations une fois commencées (activité requise).
// La valeur 1 provoque un déroulé plus satisfaisant (lire READ_ME_versions).
// Une certaine modif (121215) dans la fonction ReplaceSubset de cdiese est 
// plus satisfaisante a priori, en association avec la valeur 0, mais a 
// provoqué une simulation extrêmement longue. A reprendre.

  // Pour economiser des reconstitutions de la valeur de la vitesse
  // par la m�thode motorizedOperation_setUserFloatQuantitySpeed
  // on n'y fait cette reconstitution que si la valeur courante est < 0.
  // La presente initialisation � -9. permet cela.
  // Le calcul de la vitesse est fait une seule fois, puisque 
  // setUserFloatQuantitySpeed affecte une valeur > 0. lors de ce calcul.
  //
  SetFloatConstValue(FLOAT_QUANTITY_SPEED, (float)-9.);
};

bool motorizedSprayingOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

bool motorizedSprayingOperation_feasibilityCondition_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// Methode invoquee alors que l'activite est ouverte.
// Doit renvoyer vrai si l'operation est maintenant faisable.
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
//printf("\nentree '%s'::FC", pOt->InstanceName());
  bool result = myFloatAreaOperation_feasibilityCondition_Body(pM);
  if(result == FALSE) {
	//printf("\n'%s'::FC returns %d / non trafficalility", pOt->InstanceName(), result);
	// cette FC : + JPR 130220
	// quand la FC est non  satisfaite, et que le chantier concerné est
	// un trt EXTRA, on déclare que cette parcelle n'est plus à traiter
	//
	Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
	Entity* pChantier = NULL;
	int typeChantier = -9;
	pChantier = 
	  (Entity*)pInst->GetFirstContainer(CONCURRENT_DISTRIBUTED_ACTIVITY_SET,
										FALSE);
	if(pChantier == NULL) {
	  pChantier = 
		(Entity*)pInst->GetFirstContainer(SERIAL_DISTRIBUTED_ACTIVITY_SET,
										  FALSE);
	}
	typeChantier = App_GetDistributedActivitySetType(pChantier);
	if(typeChantier == EXTRA_PHYTO) {
	  Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
	  pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
	  Entity* pPlot = (Entity*)ooList->get_element(0);
	  pPlot->SetFloatVarValue(PERCENT_EXTRA_PHYTO_TREATED, (float)-1.);
	  pOt->SetFloatVarValue(ACHIEVEMENT_DEGREE, (float)1., FALSE);
	  //  + JPR 130223 : si un trt 'Extra' a echoue par non-faisabilite,
	  // on conserve la possibilite d'en faire 2 (comme stipule dans le
	  // constructeur de ExtraPhytoTreatmentsIteration
	  Entity* pIterate = (Entity*)pChantier->GetFirstContainer(ACTIVITY_ITERATION, FALSE);
	  pIterate->SetIntVarValue(MAX_NUMBER_OF_REPLICATIONS, 3, FALSE);
	}
  }
  return result;
};

MotorizedSprayingOperation::MotorizedSprayingOperation() {
// =========================================================
// Une classe de Entity : 'MotorizedSprayingOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(MOTORIZED_SPRAYING_OPERATION);
  //SetParentClassId(MOTORIZED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, motorizedSprayingOperation_stateTransitionProcedure_Body);
  AssignBody(FEASIBILITY_CONDITION, motorizedSprayingOperation_feasibilityCondition_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
};

bool plantationOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pPlot = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;
  //
  // augmentation de la surface de l'exploitation mise en vigne
  //
  Entity* pVineyard = pPlot->GetSuperSet();
  float x = pVineyard->GetFloatConstValue(AREA_WITH_GRAPEVINE);
  float superficie = pPlot->GetFloatConstValue(AREA);
  pVineyard->SetFloatConstValue(AREA_WITH_GRAPEVINE, x + superficie);
  
//printf("\nt = %d passage par '%s'_STP nouvelle superficie = %f", pCurrentSim->Clock(), pOt->InstanceName(), x + superficie);

  return result;
};

PlantationOperation::PlantationOperation() {
// =========================================================
// Une classe de Entity : 'PlantationOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(PLANTATION_OPERATION);
  //SetParentClassId(MANUAL_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, plantationOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float haPerDay = gRealParam_plantationOperation_overallSpeed_;
  SetFloatConstValue(FLOAT_QUANTITY_SPEED, haPerDay/(float)24.);
};

bool pruningOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pPlot = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

PruningOperation::PruningOperation() {
// =========================================================
// Une classe de Entity : 'PruningOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(PRUNING_OPERATION);
  //SetParentClassId(MANUAL_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  SetIntConstValue(STEP, 1);
   PruningOperationNominalSpeedMonitor->AttachToDescriptor(this, NOMINAL_INT_QUANTITY_SPEED);
  AssignBody(STATE_TRANSITION_PROCEDURE, pruningOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
};

ShootThinningOperation::ShootThinningOperation() {
// =========================================================
// Une classe de Entity : 'ShootThinningOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(SHOOT_THINNING_OPERATION);
  //SetParentClassId(MOTORIZED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
};

bool tillageOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pPlot = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

  //Entity* pVegetal = pPlot->GetComponent(VEGETAL_COMPARTMENT);
  //int nbRows = pVegetal->GetNumberOfElements(); 
//printf("\nt = %d passage par '%s'_STP sur '%s'", pCurrentSim->Clock(), pOt->InstanceName(), pPlot->InstanceName());

  return result;
};

bool tillageOperation_feasibilityCondition_Body(EntityMethod* pM) {
//-----------------------------------------------------------------
// M�thode invoqu�e alors que l'activit� est ouverte.
// Doit renvoyer vrai si l'op�ration est maintenant faisable.
//-----------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
//printf("\nentree '%s'::FC", pOt->InstanceName());
  bool result = myFloatAreaOperation_feasibilityCondition_Body(pM);
  if(result == FALSE) {
//printf("\n'%s'::FC returns %d / non trafficalility", pOt->InstanceName(), result);
	return result;
  }

  Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
  Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
  pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
  Entity* pPlot = (Entity*)ooList->get_element(0);
//printf("\nentree '%s'::FC plot '%s'\n", pOt->InstanceName(), pPlot->InstanceName());
  
  //
  // visite des 2 inter-rangs
  //
  Entity* pVegetal = pPlot->GetComponent(VEGETAL_COMPARTMENT);

  int nbRows = pVegetal->GetNumberOfElements();
  float maxHumidity = 0.;
  for(int j=0;j<nbRows;j++) {
	Entity* pRow_j = pVegetal->GetElement(j);
	int modality = pRow_j->GetIntConstValue(SOIL_MANAGEMENT_MODALITY);
	if((j == 0)// inter-rang avec fonction de passage
	   ||
	   (j == 1)) {      // inter-rang toujours nu (labour ou chimie)
	  if(modality == TILLAGE) {
		float humidity_j = pRow_j->ExecFloatMethod(SUM_UP_SOIL_HUMIDITY_VECTOR);
		if(humidity_j < -D_float_EPS) { // pas de couplage avec Mhydas/OpenFluid
		  maxHumidity = (float)-9.0;
		  break;
		}
		if(humidity_j > maxHumidity)
		  maxHumidity = humidity_j;
	  }
	} 
	else {
	  // sans objet
	}
  }
  //
  // test
  //
  float threshold;

  if(maxHumidity < -D_float_EPS) { // pas de couplage avec Mhydas/OpenFluid
	maxHumidity = pOt->GetFloatConstValue(LAST_SEVEN_DAYS_RAINFALL);
	threshold = gRealParam_workabilityRainMaxThreshold_tillageOperation_;
  }
  else {
	//
	// captage de la periode de l'annee
	//
	Entity* pVineyard = pPlot->GetSuperSet();
	Entity* pPlan = pVineyard->GetEntityConstValue(CONTROL_PLAN_ATTRIBUTE);
	Entity* pARB = pPlan->GetSuperEntity();
	Entity* pMultiPool = pARB->GetComponent(RESOURCE_POOL_CONJUNCTION);
	Entity* pWorkerPool = pMultiPool->GetElement(0); // par convention
	Entity* pAnyWorker = pWorkerPool->GetElement(0);
	Entity* pEmploymentPlan = 
	  pAnyWorker->GetEntityConstValue(ANNUAL_EMPLOYMENT_PLAN_ATTRIBUTE);
	int periodIndex = pEmploymentPlan->GetIntVarValue(CURRENT_ELEMENT_INDEX);
//printf("\n\tperiodIndex = %d", periodIndex);
	bool isPeakPeriod = FALSE;
	if((periodIndex == 4)
	   ||
	   (periodIndex == 5))
	  isPeakPeriod = TRUE;
	
	if(! isPeakPeriod) {
	  //
	  // prevision de pluie sur 3 jours
	  //
	  Entity* pWS = pPlot->GetEntityConstValue(WEATHER_STATION_ATTRIBUTE);
	  char* pathName = pWS->GetStringConstValue(PATH_NAME_ATTRIBUTE);
	  float cumulPluie_JP3 = 
		SumUpPredictionFloatFieldValue(pathName, FLD_RAINFALL, 1, 71)
		+ pWS->GetFloatVarValue(RAINFALL);
//printf("\n\tcumulPluie_JP3 = %4.1f", cumulPluie_JP3);
	  if(cumulPluie_JP3 > (float)10.)
		threshold = gRealParam_workabilityHumidityMaxThresholdIfRain_nonPeakPeriod_;
	  else
		threshold = gRealParam_workabilityHumidityMaxThreshold_nonPeakPeriod_;
	}
	else {
	  threshold = gRealParam_workabilityHumidityMaxThreshold_peakPeriod_;
	}
  }
//printf("\n\t-->maxHumidity = %f\t--> threshold = %3.2f", maxHumidity, threshold);
  if(maxHumidity > threshold)
	result = FALSE;

  if(! result) 
	if(gMyTraceWarnings)
	  printf("\n'%s'::FC returns FALSE / humidite du sol", pOt->InstanceName());
//printf("\n'%s'::FC returns %d", pOt->InstanceName(), result);
  return result;
};

TillageOperation::TillageOperation() {
// =========================================================
// Une classe de Entity : 'TillageOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(TILLAGE_OPERATION);
  //SetParentClassId(MOTORIZED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  Descriptor* pD;
  pD = new LastSevenDaysRainfall();       AddConstantDescriptor(pD);
  AssignBody(STATE_TRANSITION_PROCEDURE, tillageOperation_stateTransitionProcedure_Body);
  AssignBody(FEASIBILITY_CONDITION, tillageOperation_feasibilityCondition_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_tillageOperation_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);
};

bool trellisingOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName()); 

  return result;
};

TrellisingOperation::TrellisingOperation() {
// =========================================================
// Une classe de Entity : 'TrellisingOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(TRELLISING_OPERATION);
  //SetParentClassId(MANUAL_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, trellisingOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***
  
//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  int nominalSpeed = gIntParam_trellisingOperation_speed_;
  SetIntVarValue(NOMINAL_INT_QUANTITY_SPEED, nominalSpeed);

  //SetIntConstValue(PRIORITY_DEGREE, 1);   // + JPR 120907
  SetIntConstValue(PRIORITY_DEGREE, 60);   // modif JPR 120919
  // alors que la priorite generale est mise a 99 (MyFloatOperation)
};

bool verticalRailOperatedSpraying_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

VerticalRailOperatedSpraying::VerticalRailOperatedSpraying() {
// =========================================================
// Une classe de Entity : 'VerticalRailOperatedSpraying'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(VERTICAL_RAIL_OPERATED_SPRAYING);
  //SetParentClassId(MOTORIZED_SPRAYING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, verticalRailOperatedSpraying_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_verticalRailOperatedSpraying_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new VerticalRailSprayingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool airAssistedSpraying_stateTransitionProcedure_Body(EntityMethod* pM) {
//------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

AirAssistedSpraying::AirAssistedSpraying() {
// =========================================================
// Une classe de Entity : 'AirAssistedSpraying'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(AIR_ASSISTED_SPRAYING);
  //SetParentClassId(MOTORIZED_SPRAYING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, airAssistedSpraying_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_airAssistedSpraying_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new AirAssistedSprayingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool backSprayingOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//--------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//--------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

bool backSprayingOperation_feasibilityCondition_Body(EntityMethod* pM) {
//----------------------------------------------------------------------
// Methode invoquee alors que l'activite est ouverte.
// Doit renvoyer vrai si l'operation est maintenant faisable.
//----------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
//printf("\nentree '%s'::FC", pOt->InstanceName());
  bool result = myFloatAreaOperation_feasibilityCondition_Body(pM);
  if(result == FALSE) {
	//printf("\n'%s'::FC returns %d / non trafficalility", pOt->InstanceName(), result);
	// cette FC : + JPR 130220
	// quand la FC est non  satissfaite, et que le chantier concerné est
	// un trt EXTRA, on déclare que cette parcelle n'est plus à traiter
	//
	Entity* pInst = pOt->GetEntityVarValue(INSTRUCTION_CONTAINER);
	Entity* pChantier = NULL;
	int typeChantier = -9;
	pChantier = 
	  (Entity*)pInst->GetFirstContainer(CONCURRENT_DISTRIBUTED_ACTIVITY_SET,
										FALSE);
	if(pChantier == NULL) {
	  pChantier = 
		(Entity*)pInst->GetFirstContainer(SERIAL_DISTRIBUTED_ACTIVITY_SET,
										  FALSE);
	}
	typeChantier = App_GetDistributedActivitySetType(pChantier);
	if(typeChantier == EXTRA_PHYTO) {
	  Entity* pOOspec = pInst->GetEntityVarValue(OPERATED_OBJECT_SPEC_ATTRIBUTE);
	  pEntityTab* ooList = pOOspec->GetEntityTabVarValue(PRE_EXPANDED_LIST);
	  Entity* pPlot = (Entity*)ooList->get_element(0);
	  pPlot->SetFloatVarValue(PERCENT_EXTRA_PHYTO_TREATED, (float)-1.);
	  pOt->SetFloatVarValue(ACHIEVEMENT_DEGREE, (float)1., FALSE);
	  //  + JPR 130223 : si un trt 'Extra' a echoue par non-faisabilite,
	  // on conserve la possibilite d'en faire 2 (comme stipule dans le
	  // constructeur de ExtraPhytoTreatmentsIteration
	  Entity* pIterate = (Entity*)pChantier->GetFirstContainer(ACTIVITY_ITERATION, FALSE);
	  pIterate->SetIntVarValue(MAX_NUMBER_OF_REPLICATIONS, 3, FALSE);
	}
  }
  return result;
};

BackSprayingOperation::BackSprayingOperation() {
// =========================================================
// Une classe de Entity : 'BackSprayingOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(BACK_SPRAYING_OPERATION);
  //SetParentClassId(MANUAL_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, backSprayingOperation_stateTransitionProcedure_Body);
  AssignBody(FEASIBILITY_CONDITION, backSprayingOperation_feasibilityCondition_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  int nominalSpeed = gIntParam_backSprayingOperation_speed_;
  SetIntVarValue(NOMINAL_INT_QUANTITY_SPEED, nominalSpeed);

  OperationResourceSpecification* pMBSpec = new OperationResourceSpecification();
  pMBSpec->SetIntVarValue(ENTITY_SPEC_REFERENCE_CLASS_ID, 
			  BACK_SPRAYER);
  pMBSpec->SetIntConstValue(SELECTOR_FCT_ID, ANYONE);
  pMBSpec->SetIntConstValue(SELECTOR_FCT_ARG, 1);
  
  AddEntityValueElement(REQUIRED_RESOURCES, pMBSpec);

};

bool chemicalDebrancherOperatedSpraying_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

ChemicalDebrancherOperatedSpraying::ChemicalDebrancherOperatedSpraying() {
// =========================================================
// Une classe de Entity : 'ChemicalDebrancherOperatedSpraying'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(CHEMICAL_DEBRANCHER_OPERATED_SPRAYING);
  //SetParentClassId(MOTORIZED_SPRAYING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, chemicalDebrancherOperatedSpraying_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_chemicalDebrancherOperatedSpraying_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new ChemicalDebrancherSprayingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool clippingOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//----------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//----------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

ClippingOperation::ClippingOperation() {
// =========================================================
// Une classe de Entity : 'ClippingOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(CLIPPING_OPERATION);
  //SetParentClassId(MOTORIZED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, clippingOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_clippingOperation_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new ClippingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool fertilizationOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;
  
//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName()); 

  return result;
};

FertilizationOperation::FertilizationOperation() {
// =========================================================
// Une classe de Entity : 'FertilizationOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(FERTILIZATION_OPERATION);
  //SetParentClassId(MOTORIZED_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, fertilizationOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_fertilizationOperation_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new FertilizationOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool grinderOperatedShootThinning_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName()); 

  return result;
};

GrinderOperatedShootThinning::GrinderOperatedShootThinning() {
// =========================================================
// Une classe de Entity : 'GrinderOperatedShootThinning'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(GRINDER_OPERATED_SHOOT_THINNING);
  //SetParentClassId(SHOOT_THINNING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, grinderOperatedShootThinning_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());


  float nominalSpeed = gRealParam_grinderOperatedShootThinning_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new GrinderShootThinningOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool handHeldGunOperatedSpraying_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

HandHeldGunOperatedSpraying::HandHeldGunOperatedSpraying() {
// =========================================================
// Une classe de Entity : 'HandHeldGunOperatedSpraying'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(HAND_HELD_GUN_OPERATED_SPRAYING);
  //SetParentClassId(MOTORIZED_SPRAYING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, handHeldGunOperatedSpraying_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_handHeldGunOperatedSpraying_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new HandHeldGunSprayingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool horizontalRailOperatedSpraying_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

HorizontalRailOperatedSpraying::HorizontalRailOperatedSpraying() {
// =========================================================
// Une classe de Entity : 'HorizontalRailOperatedSpraying'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(HORIZONTAL_RAIL_OPERATED_SPRAYING);
  //SetParentClassId(MOTORIZED_SPRAYING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, horizontalRailOperatedSpraying_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_horizontalRailOperatedSpraying_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new HorizontalRailSprayingOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

bool manualDebranchingOperation_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName());

  return result;
};

ManualDebranchingOperation::ManualDebranchingOperation() {
// =========================================================
// Une classe de Entity : 'ManualDebranchingOperation'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(MANUAL_DEBRANCHING_OPERATION);
  //SetParentClassId(MANUAL_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, manualDebranchingOperation_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  int nominalSpeed = gIntParam_manualDebranchingOperation_speed_;
  SetIntVarValue(NOMINAL_INT_QUANTITY_SPEED, nominalSpeed);
};

bool rotaryOperatedShootThinning_stateTransitionProcedure_Body(EntityMethod* pM) {
//---------------------------------------------------------------------------
// D�finit l'impact de l'op�ration sur l'entit� op�r�e.
// L'entit� op�r�e est la valeur de l'argument OPERATION_TARGET.
// S'il s'agit d'une op�ration de transfert, consulter la documentation g�n�rale
// pour la liste et le sens de la valeur des arguments (source, destination, ...).
//---------------------------------------------------------------------------
  Operation* pOt = (Operation*)(pM->DescribedEntity());
  Entity* pE = pM->GetEntityArgValue(OPERATION_TARGET);
  bool result = TRUE;

//printf("\nt = %d passage par '%s'_STP", pCurrentSim->Clock(), pOt->InstanceName()); 

  return result;
};

RotaryOperatedShootThinning::RotaryOperatedShootThinning() {
// =========================================================
// Une classe de Entity : 'RotaryOperatedShootThinning'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(ROTARY_OPERATED_SHOOT_THINNING);
  //SetParentClassId(SHOOT_THINNING_OPERATION); // pour memoire (*)
  AddInstanceToClass();
  AssignBody(STATE_TRANSITION_PROCEDURE, rotaryOperatedShootThinning_stateTransitionProcedure_Body);
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  float nominalSpeed = gRealParam_rotaryOperatedShootThinning_speed_;
  SetFloatVarValue(NOMINAL_FLOAT_QUANTITY_SPEED, nominalSpeed);

  Entity* pRscSpec = new RotaryTillageOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

RotaryOperatedTillage::RotaryOperatedTillage() {
// =========================================================
// Une classe de Entity : 'RotaryOperatedTillage'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(ROTARY_OPERATED_TILLAGE);
  //SetParentClassId(TILLAGE_OPERATION); // pour memoire (*)
  AddInstanceToClass();
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  Entity* pRscSpec = new RotaryTillageOperationResourceSpec();
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

SpringTineOperatedTillage::SpringTineOperatedTillage() {
// =========================================================
// Une classe de Entity : 'SpringTineOperatedTillage'
// Ne pas modifier les lignes au-dessus du code spécifique.
// (*) la parente est declaree dans App_InstallMetaClasses()
// =========================================================
  SetMetaClass(SPRING_TINE_OPERATED_TILLAGE);
  //SetParentClassId(TILLAGE_OPERATION); // pour memoire (*)
  AddInstanceToClass();
// *** DEBUT DU CODE SPECIFIQUE. NE RIEN MODIFIER CI-DESSUS ***

  Entity* pRscSpec = new SpringTineTillageOperationResourceSpec(); // + 120516 4.6
  AddEntityValueElement(REQUIRED_RESOURCES, pRscSpec);
};

