int main_Auto (int argc, char* argv[])
//---------------------------------------------
// Description  : version de la fonction main pour les simulations autonomes
//                (cad non en position de serveur du client OpenFluid)
// Precondition : 
// Retour       : 
//---------------------------------------------
{
#ifdef _SWIG_
  xs_init = O_App_xs_init;
#endif
  // ci-dessous : ssi non programme dans *.dir
  InitChrono();
  StartChrono();

  // ---------------------------------------------
  // Plus grande tol�rance pour l'�galit� � 1 du degre de progression
  // des operations : -> 0.999 considere egal a 1
  // --------------------------------------------- 
  gFloatingPointEqualityTolerance = (float)0.001;

  // ---------------------------------------------
  // interpretation de la ligne de commande
  // ---------------------------------------------
  App_ParseMainArguments(argc, argv);

  // ---------------------------------------------
  // Lecture des parametres generaux du systeme
  // ---------------------------------------------
  ParseParameterFile(argv[1]);

  // ---------------------------------------------
  // Instantiation de pCurrentSim et preambule technique
  // ---------------------------------------------
  InitSimulation();

  // ---------------------------------------------
  // Instanciation superficielle du systeme de production
  // ---------------------------------------------
  Entity* pPS = new VineProducingRegion();
	
  Entity* pDS = new VirtualRegionManager();
					  
  Entity* pStrategy = new Strategy();
  Entity* pARBSet = new ActivitiesResourcesBlockSet();
  pStrategy->AddComponent(pARBSet);
  pDS->AddComponent(pStrategy);
  pPS->AddComponent(pDS);
  Entity* pOS = new VirtualRegionOperatingSystem();
  pPS->AddComponent(pOS);
  Entity* pCS = new VineGrowingTerritory();
  pPS->AddComponent(pCS);
  // ---------------------------------------------
  // Instanciation superficielle du systeme biophysique ...
  // ---------------------------------------------
  Entity* pBPS = new AgroClimaticSystem();
	
  Entity* pAgroSet = new VineyardSet();
  pBPS->AddComponent(pAgroSet);
  Entity* pClimSet = new WeatherStationSet();
  pBPS->AddComponent(pClimSet);
  // ... et attachement au ControlledSystem
  pCS->SetEntityConstValue(ATTACHED_ENTITY, pBPS);

  // ---------------------------------------------
  // Attachement des contraintes sur les ressources a l'OS
  // ---------------------------------------------
  Entity* pRSVC;
  pRSVC= new EquipmentMultiLocalization();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  pRSVC = new FarmWorkerMultiLocalization();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  pRSVC = new FarmWorkerMultiOccupation();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pRSVC);
  
  Entity* pARIC = new PlotWithSeveralOperations();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pARIC);
  
  Entity* pAIC;
  pAIC = new TooLateInWorkDayForActivityBeginning();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pAIC);
  
  pAIC = new OnlyPhytoTreatmentsAtEarlyMorning();
  pOS->AddEntityValueElement(RESOURCE_CONSTRAINT_LIST, pAIC);

  // ---------------------------------------------
  // Designation du systeme de production comme entite simulee
  // ---------------------------------------------
  pCurrentSim->SimulatedEntity(pPS);

  // ---------------------------------------------
  // exploitation des parametres de memoire et de prediction
  // ---------------------------------------------
  int memoryScope = gIntParam_virtualManager_memoryScope_;
  pDS->SetIntVarValue(MEMORY_SCOPE, memoryScope);
  int predictionScope = gIntParam_virtualManager_predictionScope_;
  pDS->SetIntVarValue(PREDICTION_SCOPE, predictionScope);
	
  // ---------------------------------------------
  // Lecture des parametres de la simulation
  // ---------------------------------------------
  ParseSimulationFile(argv[2]);
	
  // ---------------------------------------------
  // Lecture des autres fichiers-argument de la commande
  // ---------------------------------------------
  ParseOutputSpecFile(argv[3]);
  ParseStructureFile(argv[4]);

  // ---------------------------------------------
  // Construction profonde du systeme biophysique
  // ---------------------------------------------
  char* inDir = pCurrentSim->GetInDir();
  char* vineyardFile = gStringParam_install_vineyardFile_;
  char str[100] = {'\0'};
  strcat(str, inDir);
  strcat(str, vineyardFile);
  sprintf(str, "%s%s", inDir, vineyardFile);
  App_DepthBioPhysicalSettings(str);

  // ---------------------------------------------
  // Construction profonde du systeme operant
  // ---------------------------------------------
  char* rscFile = gStringParam_install_resourceFile_;
  char str2[100] = {'\0'};
  strcat(str2, inDir);
  strcat(str2, rscFile);
  sprintf(str2, "%s%s", inDir, rscFile);
  App_ResourceSettings(str2);

  // ---------------------------------------------
  // Construction profonde du systeme de decision
  // ---------------------------------------------
  char* planFile = gStringParam_install_planFile_;
  char str3[100] = {'\0'};
  strcat(str3, inDir);
  strcat(str3, rscFile);
  sprintf(str3, "%s%s", inDir, planFile);
  App_ControlPlanSettings(str3);

  // ---------------------------------------------
  // Etablissement des IsoAgeingEntity attachees aux parcelles,
  // une fois connues les varietes par parcelle (App_DepthBioPhysical.)
  // et les varietes-guides par conduite (App_ControlPlan.)
  // ---------------------------------------------
  App_AssignIsoAgeingEntityToPlots(pAgroSet);

  // ---------------------------------------------
  // Etablissement des operations exclues des SelectPlotsToX
  // ---------------------------------------------
  App_AssignTurnedOffActivitiesOnPlots(pAgroSet);
	
  // ---------------------------------------------
  // Lecture des autres fichiers-argument de la commande
  // ---------------------------------------------
  int nbArb = pARBSet->GetNumberOfElements();
  for(int i=0;i<nbArb;i++) {
	Entity* pARB_i = pARBSet->GetElement(i);
	Activity* pPlan_i = (Activity*)pARB_i->GetComponent(ACTIVITY);
	printf("\nAVANT RUN pPlan[%d] : %s pour %s\n", i, pPlan_i->InstanceName(), pPlan_i->GetEntityConstValue(VINEYARD_ATTRIBUTE)->InstanceName());
	pPlan_i->DisplaySituation(); printf("\n");
  }
  //
  // La preparation des gantt charts R ci-apres necessite
  // de ne pas desallouer les ARB et les chantiers. 
  // Le fichier declaratif *.dir ne doit donc pas contenir
  // les directives de desallocation. Cette desallocation est
  // programmee ici apr�s la preparation des gantt charts.
  // 
  C_ParseDirectiveFile(argv[5]);
  //
  // Preparation R gantt charts
  // 
  App_MakeGanttChartCommandFiles();
  
  // chrono ci-dessous : ssi non programme dans *.dir
  PauseChrono();
  PrintChrono();
  printf("\nduree = %s", gTimeString);
  CloseChrono();
  //
  // Desallocation des activites et des operations ...
  // - non commencees (*) ou entamees en l'instant de fin de simulation,
  //   donc non desallouees par un MIOS pour celles qui sont dans un chantier
  // - tq un chantier se termine avec elles, donc non desallouees par le MIOS
  //
  DeallocatePrimitiveActivityTab();
  DeallocateOperationTab();
  //
  // Desallocation des entites creees par l'application (et non par le moteur)
  // en cours de simulation et connues pour ne pas etre desallouees avant.
  //
  App_DeallocateMiscellaneousDynamicEntities();
  //
  // Desallocation generale :
  // - donc a ne pas programmer dans *.dir
  // - appel aux fonctions specialisees (au lieu de C_DeleteAll)
  // pour pouvoir y inserer un MemoryReport sans argument
  // (cad non conditionne a l'option de trace -V)
  //
  DeleteEntitySpecs();
  C_DeleteEntityInstances();
  DeleteMonitors();
  DeleteParameters();
  C_DeleteFileInstances();
  DeleteOutputSpecs();
  delete pCurrentSim;
  O_MemoryReport();
  DeleteEntityClasses();

#ifdef _INTERACTIVE_
  printf("FIN MAIN");
#endif
  return 0;
}
